"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.defaultCharacterSet = exports.CharacterSets = void 0;
const dicom_character_set_1 = require("dicom-character-set");
const vr_1 = require("./vr");
class CharacterSets {
    static isVrAffectedBySpecificCharacterSet(vr) {
        return vr === vr_1.VR.LO || vr === vr_1.VR.LT || vr === vr_1.VR.PN || vr === vr_1.VR.SH || vr === vr_1.VR.ST || vr === vr_1.VR.UT;
    }
    static fromNames(names) {
        return new CharacterSets(names);
    }
    static fromBytes(specificCharacterSetBytes) {
        return !specificCharacterSetBytes || specificCharacterSetBytes.length === 0
            ? exports.defaultCharacterSet
            : new CharacterSets(specificCharacterSetBytes.toString().trim());
    }
    static encode(s) {
        return Buffer.from(s, 'utf8');
    }
    static defaultOnly() {
        return new CharacterSets('');
    }
    constructor(charsets) {
        this.charsets = charsets;
    }
    decode(bytes, vr) {
        try {
            return (0, dicom_character_set_1.convertBytes)(this.charsets, bytes, { vr: vr.name });
        }
        catch (err) {
            console.warn('Cannot decode using character set: ' + this.charsets + ', using default instead: ' + err);
            return exports.defaultCharacterSet.decode(bytes, vr);
        }
    }
    toString() {
        return 'CharacterSets [' + this.charsets.split('\\').join(',') + ']';
    }
}
exports.CharacterSets = CharacterSets;
exports.defaultCharacterSet = CharacterSets.defaultOnly();
//# sourceMappingURL=character-sets.js.map