"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.collectFromTagPathsFlow = exports.collectFlow = void 0;
const base_1 = require("./base");
const dicom_flow_1 = require("./dicom-flow");
const dicom_elements_1 = require("./dicom-elements");
const dicom_parts_1 = require("./dicom-parts");
const value_1 = require("./value");
const elements_builder_1 = require("./elements-builder");
function collectFlow(tagCondition, stopCondition, label, maxBufferSize = 1000000) {
    return (0, dicom_flow_1.createFlow)(new (class extends (0, dicom_flow_1.EndEvent)((0, dicom_flow_1.TagPathTracking)((0, dicom_flow_1.GuaranteedDelimitationEvents)((0, dicom_flow_1.GuaranteedValueEvent)((0, dicom_flow_1.InFragments)(dicom_flow_1.DeferToPartFlow))))) {
        constructor() {
            super(...arguments);
            this.buffer = [];
            this.currentBufferSize = 0;
            this.hasEmitted = false;
            this.bytes = base_1.emptyBuffer;
            this.currentValue = undefined;
            this.currentFragment = undefined;
            this.builder = new elements_builder_1.ElementsBuilder();
        }
        elementsAndBuffer() {
            const parts = (0, base_1.prependToArray)(new dicom_parts_1.ElementsPart(label, this.builder.build()), this.buffer);
            this.hasEmitted = true;
            this.buffer = [];
            this.currentBufferSize = 0;
            return parts;
        }
        maybeAdd(element) {
            return tagCondition(this.tagPath)
                ? this.builder.addElement(element)
                : this.builder.noteElement(element);
        }
        onEnd() {
            return this.hasEmitted ? [] : this.elementsAndBuffer();
        }
        onPart(part) {
            if (this.hasEmitted) {
                return [part];
            }
            else {
                if (maxBufferSize > 0 && this.currentBufferSize > maxBufferSize) {
                    throw Error('Error collecting elements: max buffer size exceeded');
                }
                if (!(part instanceof dicom_flow_1.ValueChunkMarker) &&
                    !(part instanceof dicom_flow_1.SequenceDelimitationPartMarker) &&
                    !(part instanceof dicom_flow_1.ItemDelimitationPartMarker)) {
                    this.buffer.push(part);
                    this.currentBufferSize += part.bytes.length;
                }
                if ('tag' in part && stopCondition(this.tagPath)) {
                    return this.elementsAndBuffer();
                }
                if (part instanceof dicom_parts_1.HeaderPart) {
                    this.currentValue = new dicom_elements_1.ValueElement(part.tag, part.vr, value_1.Value.empty(), part.bigEndian, part.explicitVR);
                    this.bytes = base_1.emptyBuffer;
                    return [];
                }
                if (part instanceof dicom_parts_1.ItemPart && this.inFragments) {
                    this.currentFragment = new dicom_elements_1.FragmentElement(part.length, value_1.Value.empty(), part.bigEndian);
                    this.bytes = base_1.emptyBuffer;
                    return [];
                }
                if (part instanceof dicom_parts_1.ValueChunk) {
                    this.bytes = (0, base_1.concat)(this.bytes, part.bytes);
                    if (part.last) {
                        if (this.inFragments && this.currentFragment) {
                            this.maybeAdd(new dicom_elements_1.FragmentElement(this.currentFragment.length, new value_1.Value(this.bytes), this.currentFragment.bigEndian));
                        }
                        else if (this.currentValue) {
                            this.maybeAdd(new dicom_elements_1.ValueElement(this.currentValue.tag, this.currentValue.vr, new value_1.Value(this.bytes), this.currentValue.bigEndian, this.currentValue.explicitVR));
                        }
                        this.currentFragment = undefined;
                        this.currentValue = undefined;
                    }
                    return [];
                }
                if (part instanceof dicom_parts_1.SequencePart) {
                    this.maybeAdd(new dicom_elements_1.SequenceElement(part.tag, part.length, part.bigEndian, part.explicitVR));
                    return [];
                }
                if (part instanceof dicom_parts_1.FragmentsPart) {
                    this.maybeAdd(new dicom_elements_1.FragmentsElement(part.tag, part.vr, part.bigEndian, part.explicitVR));
                    return [];
                }
                if (part instanceof dicom_parts_1.ItemPart) {
                    this.maybeAdd(new dicom_elements_1.ItemElement(part.length, part.bigEndian));
                    return [];
                }
                if (part instanceof dicom_flow_1.ItemDelimitationPartMarker) {
                    return [];
                }
                if (part instanceof dicom_parts_1.ItemDelimitationPart) {
                    this.maybeAdd(new dicom_elements_1.ItemDelimitationElement(part.bigEndian));
                    return [];
                }
                if (part instanceof dicom_flow_1.SequenceDelimitationPartMarker) {
                    return [];
                }
                if (part instanceof dicom_parts_1.SequenceDelimitationPart) {
                    this.maybeAdd(new dicom_elements_1.SequenceDelimitationElement(part.bigEndian));
                    return [];
                }
                return [];
            }
        }
    })());
}
exports.collectFlow = collectFlow;
function collectFromTagPathsFlow(allowlist, label, maxBufferSize) {
    const maxTag = allowlist.length > 0 ? Math.max(...allowlist.map((t) => t.head().tag())) : 0;
    const tagCondition = (currentPath) => allowlist.find((t) => t.hasTrunk(currentPath) || t.isTrunkOf(currentPath)) !== undefined;
    const stopCondition = (tagPath) => allowlist.length === 0 || (tagPath.isRoot() && tagPath.tag() > maxTag);
    return collectFlow(tagCondition, stopCondition, label, maxBufferSize);
}
exports.collectFromTagPathsFlow = collectFromTagPathsFlow;
//# sourceMappingURL=collect-flow.js.map