"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Detour = void 0;
const stream_1 = require("stream");
class Detour extends stream_1.Transform {
    constructor(options, detourFlow) {
        super(options);
        this.options = options;
        this.detourFlow = detourFlow;
        this.detour = false;
        if (detourFlow) {
            this.setDetourFlow(detourFlow);
        }
    }
    setDetourFlow(detourFlow) {
        this.detourFlow = detourFlow;
    }
    setDetour(detour = true, initialChunk) {
        this.detour = detour;
        if (this.detourFlow !== undefined) {
            if (this.detour) {
                this.detourFlow.on('data', (chunk) => this.process(chunk));
                this.detourFlow.once('end', () => this.cleanup());
                this.detourFlow.once('error', (error) => this.emit('error', error));
            }
            else {
                this.detourFlow.end();
            }
        }
        if (initialChunk !== undefined && (initialChunk.length === undefined || initialChunk.length > 0)) {
            if (detour && this.detourFlow !== undefined) {
                this.detourFlow.write(initialChunk);
            }
            else {
                this.write(initialChunk);
            }
        }
    }
    cleanup() {
        // override to add custom cleanup code
    }
    _transform(chunk, encoding, callback) {
        if (this.detour !== undefined && this.detourFlow !== undefined) {
            if (!this.detourFlow.write(chunk)) {
                this.detourFlow.once('drain', callback);
            }
            else {
                process.nextTick(() => callback());
            }
        }
        else {
            this.process(chunk);
            callback();
        }
    }
    _flush(callback) {
        if (this.detour && this.detourFlow) {
            this.detourFlow.once('end', callback);
            this.detourFlow.end();
        }
        else {
            this.cleanup();
            process.nextTick(() => callback());
        }
    }
}
exports.Detour = Detour;
//# sourceMappingURL=detour.js.map