/// <reference types="node" />
import { DicomPart } from './dicom-parts';
import { Value } from './value';
import { VR } from './vr';
import { Elements } from './elements';
export declare class Element {
    readonly bigEndian: boolean;
    constructor(bigEndian?: boolean);
    toBytes(): Buffer;
    toParts(): DicomPart[];
}
export declare class ElementSet {
    readonly tag: number;
    readonly vr: VR;
    readonly bigEndian: boolean;
    readonly explicitVR: boolean;
    constructor(tag: number, vr: VR, bigEndian?: boolean, explicitVR?: boolean);
    toBytes(): Buffer;
    toElements(): Element[];
}
export declare class UnknownElement extends Element {
    constructor(bigEndian?: boolean);
}
declare class PreambleElement extends Element {
    constructor();
    toBytes(): Buffer;
    toString(): string;
    toParts(): DicomPart[];
}
export declare const preambleElement: PreambleElement;
export declare class ValueElement extends ElementSet {
    readonly value: Value;
    length: number;
    constructor(tag: number, vr: VR, value: Value, bigEndian?: boolean, explicitVR?: boolean);
    setValue(value: Value): ValueElement;
    toBytes(): Buffer;
    toParts(): DicomPart[];
    toElements(): Element[];
    toString(): string;
}
export declare class SequenceElement extends Element {
    readonly tag: number;
    readonly length: number;
    readonly explicitVR: boolean;
    indeterminate: boolean;
    constructor(tag: number, length?: number, bigEndian?: boolean, explicitVR?: boolean);
    toBytes(): Buffer;
    toParts(): DicomPart[];
    toString(): string;
}
export declare class FragmentsElement extends Element {
    readonly tag: number;
    readonly vr: VR;
    readonly explicitVR: boolean;
    constructor(tag: number, vr: VR, bigEndian?: boolean, explicitVR?: boolean);
    toBytes(): Buffer;
    toParts(): DicomPart[];
    toString(): string;
}
export declare class ItemElement extends Element {
    readonly length: number;
    indeterminate: boolean;
    constructor(length?: number, bigEndian?: boolean);
    toBytes(): Buffer;
    toParts(): DicomPart[];
    toString(): string;
}
export declare class FragmentElement extends Element {
    readonly length: number;
    readonly value: Value;
    constructor(length: number, value: Value, bigEndian?: boolean);
    toBytes(): Buffer;
    toParts(): DicomPart[];
    toString(): string;
}
export declare class ItemDelimitationElement extends Element {
    constructor(bigEndian?: boolean);
    toBytes(): Buffer;
    toParts(): DicomPart[];
    toString(): string;
}
export declare class SequenceDelimitationElement extends Element {
    constructor(bigEndian?: boolean);
    toBytes(): Buffer;
    toParts(): DicomPart[];
    toString(): string;
}
export declare class Sequence extends ElementSet {
    readonly tag: number;
    readonly length: number;
    readonly items: Item[];
    indeterminate: boolean;
    size: number;
    constructor(tag: number, length?: number, items?: Item[], bigEndian?: boolean, explicitVR?: boolean);
    item(index: number): Item;
    addItem(item: Item): Sequence;
    removeItem(index: number): Sequence;
    toBytes(): Buffer;
    toElements(): Element[];
    setItem(index: number, item: Item): Sequence;
    toString(): string;
}
export declare class Item {
    readonly elements: Elements;
    readonly length: number;
    readonly bigEndian: boolean;
    indeterminate: boolean;
    constructor(elements: Elements, length?: number, bigEndian?: boolean);
    toElements(): Element[];
    toBytes(): Buffer;
    setElements(elements: Elements): Item;
    toString(): string;
}
export declare class Fragment {
    readonly length: number;
    readonly value: Value;
    readonly bigEndian: boolean;
    constructor(length: number, value: Value, bigEndian?: boolean);
    toElement(): Element;
    toString(): string;
}
export declare class Fragments extends ElementSet {
    readonly tag: number;
    readonly vr: VR;
    readonly offsets: number[];
    readonly fragments: Fragment[];
    size: number;
    constructor(tag: number, vr: VR, offsets: number[], fragments?: Fragment[], bigEndian?: boolean, explicitVR?: boolean);
    fragment(index: number): Fragment;
    frameCount(): number;
    addFragment(fragment: Fragment): Fragments;
    toBytes(): Buffer;
    toElements(): Element[];
    setFragment(index: number, fragment: Fragment): Fragments;
    toString(): string;
}
export {};
