/// <reference types="node" />
import { Transform } from 'stream';
import { DeflatedChunk, DicomPart, FragmentsPart, HeaderPart, ItemDelimitationPart, ItemPart, MetaPart, PreamblePart, SequenceDelimitationPart, SequencePart, UnknownPart, ValueChunk } from './dicom-parts';
export declare function createFlow(flow: any): any;
export declare abstract class DicomFlow {
    abstract onPreamble(part: PreamblePart): any[];
    abstract onHeader(part: HeaderPart): any[];
    abstract onValueChunk(part: ValueChunk): any[];
    abstract onSequence(part: SequencePart): any[];
    abstract onSequenceDelimitation(part: SequenceDelimitationPart): any[];
    abstract onFragments(part: FragmentsPart): any[];
    abstract onItem(part: ItemPart): any[];
    abstract onItemDelimitation(part: ItemDelimitationPart): any[];
    abstract onDeflatedChunk(part: DeflatedChunk): any[];
    abstract onUnknown(part: UnknownPart): any[];
    abstract onPart(part: DicomPart): any[];
    baseFlow(): Transform;
    handlePart(part: DicomPart): any[];
}
/**
 * Depends on DicomFlow
 */
export declare class IdentityFlow extends DicomFlow {
    onPreamble(part: PreamblePart): DicomPart[];
    onHeader(part: HeaderPart): DicomPart[];
    onValueChunk(part: ValueChunk): DicomPart[];
    onSequence(part: SequencePart): DicomPart[];
    onSequenceDelimitation(part: SequenceDelimitationPart): DicomPart[];
    onFragments(part: FragmentsPart): DicomPart[];
    onItem(part: ItemPart): DicomPart[];
    onItemDelimitation(part: ItemDelimitationPart): DicomPart[];
    onDeflatedChunk(part: DeflatedChunk): DicomPart[];
    onUnknown(part: UnknownPart): DicomPart[];
    onPart(part: DicomPart): DicomPart[];
}
/**
 * Depends on DicomFlow
 */
export declare abstract class DeferToPartFlow extends DicomFlow {
    onPreamble(part: PreamblePart): DicomPart[];
    onHeader(part: HeaderPart): DicomPart[];
    onValueChunk(part: ValueChunk): DicomPart[];
    onSequence(part: SequencePart): DicomPart[];
    onSequenceDelimitation(part: SequenceDelimitationPart): DicomPart[];
    onFragments(part: FragmentsPart): DicomPart[];
    onDeflatedChunk(part: DeflatedChunk): DicomPart[];
    onUnknown(part: UnknownPart): DicomPart[];
    onItem(part: ItemPart): DicomPart[];
    onItemDelimitation(part: ItemDelimitationPart): DicomPart[];
}
declare class DicomStartMarker extends MetaPart {
    toString(): string;
}
export declare const dicomStartMarker: DicomStartMarker;
export declare const StartEvent: (Super: any) => any;
declare class DicomEndMarker extends MetaPart {
    toString(): string;
}
export declare const dicomEndMarker: DicomEndMarker;
export declare const EndEvent: (Super: any) => any;
export declare const InFragments: (Super: any) => any;
export declare class ValueChunkMarker extends ValueChunk {
    constructor(bigEndian: boolean);
    toString(): string;
}
export declare const GuaranteedValueEvent: (Super: any) => any;
export declare class SequenceDelimitationPartMarker extends SequenceDelimitationPart {
    constructor(bigEndian: boolean);
    toString(): string;
}
export declare class ItemDelimitationPartMarker extends ItemDelimitationPart {
    constructor(bigEndian: boolean);
    toString(): string;
}
/**
 * Depends on InFragments
 */
export declare const GuaranteedDelimitationEvents: (Super: any) => any;
/**
 * Depends on GuaranteedDelimitationEvents
 */
export declare const InSequence: (Super: any) => any;
/**
 * Depends on GuaranteedValueEvent, GuaranteedDelimitationEvents, InFragments
 */
export declare const TagPathTracking: (Super: any) => any;
/**
 * Depends on InFragments
 */
export declare const GroupLengthWarnings: (Super: any) => any;
export {};
