"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GroupLengthWarnings = exports.TagPathTracking = exports.InSequence = exports.GuaranteedDelimitationEvents = exports.ItemDelimitationPartMarker = exports.SequenceDelimitationPartMarker = exports.GuaranteedValueEvent = exports.ValueChunkMarker = exports.InFragments = exports.EndEvent = exports.dicomEndMarker = exports.StartEvent = exports.dicomStartMarker = exports.DeferToPartFlow = exports.IdentityFlow = exports.DicomFlow = exports.createFlow = void 0;
const base_1 = require("./base");
const flows_1 = require("./flows");
const dicom_parts_1 = require("./dicom-parts");
const tag_1 = require("./tag");
const tag_path_1 = require("./tag-path");
function createFlow(flow) {
    return (0, base_1.pipe)(flow.baseFlow(), (0, flows_1.flatMapFlow)(flow.handlePart.bind(flow)));
}
exports.createFlow = createFlow;
class DicomFlow {
    baseFlow() {
        return (0, flows_1.identityFlow)(true);
    }
    handlePart(part) {
        if (part instanceof dicom_parts_1.PreamblePart) {
            return this.onPreamble(part);
        }
        if (part instanceof dicom_parts_1.HeaderPart) {
            return this.onHeader(part);
        }
        if (part instanceof dicom_parts_1.ValueChunk) {
            return this.onValueChunk(part);
        }
        if (part instanceof dicom_parts_1.SequencePart) {
            return this.onSequence(part);
        }
        if (part instanceof dicom_parts_1.SequenceDelimitationPart) {
            return this.onSequenceDelimitation(part);
        }
        if (part instanceof dicom_parts_1.FragmentsPart) {
            return this.onFragments(part);
        }
        if (part instanceof dicom_parts_1.ItemPart) {
            return this.onItem(part);
        }
        if (part instanceof dicom_parts_1.ItemDelimitationPart) {
            return this.onItemDelimitation(part);
        }
        if (part instanceof dicom_parts_1.DeflatedChunk) {
            return this.onDeflatedChunk(part);
        }
        if (part instanceof dicom_parts_1.UnknownPart) {
            return this.onUnknown(part);
        }
        return this.onPart(part);
    }
}
exports.DicomFlow = DicomFlow;
/**
 * Depends on DicomFlow
 */
class IdentityFlow extends DicomFlow {
    onPreamble(part) {
        return [part];
    }
    onHeader(part) {
        return [part];
    }
    onValueChunk(part) {
        return [part];
    }
    onSequence(part) {
        return [part];
    }
    onSequenceDelimitation(part) {
        return [part];
    }
    onFragments(part) {
        return [part];
    }
    onItem(part) {
        return [part];
    }
    onItemDelimitation(part) {
        return [part];
    }
    onDeflatedChunk(part) {
        return [part];
    }
    onUnknown(part) {
        return [part];
    }
    onPart(part) {
        return [part];
    }
}
exports.IdentityFlow = IdentityFlow;
/**
 * Depends on DicomFlow
 */
class DeferToPartFlow extends DicomFlow {
    onPreamble(part) {
        return this.onPart(part);
    }
    onHeader(part) {
        return this.onPart(part);
    }
    onValueChunk(part) {
        return this.onPart(part);
    }
    onSequence(part) {
        return this.onPart(part);
    }
    onSequenceDelimitation(part) {
        return this.onPart(part);
    }
    onFragments(part) {
        return this.onPart(part);
    }
    onDeflatedChunk(part) {
        return this.onPart(part);
    }
    onUnknown(part) {
        return this.onPart(part);
    }
    onItem(part) {
        return this.onPart(part);
    }
    onItemDelimitation(part) {
        return this.onPart(part);
    }
}
exports.DeferToPartFlow = DeferToPartFlow;
class DicomStartMarker extends dicom_parts_1.MetaPart {
    toString() {
        return 'Start Marker []';
    }
}
exports.dicomStartMarker = new DicomStartMarker();
const StartEvent = (Super) => class extends Super {
    onStart() {
        throw Error('Not implemented');
    }
    baseFlow() {
        return (0, base_1.pipe)((0, flows_1.prependFlow)(exports.dicomStartMarker, true), super.baseFlow());
    }
    handlePart(part) {
        return part === exports.dicomStartMarker ? this.onStart() : super.handlePart(part);
    }
};
exports.StartEvent = StartEvent;
class DicomEndMarker extends dicom_parts_1.MetaPart {
    toString() {
        return 'End Marker []';
    }
}
exports.dicomEndMarker = new DicomEndMarker();
const EndEvent = (Super) => class extends Super {
    onEnd() {
        throw Error('Not implemented');
    }
    baseFlow() {
        return (0, base_1.pipe)((0, flows_1.appendFlow)(exports.dicomEndMarker, true), super.baseFlow());
    }
    handlePart(part) {
        return part === exports.dicomEndMarker ? this.onEnd() : super.handlePart(part);
    }
};
exports.EndEvent = EndEvent;
const InFragments = (Super) => class extends Super {
    constructor() {
        super(...arguments);
        this.inFragments = false;
    }
    onFragments(part) {
        this.inFragments = true;
        return super.onFragments(part);
    }
    onSequenceDelimitation(part) {
        this.inFragments = false;
        return super.onSequenceDelimitation(part);
    }
};
exports.InFragments = InFragments;
class ValueChunkMarker extends dicom_parts_1.ValueChunk {
    constructor(bigEndian) {
        super(bigEndian, base_1.emptyBuffer, true);
    }
    toString() {
        return `Value Chunk Marker [bigEndian=${this.bigEndian}]`;
    }
}
exports.ValueChunkMarker = ValueChunkMarker;
const GuaranteedValueEvent = (Super) => class extends Super {
    onHeader(part) {
        return part.length === 0
            ? super.onHeader(part).concat(this.onValueChunk(new ValueChunkMarker(part.bigEndian)))
            : super.onHeader(part);
    }
    onItem(part) {
        return this.inFragments && part.length === 0
            ? super.onItem(part).concat(this.onValueChunk(new ValueChunkMarker(part.bigEndian)))
            : super.onItem(part);
    }
    onValueChunk(part) {
        return super.onValueChunk(part).filter((c) => !(c instanceof ValueChunkMarker));
    }
};
exports.GuaranteedValueEvent = GuaranteedValueEvent;
class SequenceDelimitationPartMarker extends dicom_parts_1.SequenceDelimitationPart {
    constructor(bigEndian) {
        super(bigEndian, base_1.emptyBuffer);
    }
    toString() {
        return `SequenceDelimitationMarker [bigEndian=${this.bigEndian}]`;
    }
}
exports.SequenceDelimitationPartMarker = SequenceDelimitationPartMarker;
class ItemDelimitationPartMarker extends dicom_parts_1.ItemDelimitationPart {
    constructor(bigEndian) {
        super(bigEndian, base_1.emptyBuffer);
    }
    toString() {
        return `ItemDelimitationMarker [bigEndian=${this.bigEndian}]`;
    }
}
exports.ItemDelimitationPartMarker = ItemDelimitationPartMarker;
/**
 * Depends on InFragments
 */
const GuaranteedDelimitationEvents = (Super) => class extends Super {
    constructor() {
        super(...arguments);
        this.partStack = [];
    }
    onSequence(part) {
        this.subtractLength(part);
        this.partStack.unshift({ part, bytesLeft: part.length });
        return super.onSequence(part).concat(this.maybeDelimit());
    }
    onItem(part) {
        this.subtractLength(part);
        if (!this.inFragments) {
            this.partStack.unshift({ part, bytesLeft: part.length });
        }
        return super.onItem(part).concat(this.maybeDelimit());
    }
    onSequenceDelimitation(part) {
        if (this.partStack.length > 0 && !(part instanceof SequenceDelimitationPartMarker) && !this.inFragments) {
            this.partStack.shift();
        }
        return this.subtractAndEmit(part, (p) => super.onSequenceDelimitation(p).filter((d) => !(d instanceof SequenceDelimitationPartMarker)));
    }
    onItemDelimitation(part) {
        if (this.partStack.length > 0 && !(part instanceof ItemDelimitationPartMarker)) {
            this.partStack.shift();
        }
        return this.subtractAndEmit(part, (p) => super.onItemDelimitation(p).filter((d) => !(d instanceof ItemDelimitationPartMarker)));
    }
    onHeader(part) {
        return this.subtractAndEmit(part, super.onHeader.bind(this));
    }
    onValueChunk(part) {
        return this.subtractAndEmit(part, super.onValueChunk.bind(this));
    }
    onFragments(part) {
        return this.subtractAndEmit(part, super.onFragments.bind(this));
    }
    subtractLength(part) {
        this.partStack.forEach((p) => {
            if (p.bytesLeft != base_1.indeterminateLength) {
                p.bytesLeft -= part.bytes.length;
            }
        });
    }
    maybeDelimit() {
        let splitIndex = 0;
        while (splitIndex < this.partStack.length &&
            this.partStack[splitIndex].bytesLeft != base_1.indeterminateLength &&
            this.partStack[splitIndex].bytesLeft <= 0) {
            splitIndex++;
        }
        const inactive = this.partStack.slice(0, splitIndex);
        this.partStack = this.partStack.slice(splitIndex);
        const out = inactive.map((i) => i.part instanceof dicom_parts_1.ItemPart
            ? this.onItemDelimitation(new ItemDelimitationPartMarker(i.part.bigEndian))
            : this.onSequenceDelimitation(new SequenceDelimitationPartMarker(i.part.bigEndian)));
        return [].concat(...out);
    }
    subtractAndEmit(part, handle) {
        this.subtractLength(part);
        return handle(part).concat(this.maybeDelimit());
    }
};
exports.GuaranteedDelimitationEvents = GuaranteedDelimitationEvents;
/**
 * Depends on GuaranteedDelimitationEvents
 */
const InSequence = (Super) => class extends Super {
    constructor() {
        super(...arguments);
        this.sequenceStack = [];
    }
    sequenceDepth() {
        return this.sequenceStack.length;
    }
    inSequence() {
        return this.sequenceStack.length > 0;
    }
    onSequence(part) {
        this.sequenceStack.unshift(part);
        return super.onSequence(part);
    }
    onSequenceDelimitation(part) {
        if (!this.inFragments) {
            this.sequenceStack.shift();
        }
        return super.onSequenceDelimitation(part);
    }
};
exports.InSequence = InSequence;
/**
 * Depends on GuaranteedValueEvent, GuaranteedDelimitationEvents, InFragments
 */
const TagPathTracking = (Super) => class extends Super {
    constructor() {
        super(...arguments);
        this.tagPath = tag_path_1.emptyTagPath;
    }
    onHeader(part) {
        const t = this.tagPath;
        this.tagPath = t instanceof tag_path_1.TagPathItem ? t.thenTag(part.tag) : t.previous().thenTag(part.tag);
        return super.onHeader(part);
    }
    onFragments(part) {
        const t = this.tagPath;
        this.tagPath = t instanceof tag_path_1.TagPathItem ? t.thenTag(part.tag) : t.previous().thenTag(part.tag);
        return super.onFragments(part);
    }
    onSequence(part) {
        const t = this.tagPath;
        this.tagPath = t instanceof tag_path_1.TagPathItem ? t.thenSequence(part.tag) : t.previous().thenSequence(part.tag);
        return super.onSequence(part);
    }
    onSequenceDelimitation(part) {
        const t = this.tagPath;
        if (!this.inFragments) {
            this.tagPath = t.previous().thenSequenceEnd(t.tag());
        }
        return super.onSequenceDelimitation(part);
    }
    onItem(part) {
        const t = this.tagPath;
        if (!this.inFragments) {
            if (t instanceof tag_path_1.TagPathItemEnd) {
                this.tagPath = t.previous().thenItem(t.tag(), t.item + 1);
            }
            else {
                this.tagPath = t.previous().thenItem(t.tag(), 1);
            }
        }
        return super.onItem(part);
    }
    onItemDelimitation(part) {
        const t = this.tagPath;
        if (t instanceof tag_path_1.TagPathItem) {
            this.tagPath = t.previous().thenItemEnd(t.tag(), t.item);
        }
        else {
            const ti = t.previous();
            if (ti instanceof tag_path_1.TagPathItem) {
                this.tagPath = ti.previous().thenItemEnd(ti.tag(), ti.item);
            }
        }
        return super.onItemDelimitation(part);
    }
};
exports.TagPathTracking = TagPathTracking;
/**
 * Depends on InFragments
 */
const GroupLengthWarnings = (Super) => class extends Super {
    constructor() {
        super(...arguments);
        this.silent = false;
    }
    setSilent(silent) {
        this.silent = silent;
    }
    onHeader(part) {
        if (!this.silent && (0, base_1.isGroupLength)(part.tag) && part.tag !== tag_1.Tag.FileMetaInformationGroupLength) {
            console.warn('Group length attribute detected, consider removing group lengths to maintain valid ' +
                'DICOM information');
        }
        return super.onHeader(part);
    }
    onSequence(part) {
        if (!this.silent && !part.indeterminate && part.length > 0) {
            console.warn('Determinate length sequence detected, consider re-encoding sequences to indeterminate ' +
                'length to maintain valid DICOM information');
        }
        return super.onSequence(part);
    }
    onItem(part) {
        if (!this.silent && !this.inFragments && !part.indeterminate && part.length > 0) {
            console.warn('Determinate length item detected, consider re-encoding items to indeterminate length to ' +
                'maintain valid DICOM information');
        }
        return super.onItem(part);
    }
};
exports.GroupLengthWarnings = GroupLengthWarnings;
//# sourceMappingURL=dicom-flow.js.map