/// <reference types="node" />
import { Elements } from './elements';
import { VR } from './vr';
export declare class DicomPart {
    readonly bigEndian: boolean;
    readonly bytes: Buffer;
    constructor(bigEndian: boolean, bytes: Buffer);
}
export declare class MetaPart extends DicomPart {
    constructor();
}
export declare class PreamblePart extends DicomPart {
    constructor(bytes: Buffer);
    toString(): string;
}
export declare class HeaderPart extends DicomPart {
    readonly tag: number;
    readonly vr: VR;
    readonly length: number;
    readonly isFmi: boolean;
    readonly bigEndian: boolean;
    readonly explicitVR: boolean;
    readonly bytes: Buffer;
    static create(tag: number, vr: VR, length: number, bigEndian?: boolean, explicitVR?: boolean): HeaderPart;
    constructor(tag: number, vr: VR, length: number, isFmi: boolean, bigEndian: boolean, explicitVR: boolean, bytes: Buffer);
    withUpdatedLength(newLength: number): HeaderPart;
    toString(): string;
}
export declare class ValueChunk extends DicomPart {
    readonly last: boolean;
    constructor(bigEndian: boolean, bytes: Buffer, last: boolean);
    toString(): string;
}
export declare class DeflatedChunk extends DicomPart {
    constructor(bigEndian: boolean, bytes: Buffer);
    toString(): string;
}
export declare class ItemPart extends DicomPart {
    readonly length: number;
    indeterminate: boolean;
    constructor(length: number, bigEndian: boolean, bytes: Buffer);
    toString(): string;
}
export declare class ItemDelimitationPart extends DicomPart {
    constructor(bigEndian: boolean, bytes: Buffer);
    toString(): string;
}
export declare class SequencePart extends DicomPart {
    readonly tag: number;
    readonly length: number;
    readonly explicitVR: boolean;
    indeterminate: boolean;
    constructor(tag: number, length: number, bigEndian: boolean, explicitVR: boolean, bytes: Buffer);
    toString(): string;
}
export declare class SequenceDelimitationPart extends DicomPart {
    constructor(bigEndian: boolean, bytes: Buffer);
    toString(): string;
}
export declare class FragmentsPart extends DicomPart {
    readonly tag: number;
    readonly length: number;
    readonly vr: VR;
    readonly explicitVR: boolean;
    constructor(tag: number, length: number, vr: VR, bigEndian: boolean, explicitVR: boolean, bytes: Buffer);
    toString(): string;
}
export declare class UnknownPart extends DicomPart {
    constructor(bigEndian: boolean, bytes: Buffer);
    toString(): string;
}
export declare class ElementsPart extends MetaPart {
    readonly label: string;
    readonly elements: Elements;
    constructor(label: string, elements: Elements);
}
