"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.elementFlow = void 0;
const base_1 = require("./base");
const dicom_flow_1 = require("./dicom-flow");
const dicom_elements_1 = require("./dicom-elements");
const dicom_parts_1 = require("./dicom-parts");
const value_1 = require("./value");
function elementFlow() {
    return (0, dicom_flow_1.createFlow)(new (class extends (0, dicom_flow_1.GuaranteedValueEvent)((0, dicom_flow_1.InFragments)(dicom_flow_1.DeferToPartFlow)) {
        constructor() {
            super(...arguments);
            this.bytes = base_1.emptyBuffer;
        }
        onPart(part) {
            if (part instanceof dicom_parts_1.PreamblePart) {
                return [dicom_elements_1.preambleElement];
            }
            if (part instanceof dicom_parts_1.HeaderPart) {
                this.currentValue = new dicom_elements_1.ValueElement(part.tag, part.vr, value_1.Value.empty(), part.bigEndian, part.explicitVR);
                this.bytes = base_1.emptyBuffer;
                return [];
            }
            if (part instanceof dicom_parts_1.ItemPart && this.inFragments) {
                this.currentFragment = new dicom_elements_1.FragmentElement(part.length, value_1.Value.empty(), part.bigEndian);
                this.bytes = base_1.emptyBuffer;
                return [];
            }
            if (part instanceof dicom_parts_1.ValueChunk) {
                this.bytes = (0, base_1.concat)(this.bytes, part.bytes);
                if (part.last) {
                    if (this.inFragments) {
                        if (this.currentFragment === undefined) {
                            return [];
                        }
                        else {
                            return [
                                new dicom_elements_1.FragmentElement(this.currentFragment.length, new value_1.Value(this.bytes), this.currentFragment.bigEndian),
                            ];
                        }
                    }
                    else {
                        return [
                            new dicom_elements_1.ValueElement(this.currentValue.tag, this.currentValue.vr, new value_1.Value(this.bytes), this.currentValue.bigEndian, this.currentValue.explicitVR),
                        ];
                    }
                }
                else {
                    return [];
                }
            }
            if (part instanceof dicom_parts_1.SequencePart) {
                return [new dicom_elements_1.SequenceElement(part.tag, part.length, part.bigEndian, part.explicitVR)];
            }
            if (part instanceof dicom_parts_1.FragmentsPart) {
                return [new dicom_elements_1.FragmentsElement(part.tag, part.vr, part.bigEndian, part.explicitVR)];
            }
            if (part instanceof dicom_parts_1.ItemPart) {
                return [new dicom_elements_1.ItemElement(part.length, part.bigEndian)];
            }
            if (part instanceof dicom_parts_1.ItemDelimitationPart) {
                return [new dicom_elements_1.ItemDelimitationElement(part.bigEndian)];
            }
            if (part instanceof dicom_parts_1.SequenceDelimitationPart) {
                return [new dicom_elements_1.SequenceDelimitationElement(part.bigEndian)];
            }
            return [];
        }
    })());
}
exports.elementFlow = elementFlow;
//# sourceMappingURL=element-flows.js.map