/// <reference types="node" />
import { LocalDate, LocalTime, ZonedDateTime, ZoneId, ZoneOffset } from 'js-joda';
import { Element, ElementSet, ValueElement, Sequence, Item, Fragments } from './dicom-elements';
import { CharacterSets } from './character-sets';
import { DicomPart } from './dicom-parts';
import { TagPath, TagPathItem, TagPathTag, TagPathSequence } from './tag-path';
import { Value } from './value';
import { VR } from './vr';
import { PersonName } from './person-name';
export declare function parseZoneOffset(s: string): ZoneOffset;
export declare class Elements {
    characterSets: CharacterSets;
    zoneOffset: ZoneId;
    data: ElementSet[];
    static empty(characterSets?: CharacterSets, zoneOffset?: ZoneId): Elements;
    size: number;
    constructor(characterSets?: CharacterSets, zoneOffset?: ZoneId, data?: ElementSet[]);
    elementByTag(tag: number): ElementSet | undefined;
    elementByPath(tagPath: TagPathTag): ElementSet | undefined;
    valueElementByTag(tag: number): ValueElement | undefined;
    valueElementByPath(tagPath: TagPathTag): ValueElement | undefined;
    valueByTag(tag: number): Value | undefined;
    valueByPath(tagPath: TagPathTag): Value | undefined;
    bytesByTag(tag: number): Buffer | undefined;
    bytesByPath(tagPath: TagPathTag): Buffer | undefined;
    stringsByTag(tag: number): string[];
    stringsByPath(tagPath: TagPathTag): string[];
    stringByTag(tag: number): string | undefined;
    stringByPath(tagPath: TagPathTag): string | undefined;
    singleStringByTag(tag: number): string | undefined;
    singleStringByPath(tagPath: TagPathTag): string | undefined;
    numbersByTag(tag: number): number[];
    numbersByPath(tagPath: TagPathTag): number[];
    numberByTag(tag: number): number | undefined;
    numberByPath(tagPath: TagPathTag): number | undefined;
    datesByTag(tag: number): LocalDate[];
    datesByPath(tagPath: TagPathTag): LocalDate[];
    dateByTag(tag: number): LocalDate | undefined;
    dateByPath(tagPath: TagPathTag): LocalDate | undefined;
    timesByTag(tag: number): LocalTime[];
    timesByPath(tagPath: TagPathTag): LocalTime[];
    timeByTag(tag: number): LocalTime | undefined;
    timeByPath(tagPath: TagPathTag): LocalTime | undefined;
    dateTimesByTag(tag: number): ZonedDateTime[];
    dateTimesByPath(tagPath: TagPathTag): ZonedDateTime[];
    dateTimeByTag(tag: number): ZonedDateTime | undefined;
    dateTimeByPath(tagPath: TagPathTag): ZonedDateTime | undefined;
    personNamesByTag(tag: number): PersonName[];
    personNamesByPath(tagPath: TagPathTag): PersonName[];
    personNameByTag(tag: number): PersonName | undefined;
    personNameByPath(tagPath: TagPathTag): PersonName | undefined;
    urlByTag(tag: number): URL | undefined;
    urlByPath(tagPath: TagPathTag): LocalTime | undefined;
    sequenceByTag(tag: number): Sequence | undefined;
    sequenceByPath(tagPath: TagPathTag): Sequence | undefined;
    itemByTag(tag: number, index: number): Item | undefined;
    nestedByTag(tag: number, item: number): Elements | undefined;
    nestedByPath(tagPath: TagPathItem): Elements | undefined;
    fragmentsByTag(tag: number): Fragments | undefined;
    setElementSet(elementSet: ElementSet): Elements;
    setElementSets(elementSets: ElementSet[]): Elements;
    setSequence(sequence: Sequence): Elements;
    private updateSequence;
    private updatePath;
    setNested(tagPath: TagPathItem, elements: Elements): Elements;
    setNestedElementSet(tagPath: TagPathItem, elementSet: ElementSet): Elements;
    setNestedSequence(tagPath: TagPathItem, sequence: Sequence): Elements;
    addItem(tagPath: TagPathSequence, elements: Elements): Elements;
    setCharacterSets(characterSets: CharacterSets): Elements;
    setZoneOffset(zoneOffset: ZoneId): Elements;
    setValue(tag: number, vr: VR, value: Value, bigEndian?: boolean, explicitVR?: boolean): Elements;
    setBytes(tag: number, vr: VR, value: Buffer, bigEndian?: boolean, explicitVR?: boolean): Elements;
    setStrings(tag: number, values: string[], vr?: VR, bigEndian?: boolean, explicitVR?: boolean): Elements;
    setString(tag: number, value: string, vr?: VR, bigEndian?: boolean, explicitVR?: boolean): Elements;
    setNumbers(tag: number, values: number[], vr?: VR, bigEndian?: boolean, explicitVR?: boolean): Elements;
    setNumber(tag: number, value: number, vr?: VR, bigEndian?: boolean, explicitVR?: boolean): Elements;
    setDates(tag: number, values: LocalDate[], vr?: VR, bigEndian?: boolean, explicitVR?: boolean): Elements;
    setDate(tag: number, value: LocalDate, vr?: VR, bigEndian?: boolean, explicitVR?: boolean): Elements;
    setTimes(tag: number, values: LocalTime[], vr?: VR, bigEndian?: boolean, explicitVR?: boolean): Elements;
    setTime(tag: number, value: LocalTime, vr?: VR, bigEndian?: boolean, explicitVR?: boolean): Elements;
    setDateTimes(tag: number, values: ZonedDateTime[], vr?: VR, bigEndian?: boolean, explicitVR?: boolean): Elements;
    setDateTime(tag: number, value: ZonedDateTime, vr?: VR, bigEndian?: boolean, explicitVR?: boolean): Elements;
    setPersonNames(tag: number, values: PersonName[], vr?: VR, bigEndian?: boolean, explicitVR?: boolean): Elements;
    setPersonName(tag: number, value: PersonName, vr?: VR, bigEndian?: boolean, explicitVR?: boolean): Elements;
    setURL(tag: number, value: URL, vr?: VR, bigEndian?: boolean, explicitVR?: boolean): Elements;
    setNestedValue(tagPath: TagPathTag, vr: VR, value: Value, bigEndian?: boolean, explicitVR?: boolean): Elements;
    setNestedBytes(tagPath: TagPathTag, vr: VR, value: Buffer, bigEndian?: boolean, explicitVR?: boolean): Elements;
    setNestedStrings(tagPath: TagPathTag, values: string[], vr?: VR, bigEndian?: boolean, explicitVR?: boolean): Elements;
    setNestedString(tagPath: TagPathTag, value: string, vr?: VR, bigEndian?: boolean, explicitVR?: boolean): Elements;
    setNestedNumbers(tagPath: TagPathTag, values: number[], vr?: VR, bigEndian?: boolean, explicitVR?: boolean): Elements;
    setNestedNumber(tagPath: TagPathTag, value: number, vr?: VR, bigEndian?: boolean, explicitVR?: boolean): Elements;
    setNestedDates(tagPath: TagPathTag, values: LocalDate[], vr?: VR, bigEndian?: boolean, explicitVR?: boolean): Elements;
    setNestedDate(tagPath: TagPathTag, value: LocalDate, vr?: VR, bigEndian?: boolean, explicitVR?: boolean): Elements;
    setNestedTimes(tagPath: TagPathTag, values: LocalTime[], vr?: VR, bigEndian?: boolean, explicitVR?: boolean): Elements;
    setNestedTime(tagPath: TagPathTag, value: LocalTime, vr?: VR, bigEndian?: boolean, explicitVR?: boolean): Elements;
    setNestedDateTimes(tagPath: TagPathTag, values: ZonedDateTime[], vr?: VR, bigEndian?: boolean, explicitVR?: boolean): Elements;
    setNestedDateTime(tagPath: TagPathTag, value: ZonedDateTime, vr?: VR, bigEndian?: boolean, explicitVR?: boolean): Elements;
    setNestedPersonNames(tagPath: TagPathTag, values: PersonName[], vr?: VR, bigEndian?: boolean, explicitVR?: boolean): Elements;
    setNestedPersonName(tagPath: TagPathTag, value: PersonName, vr?: VR, bigEndian?: boolean, explicitVR?: boolean): Elements;
    setNestedURL(tagPath: TagPathTag, value: URL, vr?: VR, bigEndian?: boolean, explicitVR?: boolean): Elements;
    removeByTag(tag: number): Elements;
    removeByPath(tagPath: TagPath): Elements;
    filter(f: (e: ElementSet) => boolean): Elements;
    head(): ElementSet | undefined;
    isEmpty(): boolean;
    nonEmpty(): boolean;
    contains(tag: number | TagPath): boolean;
    sorted(): Elements;
    toElements(withPreamble?: boolean): Element[];
    toParts(withPreamble?: boolean): DicomPart[];
    toBytes(withPreamble?: boolean): Buffer;
    toStrings(indent: string): string[];
    toString(): string;
    private _valueByTag;
    private _valueByPath;
    private _valuesByTag;
    private _valuesByPath;
    private _traverseTrunk;
    private _insertOrdered;
}
