"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.flatMapFlow = exports.filterFlow = exports.mapFlow = exports.objectToStringFlow = exports.appendFlow = exports.prependFlow = exports.printFlow = exports.identityFlow = void 0;
const stream_1 = require("stream");
function identityFlow(objectMode = false) {
    return new stream_1.Transform({
        objectMode,
        transform(chunk, encoding, callback) {
            this.push(chunk);
            process.nextTick(() => callback());
        },
    });
}
exports.identityFlow = identityFlow;
function printFlow(objectMode = false) {
    return new stream_1.Transform({
        objectMode,
        transform(chunk, encoding, callback) {
            console.log(chunk);
            this.push(chunk);
            process.nextTick(() => callback());
        },
    });
}
exports.printFlow = printFlow;
function prependFlow(prependChunk, objectMode = false) {
    let hasEmitted = false;
    return new stream_1.Transform({
        objectMode,
        transform(chunk, encoding, callback) {
            if (!hasEmitted) {
                this.push(prependChunk);
                hasEmitted = true;
            }
            this.push(chunk);
            process.nextTick(() => callback());
        },
    });
}
exports.prependFlow = prependFlow;
function appendFlow(appendChunk, objectMode = false) {
    return new stream_1.Transform({
        objectMode,
        transform(chunk, encoding, callback) {
            this.push(chunk);
            process.nextTick(() => callback());
        },
        flush(callback) {
            process.nextTick(() => callback(null, appendChunk));
        },
    });
}
exports.appendFlow = appendFlow;
function objectToStringFlow(toStringFunction) {
    return new stream_1.Transform({
        writableObjectMode: true,
        transform(chunk, encoding, callback) {
            this.push(toStringFunction(chunk) + '\n');
            process.nextTick(() => callback());
        },
    });
}
exports.objectToStringFlow = objectToStringFlow;
function mapFlow(f) {
    return new stream_1.Transform({
        objectMode: true,
        transform(chunk, encoding, callback) {
            try {
                this.push(f(chunk));
                process.nextTick(() => callback());
            }
            catch (error) {
                process.nextTick(() => this.emit('error', error));
            }
        },
    });
}
exports.mapFlow = mapFlow;
function filterFlow(f) {
    return new stream_1.Transform({
        objectMode: true,
        transform(chunk, encoding, callback) {
            try {
                if (f(chunk) === true) {
                    this.push(chunk);
                }
                process.nextTick(() => callback());
            }
            catch (error) {
                process.nextTick(() => this.emit('error', error));
            }
        },
    });
}
exports.filterFlow = filterFlow;
function flatMapFlow(toChunks) {
    return new stream_1.Transform({
        objectMode: true,
        transform(chunk, encoding, callback) {
            try {
                for (const outChunk of toChunks(chunk)) {
                    this.push(outChunk);
                }
                process.nextTick(() => callback());
            }
            catch (error) {
                process.nextTick(() => this.emit('error', error));
            }
        },
    });
}
exports.flatMapFlow = flatMapFlow;
//# sourceMappingURL=flows.js.map