/// <reference types="node" />
import { Element } from './dicom-elements';
import { AttributeInfo } from './parsing';
import { Elements } from './elements';
export declare class Parser {
    readonly stop?: (attributeInfo: AttributeInfo, depth: number) => boolean;
    private builder;
    private byteParser;
    constructor(stop?: (attributeInfo: AttributeInfo, depth: number) => boolean);
    /**
     * Parse the input binary data, producing DICOM elements that are added to the internal builder. An elements
     * structure can be fetched based on the builder at any time.
     */
    parse(chunk: Buffer): void;
    /**
     * Called by byte parser when it is emitting the next parsed element.
     */
    next(element: Element): void;
    /**
     * Get the current elements as represented by the builder
     */
    result(): Elements;
    /**
     * Returns true the parser has completed parsing (stop condition was met)
     */
    isComplete(): boolean;
    /**
     * Called by byte parser when completing parser. Nothing to do here.
     */
    complete(): void;
    /**
     * Called by byte parser on error. Here we just throw the error.
     */
    fail(error: any): void;
}
