"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.arraySink = exports.ignoreSink = exports.byteSink = void 0;
const stream_1 = require("stream");
const base_1 = require("./base");
function byteSink(callback) {
    let buffer = base_1.emptyBuffer;
    const sink = new stream_1.Writable({
        write(chunk, encoding, cb) {
            buffer = (0, base_1.concat)(buffer, chunk);
            process.nextTick(() => cb());
        },
    });
    sink.once('finish', () => {
        callback(buffer);
    });
    return sink;
}
exports.byteSink = byteSink;
function ignoreSink(objectMode = false) {
    return new stream_1.Writable({
        objectMode,
        write(chunk, encoding, callback) {
            process.nextTick(() => callback());
        },
    });
}
exports.ignoreSink = ignoreSink;
function arraySink(arrayCallback) {
    const array = [];
    const sink = new stream_1.Writable({
        objectMode: true,
        write(chunk, encoding, callback) {
            array.push(chunk);
            process.nextTick(() => callback());
        },
    });
    sink.once('finish', () => arrayCallback(array));
    return sink;
}
exports.arraySink = arraySink;
//# sourceMappingURL=sinks.js.map