"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.arraySource = exports.singleSource = void 0;
const stream_1 = require("stream");
function singleSource(element, objectMode = false) {
    return new stream_1.Readable({
        objectMode,
        read() {
            this.push(element);
            this.push(null);
        },
    });
}
exports.singleSource = singleSource;
function arraySource(array, objectMode = false) {
    let pos = 0;
    return new stream_1.Readable({
        highWaterMark: 1,
        objectMode,
        read(size) {
            size = size || 1;
            const maxPos = Math.min(pos + size, array.length);
            let i = pos;
            while (i < maxPos && this.push(array[i++])) {
                // do nothing
            }
            if (i === array.length) {
                this.push(null);
            }
            pos = i;
        },
    });
}
exports.arraySource = arraySource;
//# sourceMappingURL=sources.js.map