/// <reference types="node" />
import { LocalDate, LocalTime, ZonedDateTime, ZoneId } from 'js-joda';
import { CharacterSets } from './character-sets';
import { VR } from './vr';
import { PersonName } from './person-name';
export declare class Value {
    readonly bytes: Buffer;
    static fromString(vr: VR, value: string, bigEndian?: boolean): Value;
    static fromStrings(vr: VR, values: string[], bigEndian?: boolean): Value;
    static fromBuffer(vr: VR, buffer: Buffer): Value;
    static fromBytes(vr: VR, bytes: number[]): Value;
    static fromNumber(vr: VR, value: number, bigEndian?: boolean): Value;
    static fromNumbers(vr: VR, values: number[], bigEndian?: boolean): Value;
    static fromDate(vr: VR, value: LocalDate): Value;
    static fromDates(vr: VR, values: LocalDate[]): Value;
    static fromTime(vr: VR, value: LocalTime): Value;
    static fromTimes(vr: VR, values: LocalTime[]): Value;
    static fromDateTime(vr: VR, value: ZonedDateTime): Value;
    static fromDateTimes(vr: VR, values: ZonedDateTime[]): Value;
    static fromPersonName(vr: VR, value: PersonName): Value;
    static fromPersonNames(vr: VR, values: PersonName[]): Value;
    static fromURL(vr: VR, value: URL): Value;
    static empty(): Value;
    private static headOption;
    length: number;
    constructor(bytes: Buffer);
    toStrings(vr: VR, bigEndian?: boolean, characterSets?: CharacterSets): string[];
    toSingleString(vr: VR, bigEndian?: boolean, characterSets?: CharacterSets): string;
    toNumbers(vr: VR, bigEndian?: boolean): number[];
    toDates(vr?: VR): LocalDate[];
    toTimes(vr?: VR): LocalTime[];
    toDateTimes(vr?: VR, zone?: ZoneId): ZonedDateTime[];
    toPersonNames(vr?: VR, characterSets?: CharacterSets): PersonName[];
    toURL(vr?: VR): URL;
    toString(vr: VR, bigEndian?: boolean, characterSets?: CharacterSets): string;
    toNumber(vr: VR, bigEndian?: boolean): number;
    toDate(vr?: VR): LocalDate;
    toTime(vr?: VR): LocalTime;
    toDateTime(vr?: VR, zone?: ZoneId): ZonedDateTime;
    toPersonName(vr?: VR, characterSets?: CharacterSets): PersonName;
    append(bytes: Buffer): Value;
    ensurePadding(vr: VR): Value;
}
export declare function parsePersonName(s: string): PersonName;
