"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TagToVR = void 0;
const tag_1 = require("./tag");
const vr_1 = require("./vr");
class TagToVR {
    static adjustTag(tag) {
        if ((tag & 0xffe00000) === 0x50000000 || (tag & 0xffe00000) === 0x60000000) {
            return tag & 0xffe0ffff;
        }
        if ((tag & 0xff000000) === 0x7f000000 && (tag & 0xffff0000) !== 0x7fe00000) {
            return tag & 0xff00ffff;
        }
        return tag;
    }
    static vrOf(tag) {
        if ((tag & 0x0000ffff) === 0) {
            return vr_1.VR.UL; // group length
        }
        if ((tag & 0x00010000) !== 0) {
            // private creator ID
            return (tag & 0x0000ff00) === 0 && (tag & 0x000000f0) !== 0 ? vr_1.VR.LO : vr_1.VR.UN;
        }
        if ((tag & 0xffffff00) === tag_1.Tag.SourceImageIDs) {
            return vr_1.VR.CS;
        }
        const key = TagToVR.adjustTag(tag).toString(16);
        return key in TagToVR.dict ? TagToVR.dict[key] : vr_1.VR.UN;
    }
}
TagToVR.dict = {
    '0': vr_1.VR.UL,
    '2': vr_1.VR.UI,
    '3': vr_1.VR.UI,
    '100': vr_1.VR.US,
    '110': vr_1.VR.US,
    '120': vr_1.VR.US,
    '600': vr_1.VR.AE,
    '700': vr_1.VR.US,
    '800': vr_1.VR.US,
    '900': vr_1.VR.US,
    '901': vr_1.VR.AT,
    '902': vr_1.VR.LO,
    '903': vr_1.VR.US,
    '1000': vr_1.VR.UI,
    '1001': vr_1.VR.UI,
    '1002': vr_1.VR.US,
    '1005': vr_1.VR.AT,
    '1008': vr_1.VR.US,
    '1020': vr_1.VR.US,
    '1021': vr_1.VR.US,
    '1022': vr_1.VR.US,
    '1023': vr_1.VR.US,
    '1030': vr_1.VR.AE,
    '1031': vr_1.VR.US,
    '20000': vr_1.VR.UL,
    '20001': vr_1.VR.OB,
    '20002': vr_1.VR.UI,
    '20003': vr_1.VR.UI,
    '20010': vr_1.VR.UI,
    '20012': vr_1.VR.UI,
    '20013': vr_1.VR.SH,
    '20016': vr_1.VR.AE,
    '20017': vr_1.VR.AE,
    '20018': vr_1.VR.AE,
    '20100': vr_1.VR.UI,
    '20102': vr_1.VR.OB,
    '41130': vr_1.VR.CS,
    '41141': vr_1.VR.CS,
    '41142': vr_1.VR.CS,
    '41200': vr_1.VR.UL,
    '41202': vr_1.VR.UL,
    '41212': vr_1.VR.US,
    '41220': vr_1.VR.SQ,
    '41400': vr_1.VR.UL,
    '41410': vr_1.VR.US,
    '41420': vr_1.VR.UL,
    '41430': vr_1.VR.CS,
    '41432': vr_1.VR.UI,
    '41500': vr_1.VR.CS,
    '41504': vr_1.VR.UL,
    '41510': vr_1.VR.UI,
    '41511': vr_1.VR.UI,
    '41512': vr_1.VR.UI,
    '4151a': vr_1.VR.UI,
    '41600': vr_1.VR.UL,
    '80001': vr_1.VR.UL,
    '80005': vr_1.VR.CS,
    '80006': vr_1.VR.SQ,
    '80008': vr_1.VR.CS,
    '80010': vr_1.VR.SH,
    '80012': vr_1.VR.DA,
    '80013': vr_1.VR.TM,
    '80014': vr_1.VR.UI,
    '80015': vr_1.VR.DT,
    '80016': vr_1.VR.UI,
    '80018': vr_1.VR.UI,
    '8001a': vr_1.VR.UI,
    '8001b': vr_1.VR.UI,
    '80020': vr_1.VR.DA,
    '80021': vr_1.VR.DA,
    '80022': vr_1.VR.DA,
    '80023': vr_1.VR.DA,
    '80024': vr_1.VR.DA,
    '80025': vr_1.VR.DA,
    '8002a': vr_1.VR.DT,
    '80030': vr_1.VR.TM,
    '80031': vr_1.VR.TM,
    '80032': vr_1.VR.TM,
    '80033': vr_1.VR.TM,
    '80034': vr_1.VR.TM,
    '80035': vr_1.VR.TM,
    '80040': vr_1.VR.US,
    '80041': vr_1.VR.LO,
    '80042': vr_1.VR.CS,
    '80050': vr_1.VR.SH,
    '80051': vr_1.VR.SQ,
    '80052': vr_1.VR.CS,
    '80053': vr_1.VR.CS,
    '80054': vr_1.VR.AE,
    '80055': vr_1.VR.AE,
    '80056': vr_1.VR.CS,
    '80058': vr_1.VR.UI,
    '80060': vr_1.VR.CS,
    '80061': vr_1.VR.CS,
    '80062': vr_1.VR.UI,
    '80063': vr_1.VR.SQ,
    '80064': vr_1.VR.CS,
    '80068': vr_1.VR.CS,
    '80070': vr_1.VR.LO,
    '80080': vr_1.VR.LO,
    '80081': vr_1.VR.ST,
    '80082': vr_1.VR.SQ,
    '80090': vr_1.VR.PN,
    '80092': vr_1.VR.ST,
    '80094': vr_1.VR.SH,
    '80096': vr_1.VR.SQ,
    '8009c': vr_1.VR.PN,
    '8009d': vr_1.VR.SQ,
    '80100': vr_1.VR.SH,
    '80101': vr_1.VR.LO,
    '80102': vr_1.VR.SH,
    '80103': vr_1.VR.SH,
    '80104': vr_1.VR.LO,
    '80105': vr_1.VR.CS,
    '80106': vr_1.VR.DT,
    '80107': vr_1.VR.DT,
    '80108': vr_1.VR.LT,
    '80109': vr_1.VR.SQ,
    '8010a': vr_1.VR.CS,
    '8010b': vr_1.VR.CS,
    '8010c': vr_1.VR.UI,
    '8010d': vr_1.VR.UI,
    '8010e': vr_1.VR.UR,
    '8010f': vr_1.VR.CS,
    '80110': vr_1.VR.SQ,
    '80112': vr_1.VR.LO,
    '80114': vr_1.VR.ST,
    '80115': vr_1.VR.ST,
    '80116': vr_1.VR.ST,
    '80117': vr_1.VR.UI,
    '80118': vr_1.VR.UI,
    '80119': vr_1.VR.UC,
    '80120': vr_1.VR.UR,
    '80121': vr_1.VR.SQ,
    '80122': vr_1.VR.LO,
    '80123': vr_1.VR.SQ,
    '80124': vr_1.VR.SQ,
    '80201': vr_1.VR.SH,
    '80220': vr_1.VR.SQ,
    '80221': vr_1.VR.CS,
    '80222': vr_1.VR.LO,
    '80300': vr_1.VR.SQ,
    '80301': vr_1.VR.US,
    '80302': vr_1.VR.LO,
    '80303': vr_1.VR.CS,
    '80304': vr_1.VR.US,
    '80305': vr_1.VR.SQ,
    '80306': vr_1.VR.US,
    '80307': vr_1.VR.CS,
    '80308': vr_1.VR.US,
    '80309': vr_1.VR.UL,
    '8030a': vr_1.VR.CS,
    '8030b': vr_1.VR.UL,
    '8030c': vr_1.VR.UC,
    '8030d': vr_1.VR.UC,
    '8030e': vr_1.VR.UT,
    '8030f': vr_1.VR.UT,
    '80310': vr_1.VR.SQ,
    '81000': vr_1.VR.AE,
    '81010': vr_1.VR.SH,
    '81030': vr_1.VR.LO,
    '81032': vr_1.VR.SQ,
    '8103e': vr_1.VR.LO,
    '8103f': vr_1.VR.SQ,
    '81040': vr_1.VR.LO,
    '81048': vr_1.VR.PN,
    '81049': vr_1.VR.SQ,
    '81050': vr_1.VR.PN,
    '81052': vr_1.VR.SQ,
    '81060': vr_1.VR.PN,
    '81062': vr_1.VR.SQ,
    '81070': vr_1.VR.PN,
    '81072': vr_1.VR.SQ,
    '81080': vr_1.VR.LO,
    '81084': vr_1.VR.SQ,
    '81090': vr_1.VR.LO,
    '81100': vr_1.VR.SQ,
    '81110': vr_1.VR.SQ,
    '81111': vr_1.VR.SQ,
    '81115': vr_1.VR.SQ,
    '81120': vr_1.VR.SQ,
    '81125': vr_1.VR.SQ,
    '81130': vr_1.VR.SQ,
    '81134': vr_1.VR.SQ,
    '8113a': vr_1.VR.SQ,
    '81140': vr_1.VR.SQ,
    '81145': vr_1.VR.SQ,
    '8114a': vr_1.VR.SQ,
    '8114b': vr_1.VR.SQ,
    '81150': vr_1.VR.UI,
    '81155': vr_1.VR.UI,
    '81156': vr_1.VR.SQ,
    '8115a': vr_1.VR.UI,
    '81160': vr_1.VR.IS,
    '81161': vr_1.VR.UL,
    '81162': vr_1.VR.UL,
    '81163': vr_1.VR.FD,
    '81164': vr_1.VR.SQ,
    '81167': vr_1.VR.UI,
    '81190': vr_1.VR.UR,
    '81195': vr_1.VR.UI,
    '81196': vr_1.VR.US,
    '81197': vr_1.VR.US,
    '81198': vr_1.VR.SQ,
    '81199': vr_1.VR.SQ,
    '8119a': vr_1.VR.SQ,
    '81200': vr_1.VR.SQ,
    '81250': vr_1.VR.SQ,
    '82110': vr_1.VR.CS,
    '82111': vr_1.VR.ST,
    '82112': vr_1.VR.SQ,
    '82120': vr_1.VR.SH,
    '82122': vr_1.VR.IS,
    '82124': vr_1.VR.IS,
    '82127': vr_1.VR.SH,
    '82128': vr_1.VR.IS,
    '82129': vr_1.VR.IS,
    '8212a': vr_1.VR.IS,
    '82130': vr_1.VR.DS,
    '82132': vr_1.VR.LO,
    '82133': vr_1.VR.SQ,
    '82134': vr_1.VR.FD,
    '82135': vr_1.VR.SQ,
    '82142': vr_1.VR.IS,
    '82143': vr_1.VR.IS,
    '82144': vr_1.VR.IS,
    '82200': vr_1.VR.CS,
    '82204': vr_1.VR.CS,
    '82208': vr_1.VR.CS,
    '82218': vr_1.VR.SQ,
    '82220': vr_1.VR.SQ,
    '82228': vr_1.VR.SQ,
    '82229': vr_1.VR.SQ,
    '82230': vr_1.VR.SQ,
    '82240': vr_1.VR.SQ,
    '82242': vr_1.VR.SQ,
    '82244': vr_1.VR.SQ,
    '82246': vr_1.VR.SQ,
    '82251': vr_1.VR.SQ,
    '82253': vr_1.VR.SQ,
    '82255': vr_1.VR.SQ,
    '82256': vr_1.VR.ST,
    '82257': vr_1.VR.SQ,
    '82258': vr_1.VR.ST,
    '82259': vr_1.VR.SQ,
    '8225a': vr_1.VR.SQ,
    '8225c': vr_1.VR.SQ,
    '83001': vr_1.VR.SQ,
    '83010': vr_1.VR.UI,
    '83011': vr_1.VR.SQ,
    '83012': vr_1.VR.UI,
    '84000': vr_1.VR.LT,
    '89007': vr_1.VR.CS,
    '89092': vr_1.VR.SQ,
    '89121': vr_1.VR.SQ,
    '89123': vr_1.VR.UI,
    '89124': vr_1.VR.SQ,
    '89154': vr_1.VR.SQ,
    '89205': vr_1.VR.CS,
    '89206': vr_1.VR.CS,
    '89207': vr_1.VR.CS,
    '89208': vr_1.VR.CS,
    '89209': vr_1.VR.CS,
    '89215': vr_1.VR.SQ,
    '89237': vr_1.VR.SQ,
    '89410': vr_1.VR.SQ,
    '89458': vr_1.VR.SQ,
    '89459': vr_1.VR.FL,
    '89460': vr_1.VR.CS,
    '100010': vr_1.VR.PN,
    '100020': vr_1.VR.LO,
    '100021': vr_1.VR.LO,
    '100022': vr_1.VR.CS,
    '100024': vr_1.VR.SQ,
    '100026': vr_1.VR.SQ,
    '100027': vr_1.VR.SQ,
    '100028': vr_1.VR.US,
    '100030': vr_1.VR.DA,
    '100032': vr_1.VR.TM,
    '100033': vr_1.VR.LO,
    '100034': vr_1.VR.LO,
    '100035': vr_1.VR.CS,
    '100040': vr_1.VR.CS,
    '100050': vr_1.VR.SQ,
    '100101': vr_1.VR.SQ,
    '100102': vr_1.VR.SQ,
    '100200': vr_1.VR.CS,
    '100201': vr_1.VR.SQ,
    '100212': vr_1.VR.UC,
    '100213': vr_1.VR.LO,
    '100214': vr_1.VR.LO,
    '100215': vr_1.VR.SQ,
    '100216': vr_1.VR.SQ,
    '100217': vr_1.VR.LO,
    '100218': vr_1.VR.UT,
    '100219': vr_1.VR.SQ,
    '100221': vr_1.VR.SQ,
    '100222': vr_1.VR.UC,
    '100223': vr_1.VR.LO,
    '100229': vr_1.VR.SQ,
    '101000': vr_1.VR.LO,
    '101001': vr_1.VR.PN,
    '101002': vr_1.VR.SQ,
    '101005': vr_1.VR.PN,
    '101010': vr_1.VR.AS,
    '101020': vr_1.VR.DS,
    '101021': vr_1.VR.SQ,
    '101022': vr_1.VR.DS,
    '101023': vr_1.VR.DS,
    '101024': vr_1.VR.DS,
    '101030': vr_1.VR.DS,
    '101040': vr_1.VR.LO,
    '101050': vr_1.VR.LO,
    '101060': vr_1.VR.PN,
    '101080': vr_1.VR.LO,
    '101081': vr_1.VR.LO,
    '101090': vr_1.VR.LO,
    '101100': vr_1.VR.SQ,
    '102000': vr_1.VR.LO,
    '102110': vr_1.VR.LO,
    '102150': vr_1.VR.LO,
    '102152': vr_1.VR.LO,
    '102154': vr_1.VR.SH,
    '102155': vr_1.VR.LT,
    '102160': vr_1.VR.SH,
    '102180': vr_1.VR.SH,
    '1021a0': vr_1.VR.CS,
    '1021b0': vr_1.VR.LT,
    '1021c0': vr_1.VR.US,
    '1021d0': vr_1.VR.DA,
    '1021f0': vr_1.VR.LO,
    '102201': vr_1.VR.LO,
    '102202': vr_1.VR.SQ,
    '102203': vr_1.VR.CS,
    '102210': vr_1.VR.CS,
    '102292': vr_1.VR.LO,
    '102293': vr_1.VR.SQ,
    '102294': vr_1.VR.SQ,
    '102295': vr_1.VR.LO,
    '102296': vr_1.VR.SQ,
    '102297': vr_1.VR.PN,
    '102298': vr_1.VR.CS,
    '102299': vr_1.VR.LO,
    '104000': vr_1.VR.LT,
    '109431': vr_1.VR.FL,
    '120010': vr_1.VR.LO,
    '120020': vr_1.VR.LO,
    '120021': vr_1.VR.LO,
    '120030': vr_1.VR.LO,
    '120031': vr_1.VR.LO,
    '120040': vr_1.VR.LO,
    '120042': vr_1.VR.LO,
    '120050': vr_1.VR.LO,
    '120051': vr_1.VR.ST,
    '120052': vr_1.VR.FD,
    '120053': vr_1.VR.CS,
    '120060': vr_1.VR.LO,
    '120062': vr_1.VR.CS,
    '120063': vr_1.VR.LO,
    '120064': vr_1.VR.SQ,
    '120071': vr_1.VR.LO,
    '120072': vr_1.VR.LO,
    '120081': vr_1.VR.LO,
    '120082': vr_1.VR.LO,
    '120083': vr_1.VR.SQ,
    '120084': vr_1.VR.CS,
    '120085': vr_1.VR.CS,
    '120086': vr_1.VR.DA,
    '120087': vr_1.VR.DA,
    '140023': vr_1.VR.ST,
    '140024': vr_1.VR.ST,
    '140025': vr_1.VR.ST,
    '140028': vr_1.VR.ST,
    '140030': vr_1.VR.DS,
    '140032': vr_1.VR.DS,
    '140034': vr_1.VR.DS,
    '140042': vr_1.VR.ST,
    '140044': vr_1.VR.ST,
    '140045': vr_1.VR.ST,
    '140046': vr_1.VR.LT,
    '140050': vr_1.VR.CS,
    '140052': vr_1.VR.CS,
    '140054': vr_1.VR.DS,
    '140056': vr_1.VR.DS,
    '140100': vr_1.VR.LO,
    '140101': vr_1.VR.CS,
    '140102': vr_1.VR.DA,
    '140103': vr_1.VR.TM,
    '140104': vr_1.VR.PN,
    '140105': vr_1.VR.ST,
    '140106': vr_1.VR.SQ,
    '140107': vr_1.VR.CS,
    '140108': vr_1.VR.CS,
    '141010': vr_1.VR.ST,
    '141020': vr_1.VR.DA,
    '141040': vr_1.VR.ST,
    '142002': vr_1.VR.SQ,
    '142004': vr_1.VR.IS,
    '142006': vr_1.VR.PN,
    '142008': vr_1.VR.IS,
    '142012': vr_1.VR.SQ,
    '142014': vr_1.VR.IS,
    '142016': vr_1.VR.SH,
    '142018': vr_1.VR.ST,
    '14201a': vr_1.VR.CS,
    '14201c': vr_1.VR.CS,
    '14201e': vr_1.VR.SQ,
    '142030': vr_1.VR.SQ,
    '142032': vr_1.VR.SH,
    '142202': vr_1.VR.IS,
    '142204': vr_1.VR.SQ,
    '142206': vr_1.VR.ST,
    '142208': vr_1.VR.CS,
    '14220a': vr_1.VR.IS,
    '14220c': vr_1.VR.CS,
    '14220e': vr_1.VR.CS,
    '142210': vr_1.VR.OB,
    '142220': vr_1.VR.SQ,
    '142222': vr_1.VR.ST,
    '142224': vr_1.VR.IS,
    '142226': vr_1.VR.IS,
    '142228': vr_1.VR.CS,
    '14222a': vr_1.VR.DS,
    '14222c': vr_1.VR.DS,
    '143011': vr_1.VR.DS,
    '143012': vr_1.VR.DS,
    '143020': vr_1.VR.SQ,
    '143022': vr_1.VR.ST,
    '143024': vr_1.VR.DS,
    '143026': vr_1.VR.DS,
    '143028': vr_1.VR.DS,
    '143040': vr_1.VR.SQ,
    '143050': vr_1.VR.OW,
    '143060': vr_1.VR.SQ,
    '143070': vr_1.VR.OW,
    '143071': vr_1.VR.DS,
    '143072': vr_1.VR.DS,
    '143073': vr_1.VR.DS,
    '143074': vr_1.VR.LO,
    '143075': vr_1.VR.DS,
    '143076': vr_1.VR.DA,
    '143077': vr_1.VR.TM,
    '143080': vr_1.VR.OB,
    '143099': vr_1.VR.LT,
    '144002': vr_1.VR.SQ,
    '144004': vr_1.VR.CS,
    '144006': vr_1.VR.LT,
    '144008': vr_1.VR.SQ,
    '14400a': vr_1.VR.CS,
    '14400c': vr_1.VR.LT,
    '14400e': vr_1.VR.SQ,
    '14400f': vr_1.VR.LT,
    '144010': vr_1.VR.SQ,
    '144011': vr_1.VR.SQ,
    '144012': vr_1.VR.US,
    '144013': vr_1.VR.CS,
    '144014': vr_1.VR.DS,
    '144015': vr_1.VR.DS,
    '144016': vr_1.VR.DS,
    '144017': vr_1.VR.DS,
    '144018': vr_1.VR.DS,
    '144019': vr_1.VR.DS,
    '14401a': vr_1.VR.DS,
    '14401b': vr_1.VR.DS,
    '14401c': vr_1.VR.DS,
    '14401d': vr_1.VR.DS,
    '144020': vr_1.VR.SQ,
    '144022': vr_1.VR.DS,
    '144024': vr_1.VR.DS,
    '144026': vr_1.VR.CS,
    '144028': vr_1.VR.DS,
    '144030': vr_1.VR.SQ,
    '144031': vr_1.VR.DS,
    '144032': vr_1.VR.CS,
    '144033': vr_1.VR.IS,
    '144034': vr_1.VR.DS,
    '144035': vr_1.VR.SQ,
    '144036': vr_1.VR.CS,
    '144038': vr_1.VR.DS,
    '14403a': vr_1.VR.DS,
    '14403c': vr_1.VR.DS,
    '144040': vr_1.VR.SQ,
    '144050': vr_1.VR.SQ,
    '144051': vr_1.VR.SQ,
    '144052': vr_1.VR.DS,
    '144054': vr_1.VR.ST,
    '144056': vr_1.VR.ST,
    '144057': vr_1.VR.DS,
    '144058': vr_1.VR.DS,
    '144059': vr_1.VR.DS,
    '14405a': vr_1.VR.DS,
    '14405c': vr_1.VR.ST,
    '144060': vr_1.VR.SQ,
    '144062': vr_1.VR.DS,
    '144064': vr_1.VR.DS,
    '144070': vr_1.VR.SQ,
    '144072': vr_1.VR.ST,
    '144074': vr_1.VR.SH,
    '144076': vr_1.VR.DA,
    '144078': vr_1.VR.DA,
    '14407a': vr_1.VR.DA,
    '14407c': vr_1.VR.TM,
    '14407e': vr_1.VR.DA,
    '144080': vr_1.VR.SQ,
    '144081': vr_1.VR.CS,
    '144082': vr_1.VR.LT,
    '144083': vr_1.VR.SQ,
    '144084': vr_1.VR.DS,
    '144085': vr_1.VR.DS,
    '144086': vr_1.VR.SQ,
    '144087': vr_1.VR.SQ,
    '144088': vr_1.VR.DS,
    '144089': vr_1.VR.DS,
    '14408b': vr_1.VR.DS,
    '14408c': vr_1.VR.DS,
    '14408d': vr_1.VR.DS,
    '14408e': vr_1.VR.DS,
    '144091': vr_1.VR.SQ,
    '144092': vr_1.VR.DS,
    '14409a': vr_1.VR.SQ,
    '14409b': vr_1.VR.ST,
    '14409c': vr_1.VR.DS,
    '14409d': vr_1.VR.DS,
    '14409f': vr_1.VR.DS,
    '1440a0': vr_1.VR.LO,
    '1440a1': vr_1.VR.LO,
    '1440a2': vr_1.VR.LO,
    '145002': vr_1.VR.IS,
    '145004': vr_1.VR.IS,
    '145100': vr_1.VR.US,
    '145101': vr_1.VR.DS,
    '145102': vr_1.VR.DS,
    '145103': vr_1.VR.DS,
    '145104': vr_1.VR.DS,
    '145105': vr_1.VR.CS,
    '145106': vr_1.VR.SH,
    '145107': vr_1.VR.DS,
    '145108': vr_1.VR.DS,
    '145109': vr_1.VR.CS,
    '14510a': vr_1.VR.DS,
    '14510b': vr_1.VR.SH,
    '14510c': vr_1.VR.DS,
    '14510d': vr_1.VR.DS,
    '14510e': vr_1.VR.DS,
    '14510f': vr_1.VR.SH,
    '145110': vr_1.VR.SH,
    '145111': vr_1.VR.LO,
    '145112': vr_1.VR.DS,
    '145113': vr_1.VR.DS,
    '145114': vr_1.VR.DS,
    '145115': vr_1.VR.DS,
    '145116': vr_1.VR.DS,
    '145117': vr_1.VR.DS,
    '145118': vr_1.VR.DS,
    '145119': vr_1.VR.DS,
    '14511a': vr_1.VR.DS,
    '14511b': vr_1.VR.DS,
    '14511c': vr_1.VR.DS,
    '14511d': vr_1.VR.DS,
    '14511e': vr_1.VR.CS,
    '14511f': vr_1.VR.DS,
    '180010': vr_1.VR.LO,
    '180012': vr_1.VR.SQ,
    '180013': vr_1.VR.FL,
    '180014': vr_1.VR.SQ,
    '180015': vr_1.VR.CS,
    '180020': vr_1.VR.CS,
    '180021': vr_1.VR.CS,
    '180022': vr_1.VR.CS,
    '180023': vr_1.VR.CS,
    '180024': vr_1.VR.SH,
    '180025': vr_1.VR.CS,
    '180026': vr_1.VR.SQ,
    '180027': vr_1.VR.TM,
    '180028': vr_1.VR.DS,
    '180029': vr_1.VR.SQ,
    '18002a': vr_1.VR.SQ,
    '180030': vr_1.VR.LO,
    '180031': vr_1.VR.LO,
    '180032': vr_1.VR.DS,
    '180033': vr_1.VR.DS,
    '180034': vr_1.VR.LO,
    '180035': vr_1.VR.TM,
    '180036': vr_1.VR.SQ,
    '180037': vr_1.VR.CS,
    '180038': vr_1.VR.CS,
    '180039': vr_1.VR.CS,
    '18003a': vr_1.VR.ST,
    '180040': vr_1.VR.IS,
    '180042': vr_1.VR.CS,
    '180050': vr_1.VR.DS,
    '180060': vr_1.VR.DS,
    '180070': vr_1.VR.IS,
    '180071': vr_1.VR.CS,
    '180072': vr_1.VR.DS,
    '180073': vr_1.VR.CS,
    '180074': vr_1.VR.IS,
    '180075': vr_1.VR.IS,
    '180080': vr_1.VR.DS,
    '180081': vr_1.VR.DS,
    '180082': vr_1.VR.DS,
    '180083': vr_1.VR.DS,
    '180084': vr_1.VR.DS,
    '180085': vr_1.VR.SH,
    '180086': vr_1.VR.IS,
    '180087': vr_1.VR.DS,
    '180088': vr_1.VR.DS,
    '180089': vr_1.VR.IS,
    '180090': vr_1.VR.DS,
    '180091': vr_1.VR.IS,
    '180093': vr_1.VR.DS,
    '180094': vr_1.VR.DS,
    '180095': vr_1.VR.DS,
    '181000': vr_1.VR.LO,
    '181002': vr_1.VR.UI,
    '181003': vr_1.VR.LO,
    '181004': vr_1.VR.LO,
    '181005': vr_1.VR.LO,
    '181006': vr_1.VR.LO,
    '181007': vr_1.VR.LO,
    '181008': vr_1.VR.LO,
    '181009': vr_1.VR.UT,
    '18100a': vr_1.VR.SQ,
    '181010': vr_1.VR.LO,
    '181011': vr_1.VR.LO,
    '181012': vr_1.VR.DA,
    '181014': vr_1.VR.TM,
    '181016': vr_1.VR.LO,
    '181017': vr_1.VR.LO,
    '181018': vr_1.VR.LO,
    '181019': vr_1.VR.LO,
    '18101a': vr_1.VR.LO,
    '18101b': vr_1.VR.LO,
    '181020': vr_1.VR.LO,
    '181022': vr_1.VR.SH,
    '181023': vr_1.VR.LO,
    '181030': vr_1.VR.LO,
    '181040': vr_1.VR.LO,
    '181041': vr_1.VR.DS,
    '181042': vr_1.VR.TM,
    '181043': vr_1.VR.TM,
    '181044': vr_1.VR.DS,
    '181045': vr_1.VR.IS,
    '181046': vr_1.VR.DS,
    '181047': vr_1.VR.DS,
    '181048': vr_1.VR.CS,
    '181049': vr_1.VR.DS,
    '181050': vr_1.VR.DS,
    '181060': vr_1.VR.DS,
    '181061': vr_1.VR.LO,
    '181062': vr_1.VR.IS,
    '181063': vr_1.VR.DS,
    '181064': vr_1.VR.LO,
    '181065': vr_1.VR.DS,
    '181066': vr_1.VR.DS,
    '181067': vr_1.VR.DS,
    '181068': vr_1.VR.DS,
    '181069': vr_1.VR.DS,
    '18106a': vr_1.VR.CS,
    '18106c': vr_1.VR.US,
    '18106e': vr_1.VR.UL,
    '181070': vr_1.VR.LO,
    '181071': vr_1.VR.DS,
    '181072': vr_1.VR.TM,
    '181073': vr_1.VR.TM,
    '181074': vr_1.VR.DS,
    '181075': vr_1.VR.DS,
    '181076': vr_1.VR.DS,
    '181077': vr_1.VR.DS,
    '181078': vr_1.VR.DT,
    '181079': vr_1.VR.DT,
    '181080': vr_1.VR.CS,
    '181081': vr_1.VR.IS,
    '181082': vr_1.VR.IS,
    '181083': vr_1.VR.IS,
    '181084': vr_1.VR.IS,
    '181085': vr_1.VR.LO,
    '181086': vr_1.VR.IS,
    '181088': vr_1.VR.IS,
    '181090': vr_1.VR.IS,
    '181094': vr_1.VR.IS,
    '181100': vr_1.VR.DS,
    '181110': vr_1.VR.DS,
    '181111': vr_1.VR.DS,
    '181114': vr_1.VR.DS,
    '181120': vr_1.VR.DS,
    '181121': vr_1.VR.DS,
    '181130': vr_1.VR.DS,
    '181131': vr_1.VR.DS,
    '181134': vr_1.VR.CS,
    '181135': vr_1.VR.DS,
    '181136': vr_1.VR.DS,
    '181137': vr_1.VR.DS,
    '181138': vr_1.VR.DS,
    '18113a': vr_1.VR.CS,
    '181140': vr_1.VR.CS,
    '181141': vr_1.VR.DS,
    '181142': vr_1.VR.DS,
    '181143': vr_1.VR.DS,
    '181144': vr_1.VR.DS,
    '181145': vr_1.VR.DS,
    '181146': vr_1.VR.DS,
    '181147': vr_1.VR.CS,
    '181149': vr_1.VR.IS,
    '181150': vr_1.VR.IS,
    '181151': vr_1.VR.IS,
    '181152': vr_1.VR.IS,
    '181153': vr_1.VR.IS,
    '181154': vr_1.VR.DS,
    '181155': vr_1.VR.CS,
    '181156': vr_1.VR.CS,
    '18115a': vr_1.VR.CS,
    '18115e': vr_1.VR.DS,
    '181160': vr_1.VR.SH,
    '181161': vr_1.VR.LO,
    '181162': vr_1.VR.DS,
    '181164': vr_1.VR.DS,
    '181166': vr_1.VR.CS,
    '181170': vr_1.VR.IS,
    '181180': vr_1.VR.SH,
    '181181': vr_1.VR.CS,
    '181182': vr_1.VR.IS,
    '181183': vr_1.VR.DS,
    '181184': vr_1.VR.DS,
    '181190': vr_1.VR.DS,
    '181191': vr_1.VR.CS,
    '1811a0': vr_1.VR.DS,
    '1811a2': vr_1.VR.DS,
    '1811a3': vr_1.VR.DS,
    '1811a4': vr_1.VR.LO,
    '1811a5': vr_1.VR.DS,
    '181200': vr_1.VR.DA,
    '181201': vr_1.VR.TM,
    '181202': vr_1.VR.DT,
    '181210': vr_1.VR.SH,
    '181240': vr_1.VR.IS,
    '181242': vr_1.VR.IS,
    '181243': vr_1.VR.IS,
    '181244': vr_1.VR.US,
    '181250': vr_1.VR.SH,
    '181251': vr_1.VR.SH,
    '181260': vr_1.VR.SH,
    '181261': vr_1.VR.LO,
    '181271': vr_1.VR.FD,
    '181272': vr_1.VR.SQ,
    '181300': vr_1.VR.DS,
    '181301': vr_1.VR.CS,
    '181302': vr_1.VR.IS,
    '181310': vr_1.VR.US,
    '181312': vr_1.VR.CS,
    '181314': vr_1.VR.DS,
    '181315': vr_1.VR.CS,
    '181316': vr_1.VR.DS,
    '181318': vr_1.VR.DS,
    '181320': vr_1.VR.FL,
    '181400': vr_1.VR.LO,
    '181401': vr_1.VR.LO,
    '181402': vr_1.VR.CS,
    '181403': vr_1.VR.CS,
    '181404': vr_1.VR.US,
    '181405': vr_1.VR.IS,
    '181411': vr_1.VR.DS,
    '181412': vr_1.VR.DS,
    '181413': vr_1.VR.DS,
    '181450': vr_1.VR.DS,
    '181460': vr_1.VR.DS,
    '181470': vr_1.VR.DS,
    '181480': vr_1.VR.DS,
    '181490': vr_1.VR.CS,
    '181491': vr_1.VR.CS,
    '181495': vr_1.VR.IS,
    '181500': vr_1.VR.CS,
    '181508': vr_1.VR.CS,
    '181510': vr_1.VR.DS,
    '181511': vr_1.VR.DS,
    '181520': vr_1.VR.DS,
    '181521': vr_1.VR.DS,
    '181530': vr_1.VR.DS,
    '181531': vr_1.VR.DS,
    '181600': vr_1.VR.CS,
    '181602': vr_1.VR.IS,
    '181604': vr_1.VR.IS,
    '181606': vr_1.VR.IS,
    '181608': vr_1.VR.IS,
    '181610': vr_1.VR.IS,
    '181612': vr_1.VR.IS,
    '181620': vr_1.VR.IS,
    '181622': vr_1.VR.US,
    '181623': vr_1.VR.US,
    '181624': vr_1.VR.US,
    '181700': vr_1.VR.CS,
    '181702': vr_1.VR.IS,
    '181704': vr_1.VR.IS,
    '181706': vr_1.VR.IS,
    '181708': vr_1.VR.IS,
    '181710': vr_1.VR.IS,
    '181712': vr_1.VR.IS,
    '181720': vr_1.VR.IS,
    '181800': vr_1.VR.CS,
    '181801': vr_1.VR.SH,
    '181802': vr_1.VR.CS,
    '181803': vr_1.VR.LO,
    '182001': vr_1.VR.IS,
    '182002': vr_1.VR.SH,
    '182003': vr_1.VR.DS,
    '182004': vr_1.VR.DS,
    '182005': vr_1.VR.DS,
    '182006': vr_1.VR.SH,
    '182010': vr_1.VR.DS,
    '182020': vr_1.VR.CS,
    '182030': vr_1.VR.DS,
    '182041': vr_1.VR.SQ,
    '182042': vr_1.VR.UI,
    '182043': vr_1.VR.FL,
    '182044': vr_1.VR.FL,
    '182045': vr_1.VR.SH,
    '182046': vr_1.VR.FL,
    '183100': vr_1.VR.CS,
    '183101': vr_1.VR.DS,
    '183102': vr_1.VR.DS,
    '183103': vr_1.VR.IS,
    '183104': vr_1.VR.IS,
    '183105': vr_1.VR.IS,
    '184000': vr_1.VR.LT,
    '185000': vr_1.VR.SH,
    '185010': vr_1.VR.LO,
    '185012': vr_1.VR.DS,
    '185020': vr_1.VR.LO,
    '185021': vr_1.VR.LO,
    '185022': vr_1.VR.DS,
    '185024': vr_1.VR.DS,
    '185026': vr_1.VR.DS,
    '185027': vr_1.VR.DS,
    '185028': vr_1.VR.DS,
    '185029': vr_1.VR.DS,
    '185030': vr_1.VR.DS,
    '185040': vr_1.VR.DS,
    '185050': vr_1.VR.IS,
    '185100': vr_1.VR.CS,
    '185101': vr_1.VR.CS,
    '185104': vr_1.VR.SQ,
    '185210': vr_1.VR.DS,
    '185212': vr_1.VR.DS,
    '186000': vr_1.VR.DS,
    '186011': vr_1.VR.SQ,
    '186012': vr_1.VR.US,
    '186014': vr_1.VR.US,
    '186016': vr_1.VR.UL,
    '186018': vr_1.VR.UL,
    '18601a': vr_1.VR.UL,
    '18601c': vr_1.VR.UL,
    '18601e': vr_1.VR.UL,
    '186020': vr_1.VR.SL,
    '186022': vr_1.VR.SL,
    '186024': vr_1.VR.US,
    '186026': vr_1.VR.US,
    '186028': vr_1.VR.FD,
    '18602a': vr_1.VR.FD,
    '18602c': vr_1.VR.FD,
    '18602e': vr_1.VR.FD,
    '186030': vr_1.VR.UL,
    '186031': vr_1.VR.CS,
    '186032': vr_1.VR.UL,
    '186034': vr_1.VR.FD,
    '186036': vr_1.VR.FD,
    '186038': vr_1.VR.UL,
    '186039': vr_1.VR.SL,
    '18603a': vr_1.VR.UL,
    '18603b': vr_1.VR.SL,
    '18603c': vr_1.VR.UL,
    '18603d': vr_1.VR.SL,
    '18603e': vr_1.VR.UL,
    '18603f': vr_1.VR.SL,
    '186040': vr_1.VR.UL,
    '186041': vr_1.VR.SL,
    '186042': vr_1.VR.UL,
    '186043': vr_1.VR.SL,
    '186044': vr_1.VR.US,
    '186046': vr_1.VR.UL,
    '186048': vr_1.VR.UL,
    '18604a': vr_1.VR.UL,
    '18604c': vr_1.VR.US,
    '18604e': vr_1.VR.US,
    '186050': vr_1.VR.UL,
    '186052': vr_1.VR.UL,
    '186054': vr_1.VR.FD,
    '186056': vr_1.VR.UL,
    '186058': vr_1.VR.UL,
    '18605a': vr_1.VR.FL,
    '186060': vr_1.VR.FL,
    '187000': vr_1.VR.CS,
    '187001': vr_1.VR.DS,
    '187004': vr_1.VR.CS,
    '187005': vr_1.VR.CS,
    '187006': vr_1.VR.LT,
    '187008': vr_1.VR.LT,
    '18700a': vr_1.VR.SH,
    '18700c': vr_1.VR.DA,
    '18700e': vr_1.VR.TM,
    '187010': vr_1.VR.IS,
    '187011': vr_1.VR.IS,
    '187012': vr_1.VR.DS,
    '187014': vr_1.VR.DS,
    '187016': vr_1.VR.DS,
    '18701a': vr_1.VR.DS,
    '187020': vr_1.VR.DS,
    '187022': vr_1.VR.DS,
    '187024': vr_1.VR.CS,
    '187026': vr_1.VR.DS,
    '187028': vr_1.VR.DS,
    '18702a': vr_1.VR.LO,
    '18702b': vr_1.VR.LO,
    '187030': vr_1.VR.DS,
    '187032': vr_1.VR.DS,
    '187034': vr_1.VR.CS,
    '187036': vr_1.VR.FL,
    '187038': vr_1.VR.FL,
    '187040': vr_1.VR.LT,
    '187041': vr_1.VR.LT,
    '187042': vr_1.VR.DS,
    '187044': vr_1.VR.DS,
    '187046': vr_1.VR.IS,
    '187048': vr_1.VR.DS,
    '18704c': vr_1.VR.DS,
    '187050': vr_1.VR.CS,
    '187052': vr_1.VR.DS,
    '187054': vr_1.VR.DS,
    '187056': vr_1.VR.FL,
    '187058': vr_1.VR.FL,
    '187060': vr_1.VR.CS,
    '187062': vr_1.VR.LT,
    '187064': vr_1.VR.CS,
    '187065': vr_1.VR.DS,
    '188150': vr_1.VR.DS,
    '188151': vr_1.VR.DS,
    '189004': vr_1.VR.CS,
    '189005': vr_1.VR.SH,
    '189006': vr_1.VR.SQ,
    '189008': vr_1.VR.CS,
    '189009': vr_1.VR.CS,
    '189010': vr_1.VR.CS,
    '189011': vr_1.VR.CS,
    '189012': vr_1.VR.CS,
    '189014': vr_1.VR.CS,
    '189015': vr_1.VR.CS,
    '189016': vr_1.VR.CS,
    '189017': vr_1.VR.CS,
    '189018': vr_1.VR.CS,
    '189019': vr_1.VR.FD,
    '189020': vr_1.VR.CS,
    '189021': vr_1.VR.CS,
    '189022': vr_1.VR.CS,
    '189024': vr_1.VR.CS,
    '189025': vr_1.VR.CS,
    '189026': vr_1.VR.CS,
    '189027': vr_1.VR.CS,
    '189028': vr_1.VR.CS,
    '189029': vr_1.VR.CS,
    '189030': vr_1.VR.FD,
    '189032': vr_1.VR.CS,
    '189033': vr_1.VR.CS,
    '189034': vr_1.VR.CS,
    '189035': vr_1.VR.FD,
    '189036': vr_1.VR.CS,
    '189037': vr_1.VR.CS,
    '189041': vr_1.VR.LO,
    '189042': vr_1.VR.SQ,
    '189043': vr_1.VR.CS,
    '189044': vr_1.VR.CS,
    '189045': vr_1.VR.SQ,
    '189046': vr_1.VR.LO,
    '189047': vr_1.VR.SH,
    '189048': vr_1.VR.CS,
    '189049': vr_1.VR.SQ,
    '189050': vr_1.VR.LO,
    '189051': vr_1.VR.CS,
    '189052': vr_1.VR.FD,
    '189053': vr_1.VR.FD,
    '189054': vr_1.VR.CS,
    '189058': vr_1.VR.US,
    '189059': vr_1.VR.CS,
    '189060': vr_1.VR.CS,
    '189061': vr_1.VR.FD,
    '189062': vr_1.VR.CS,
    '189063': vr_1.VR.FD,
    '189064': vr_1.VR.CS,
    '189065': vr_1.VR.CS,
    '189066': vr_1.VR.US,
    '189067': vr_1.VR.CS,
    '189069': vr_1.VR.FD,
    '189070': vr_1.VR.FD,
    '189073': vr_1.VR.FD,
    '189074': vr_1.VR.DT,
    '189075': vr_1.VR.CS,
    '189076': vr_1.VR.SQ,
    '189077': vr_1.VR.CS,
    '189078': vr_1.VR.CS,
    '189079': vr_1.VR.FD,
    '189080': vr_1.VR.ST,
    '189081': vr_1.VR.CS,
    '189082': vr_1.VR.FD,
    '189083': vr_1.VR.SQ,
    '189084': vr_1.VR.SQ,
    '189085': vr_1.VR.CS,
    '189087': vr_1.VR.FD,
    '189089': vr_1.VR.FD,
    '189090': vr_1.VR.FD,
    '189091': vr_1.VR.FD,
    '189092': vr_1.VR.SQ,
    '189093': vr_1.VR.US,
    '189094': vr_1.VR.CS,
    '189095': vr_1.VR.UL,
    '189096': vr_1.VR.FD,
    '189098': vr_1.VR.FD,
    '189100': vr_1.VR.CS,
    '189101': vr_1.VR.CS,
    '189103': vr_1.VR.SQ,
    '189104': vr_1.VR.FD,
    '189105': vr_1.VR.FD,
    '189106': vr_1.VR.FD,
    '189107': vr_1.VR.SQ,
    '189112': vr_1.VR.SQ,
    '189114': vr_1.VR.SQ,
    '189115': vr_1.VR.SQ,
    '189117': vr_1.VR.SQ,
    '189118': vr_1.VR.SQ,
    '189119': vr_1.VR.SQ,
    '189125': vr_1.VR.SQ,
    '189126': vr_1.VR.SQ,
    '189127': vr_1.VR.UL,
    '189147': vr_1.VR.CS,
    '189151': vr_1.VR.DT,
    '189152': vr_1.VR.SQ,
    '189155': vr_1.VR.FD,
    '189159': vr_1.VR.UL,
    '189166': vr_1.VR.CS,
    '189168': vr_1.VR.FD,
    '189169': vr_1.VR.CS,
    '189170': vr_1.VR.CS,
    '189171': vr_1.VR.CS,
    '189172': vr_1.VR.CS,
    '189173': vr_1.VR.CS,
    '189174': vr_1.VR.CS,
    '189175': vr_1.VR.LO,
    '189176': vr_1.VR.SQ,
    '189177': vr_1.VR.CS,
    '189178': vr_1.VR.CS,
    '189179': vr_1.VR.CS,
    '189180': vr_1.VR.CS,
    '189181': vr_1.VR.FD,
    '189182': vr_1.VR.FD,
    '189183': vr_1.VR.CS,
    '189184': vr_1.VR.FD,
    '189185': vr_1.VR.ST,
    '189186': vr_1.VR.SH,
    '189195': vr_1.VR.FD,
    '189196': vr_1.VR.FD,
    '189197': vr_1.VR.SQ,
    '189198': vr_1.VR.CS,
    '189199': vr_1.VR.CS,
    '189200': vr_1.VR.CS,
    '189214': vr_1.VR.CS,
    '189217': vr_1.VR.FD,
    '189218': vr_1.VR.FD,
    '189219': vr_1.VR.SS,
    '189220': vr_1.VR.FD,
    '189226': vr_1.VR.SQ,
    '189227': vr_1.VR.SQ,
    '189231': vr_1.VR.US,
    '189232': vr_1.VR.US,
    '189234': vr_1.VR.UL,
    '189236': vr_1.VR.CS,
    '189239': vr_1.VR.SQ,
    '189240': vr_1.VR.US,
    '189241': vr_1.VR.US,
    '189250': vr_1.VR.CS,
    '189251': vr_1.VR.SQ,
    '189252': vr_1.VR.LO,
    '189253': vr_1.VR.US,
    '189254': vr_1.VR.FD,
    '189255': vr_1.VR.FD,
    '189256': vr_1.VR.FD,
    '189257': vr_1.VR.CS,
    '189258': vr_1.VR.UL,
    '189259': vr_1.VR.CS,
    '18925a': vr_1.VR.FD,
    '18925b': vr_1.VR.LO,
    '18925c': vr_1.VR.CS,
    '18925d': vr_1.VR.SQ,
    '18925e': vr_1.VR.LO,
    '18925f': vr_1.VR.UL,
    '189260': vr_1.VR.SQ,
    '189295': vr_1.VR.FD,
    '189296': vr_1.VR.FD,
    '189297': vr_1.VR.CS,
    '189298': vr_1.VR.IS,
    '189301': vr_1.VR.SQ,
    '189302': vr_1.VR.CS,
    '189303': vr_1.VR.FD,
    '189304': vr_1.VR.SQ,
    '189305': vr_1.VR.FD,
    '189306': vr_1.VR.FD,
    '189307': vr_1.VR.FD,
    '189308': vr_1.VR.SQ,
    '189309': vr_1.VR.FD,
    '189310': vr_1.VR.FD,
    '189311': vr_1.VR.FD,
    '189312': vr_1.VR.SQ,
    '189313': vr_1.VR.FD,
    '189314': vr_1.VR.SQ,
    '189315': vr_1.VR.CS,
    '189316': vr_1.VR.CS,
    '189317': vr_1.VR.FD,
    '189318': vr_1.VR.FD,
    '189319': vr_1.VR.FD,
    '189320': vr_1.VR.SH,
    '189321': vr_1.VR.SQ,
    '189322': vr_1.VR.FD,
    '189323': vr_1.VR.CS,
    '189324': vr_1.VR.FD,
    '189325': vr_1.VR.SQ,
    '189326': vr_1.VR.SQ,
    '189327': vr_1.VR.FD,
    '189328': vr_1.VR.FD,
    '189329': vr_1.VR.SQ,
    '189330': vr_1.VR.FD,
    '189332': vr_1.VR.FD,
    '189333': vr_1.VR.CS,
    '189334': vr_1.VR.CS,
    '189335': vr_1.VR.FD,
    '189337': vr_1.VR.US,
    '189338': vr_1.VR.SQ,
    '189340': vr_1.VR.SQ,
    '189341': vr_1.VR.SQ,
    '189342': vr_1.VR.CS,
    '189343': vr_1.VR.CS,
    '189344': vr_1.VR.CS,
    '189345': vr_1.VR.FD,
    '189346': vr_1.VR.SQ,
    '189351': vr_1.VR.FL,
    '189352': vr_1.VR.FL,
    '189353': vr_1.VR.FL,
    '189360': vr_1.VR.SQ,
    '189401': vr_1.VR.SQ,
    '189402': vr_1.VR.FL,
    '189403': vr_1.VR.FL,
    '189404': vr_1.VR.FL,
    '189405': vr_1.VR.SQ,
    '189406': vr_1.VR.SQ,
    '189407': vr_1.VR.SQ,
    '189410': vr_1.VR.CS,
    '189412': vr_1.VR.SQ,
    '189417': vr_1.VR.SQ,
    '189420': vr_1.VR.CS,
    '189423': vr_1.VR.LO,
    '189424': vr_1.VR.LT,
    '189425': vr_1.VR.CS,
    '189426': vr_1.VR.FL,
    '189427': vr_1.VR.CS,
    '189428': vr_1.VR.FL,
    '189429': vr_1.VR.FL,
    '189430': vr_1.VR.FL,
    '189432': vr_1.VR.SQ,
    '189433': vr_1.VR.LO,
    '189434': vr_1.VR.SQ,
    '189435': vr_1.VR.CS,
    '189436': vr_1.VR.SS,
    '189437': vr_1.VR.SS,
    '189438': vr_1.VR.SS,
    '189439': vr_1.VR.SS,
    '189440': vr_1.VR.SS,
    '189441': vr_1.VR.US,
    '189442': vr_1.VR.SS,
    '189447': vr_1.VR.FL,
    '189449': vr_1.VR.FL,
    '189451': vr_1.VR.SQ,
    '189452': vr_1.VR.FL,
    '189455': vr_1.VR.SQ,
    '189456': vr_1.VR.SQ,
    '189457': vr_1.VR.CS,
    '189461': vr_1.VR.FL,
    '189462': vr_1.VR.SQ,
    '189463': vr_1.VR.FL,
    '189464': vr_1.VR.FL,
    '189465': vr_1.VR.FL,
    '189466': vr_1.VR.FL,
    '189467': vr_1.VR.FL,
    '189468': vr_1.VR.FL,
    '189469': vr_1.VR.FL,
    '189470': vr_1.VR.FL,
    '189471': vr_1.VR.FL,
    '189472': vr_1.VR.SQ,
    '189473': vr_1.VR.FL,
    '189474': vr_1.VR.CS,
    '189476': vr_1.VR.SQ,
    '189477': vr_1.VR.SQ,
    '189504': vr_1.VR.SQ,
    '189506': vr_1.VR.SQ,
    '189507': vr_1.VR.SQ,
    '189508': vr_1.VR.FL,
    '189509': vr_1.VR.FL,
    '189510': vr_1.VR.FL,
    '189511': vr_1.VR.FL,
    '189514': vr_1.VR.FL,
    '189515': vr_1.VR.FL,
    '189516': vr_1.VR.DT,
    '189517': vr_1.VR.DT,
    '189518': vr_1.VR.SS,
    '189519': vr_1.VR.SS,
    '189524': vr_1.VR.LO,
    '189525': vr_1.VR.LO,
    '189526': vr_1.VR.LO,
    '189527': vr_1.VR.CS,
    '189528': vr_1.VR.LO,
    '189530': vr_1.VR.SQ,
    '189531': vr_1.VR.LO,
    '189538': vr_1.VR.SQ,
    '189541': vr_1.VR.SQ,
    '189542': vr_1.VR.SQ,
    '189543': vr_1.VR.FD,
    '189544': vr_1.VR.FD,
    '189545': vr_1.VR.FD,
    '189546': vr_1.VR.FD,
    '189547': vr_1.VR.FD,
    '189548': vr_1.VR.FD,
    '189549': vr_1.VR.FD,
    '189550': vr_1.VR.FD,
    '189551': vr_1.VR.FD,
    '189552': vr_1.VR.FD,
    '189553': vr_1.VR.FD,
    '189554': vr_1.VR.FD,
    '189555': vr_1.VR.SQ,
    '189556': vr_1.VR.SQ,
    '189557': vr_1.VR.FD,
    '189558': vr_1.VR.FD,
    '189559': vr_1.VR.CS,
    '189601': vr_1.VR.SQ,
    '189602': vr_1.VR.FD,
    '189603': vr_1.VR.FD,
    '189604': vr_1.VR.FD,
    '189605': vr_1.VR.FD,
    '189606': vr_1.VR.FD,
    '189607': vr_1.VR.FD,
    '189621': vr_1.VR.SQ,
    '189622': vr_1.VR.CS,
    '189623': vr_1.VR.DT,
    '189624': vr_1.VR.CS,
    '189701': vr_1.VR.DT,
    '189715': vr_1.VR.FD,
    '189716': vr_1.VR.FD,
    '189717': vr_1.VR.FD,
    '189718': vr_1.VR.FD,
    '189719': vr_1.VR.FD,
    '189720': vr_1.VR.FD,
    '189721': vr_1.VR.FD,
    '189722': vr_1.VR.FD,
    '189723': vr_1.VR.FD,
    '189724': vr_1.VR.FD,
    '189725': vr_1.VR.CS,
    '189726': vr_1.VR.FD,
    '189727': vr_1.VR.FD,
    '189729': vr_1.VR.US,
    '189732': vr_1.VR.SQ,
    '189733': vr_1.VR.SQ,
    '189734': vr_1.VR.SQ,
    '189735': vr_1.VR.SQ,
    '189736': vr_1.VR.SQ,
    '189737': vr_1.VR.SQ,
    '189738': vr_1.VR.CS,
    '189739': vr_1.VR.US,
    '189740': vr_1.VR.US,
    '189749': vr_1.VR.SQ,
    '189751': vr_1.VR.SQ,
    '189755': vr_1.VR.CS,
    '189756': vr_1.VR.CS,
    '189758': vr_1.VR.CS,
    '189759': vr_1.VR.CS,
    '189760': vr_1.VR.CS,
    '189761': vr_1.VR.CS,
    '189762': vr_1.VR.CS,
    '189763': vr_1.VR.CS,
    '189764': vr_1.VR.CS,
    '189765': vr_1.VR.CS,
    '189766': vr_1.VR.CS,
    '189767': vr_1.VR.CS,
    '189768': vr_1.VR.CS,
    '189769': vr_1.VR.CS,
    '189770': vr_1.VR.CS,
    '189771': vr_1.VR.SQ,
    '189772': vr_1.VR.SQ,
    '189801': vr_1.VR.FD,
    '189803': vr_1.VR.SQ,
    '189804': vr_1.VR.DT,
    '189805': vr_1.VR.FD,
    '189806': vr_1.VR.SQ,
    '189807': vr_1.VR.SQ,
    '189808': vr_1.VR.CS,
    '189809': vr_1.VR.SQ,
    '18980b': vr_1.VR.CS,
    '18980c': vr_1.VR.CS,
    '18980d': vr_1.VR.SQ,
    '18980e': vr_1.VR.SQ,
    '18980f': vr_1.VR.SQ,
    '189810': vr_1.VR.SS,
    '189900': vr_1.VR.LO,
    '189901': vr_1.VR.UT,
    '189902': vr_1.VR.SQ,
    '189903': vr_1.VR.SQ,
    '189904': vr_1.VR.DS,
    '189905': vr_1.VR.CS,
    '189906': vr_1.VR.SQ,
    '189907': vr_1.VR.SQ,
    '189908': vr_1.VR.UC,
    '189909': vr_1.VR.SQ,
    '18990a': vr_1.VR.UC,
    '18990b': vr_1.VR.SQ,
    '18990c': vr_1.VR.SQ,
    '18990d': vr_1.VR.SQ,
    '18990e': vr_1.VR.SQ,
    '18990f': vr_1.VR.UT,
    '189910': vr_1.VR.UT,
    '189911': vr_1.VR.SQ,
    '189912': vr_1.VR.SQ,
    '189913': vr_1.VR.SQ,
    '189914': vr_1.VR.SQ,
    '189915': vr_1.VR.US,
    '189916': vr_1.VR.LO,
    '189917': vr_1.VR.UT,
    '189918': vr_1.VR.CS,
    '189919': vr_1.VR.DT,
    '18991a': vr_1.VR.UT,
    '18991b': vr_1.VR.SQ,
    '18991c': vr_1.VR.SQ,
    '18991d': vr_1.VR.SQ,
    '18991e': vr_1.VR.UI,
    '18991f': vr_1.VR.SQ,
    '189920': vr_1.VR.SQ,
    '189921': vr_1.VR.US,
    '189922': vr_1.VR.LO,
    '189923': vr_1.VR.UT,
    '189924': vr_1.VR.UT,
    '189930': vr_1.VR.CS,
    '189931': vr_1.VR.SQ,
    '189932': vr_1.VR.SQ,
    '189933': vr_1.VR.SQ,
    '189934': vr_1.VR.SQ,
    '189935': vr_1.VR.SQ,
    '189936': vr_1.VR.SQ,
    '189937': vr_1.VR.LO,
    '189938': vr_1.VR.US,
    '189939': vr_1.VR.US,
    '18993a': vr_1.VR.US,
    '18993b': vr_1.VR.SQ,
    '18993c': vr_1.VR.SQ,
    '18993d': vr_1.VR.SQ,
    '18993e': vr_1.VR.SQ,
    '189941': vr_1.VR.UT,
    '189942': vr_1.VR.FD,
    '189943': vr_1.VR.FD,
    '189944': vr_1.VR.CS,
    '189945': vr_1.VR.FD,
    '189946': vr_1.VR.FD,
    '189947': vr_1.VR.CS,
    '18a001': vr_1.VR.SQ,
    '18a002': vr_1.VR.DT,
    '18a003': vr_1.VR.ST,
    '20000d': vr_1.VR.UI,
    '20000e': vr_1.VR.UI,
    '200010': vr_1.VR.SH,
    '200011': vr_1.VR.IS,
    '200012': vr_1.VR.IS,
    '200013': vr_1.VR.IS,
    '200014': vr_1.VR.IS,
    '200015': vr_1.VR.IS,
    '200016': vr_1.VR.IS,
    '200017': vr_1.VR.IS,
    '200018': vr_1.VR.IS,
    '200019': vr_1.VR.IS,
    '200020': vr_1.VR.CS,
    '200022': vr_1.VR.IS,
    '200024': vr_1.VR.IS,
    '200026': vr_1.VR.IS,
    '200030': vr_1.VR.DS,
    '200032': vr_1.VR.DS,
    '200035': vr_1.VR.DS,
    '200037': vr_1.VR.DS,
    '200050': vr_1.VR.DS,
    '200052': vr_1.VR.UI,
    '200060': vr_1.VR.CS,
    '200062': vr_1.VR.CS,
    '200070': vr_1.VR.LO,
    '200080': vr_1.VR.CS,
    '2000aa': vr_1.VR.IS,
    '200100': vr_1.VR.IS,
    '200105': vr_1.VR.IS,
    '200110': vr_1.VR.DS,
    '200200': vr_1.VR.UI,
    '200242': vr_1.VR.UI,
    '201000': vr_1.VR.IS,
    '201001': vr_1.VR.IS,
    '201002': vr_1.VR.IS,
    '201003': vr_1.VR.IS,
    '201004': vr_1.VR.IS,
    '201005': vr_1.VR.IS,
    '201020': vr_1.VR.LO,
    '20103f': vr_1.VR.LO,
    '201040': vr_1.VR.LO,
    '201041': vr_1.VR.DS,
    '201070': vr_1.VR.IS,
    '201200': vr_1.VR.IS,
    '201202': vr_1.VR.IS,
    '201204': vr_1.VR.IS,
    '201206': vr_1.VR.IS,
    '201208': vr_1.VR.IS,
    '201209': vr_1.VR.IS,
    '203100': vr_1.VR.CS,
    '203401': vr_1.VR.CS,
    '203402': vr_1.VR.CS,
    '203403': vr_1.VR.DA,
    '203404': vr_1.VR.LO,
    '203405': vr_1.VR.TM,
    '203406': vr_1.VR.LO,
    '204000': vr_1.VR.LT,
    '205000': vr_1.VR.AT,
    '205002': vr_1.VR.LO,
    '209056': vr_1.VR.SH,
    '209057': vr_1.VR.UL,
    '209071': vr_1.VR.SQ,
    '209072': vr_1.VR.CS,
    '209111': vr_1.VR.SQ,
    '209113': vr_1.VR.SQ,
    '209116': vr_1.VR.SQ,
    '209128': vr_1.VR.UL,
    '209153': vr_1.VR.FD,
    '209154': vr_1.VR.FL,
    '209155': vr_1.VR.FL,
    '209156': vr_1.VR.US,
    '209157': vr_1.VR.UL,
    '209158': vr_1.VR.LT,
    '209161': vr_1.VR.UI,
    '209162': vr_1.VR.US,
    '209163': vr_1.VR.US,
    '209164': vr_1.VR.UI,
    '209165': vr_1.VR.AT,
    '209167': vr_1.VR.AT,
    '209170': vr_1.VR.SQ,
    '209171': vr_1.VR.SQ,
    '209172': vr_1.VR.SQ,
    '209213': vr_1.VR.LO,
    '209221': vr_1.VR.SQ,
    '209222': vr_1.VR.SQ,
    '209228': vr_1.VR.UL,
    '209238': vr_1.VR.LO,
    '209241': vr_1.VR.FL,
    '209245': vr_1.VR.FL,
    '209246': vr_1.VR.FL,
    '209247': vr_1.VR.CS,
    '209248': vr_1.VR.FL,
    '209249': vr_1.VR.CS,
    '209250': vr_1.VR.CS,
    '209251': vr_1.VR.FD,
    '209252': vr_1.VR.FD,
    '209253': vr_1.VR.SQ,
    '209254': vr_1.VR.FD,
    '209255': vr_1.VR.FD,
    '209256': vr_1.VR.FD,
    '209257': vr_1.VR.FD,
    '209301': vr_1.VR.FD,
    '209302': vr_1.VR.FD,
    '209307': vr_1.VR.CS,
    '209308': vr_1.VR.FD,
    '209309': vr_1.VR.FD,
    '20930a': vr_1.VR.FD,
    '20930b': vr_1.VR.CS,
    '20930c': vr_1.VR.CS,
    '20930d': vr_1.VR.FD,
    '20930e': vr_1.VR.SQ,
    '20930f': vr_1.VR.SQ,
    '209310': vr_1.VR.SQ,
    '209311': vr_1.VR.CS,
    '209312': vr_1.VR.UI,
    '209313': vr_1.VR.UI,
    '209421': vr_1.VR.LO,
    '209450': vr_1.VR.SQ,
    '209453': vr_1.VR.LO,
    '209518': vr_1.VR.US,
    '209529': vr_1.VR.SQ,
    '209536': vr_1.VR.US,
    '220001': vr_1.VR.US,
    '220002': vr_1.VR.US,
    '220003': vr_1.VR.US,
    '220004': vr_1.VR.US,
    '220005': vr_1.VR.CS,
    '220006': vr_1.VR.SQ,
    '220007': vr_1.VR.FL,
    '220008': vr_1.VR.FL,
    '220009': vr_1.VR.FL,
    '22000a': vr_1.VR.FL,
    '22000b': vr_1.VR.FL,
    '22000c': vr_1.VR.FL,
    '22000d': vr_1.VR.CS,
    '22000e': vr_1.VR.FL,
    '220010': vr_1.VR.FL,
    '220011': vr_1.VR.FL,
    '220012': vr_1.VR.FL,
    '220013': vr_1.VR.FL,
    '220014': vr_1.VR.FL,
    '220015': vr_1.VR.SQ,
    '220016': vr_1.VR.SQ,
    '220017': vr_1.VR.SQ,
    '220018': vr_1.VR.SQ,
    '220019': vr_1.VR.SQ,
    '22001a': vr_1.VR.SQ,
    '22001b': vr_1.VR.SQ,
    '22001c': vr_1.VR.SQ,
    '22001d': vr_1.VR.SQ,
    '22001e': vr_1.VR.FL,
    '220020': vr_1.VR.SQ,
    '220021': vr_1.VR.SQ,
    '220022': vr_1.VR.SQ,
    '220028': vr_1.VR.CS,
    '220030': vr_1.VR.FL,
    '220031': vr_1.VR.SQ,
    '220032': vr_1.VR.FL,
    '220035': vr_1.VR.FL,
    '220036': vr_1.VR.FL,
    '220037': vr_1.VR.FL,
    '220038': vr_1.VR.FL,
    '220039': vr_1.VR.CS,
    '220041': vr_1.VR.FL,
    '220042': vr_1.VR.SQ,
    '220048': vr_1.VR.FL,
    '220049': vr_1.VR.FL,
    '22004e': vr_1.VR.DS,
    '220055': vr_1.VR.FL,
    '220056': vr_1.VR.FL,
    '220057': vr_1.VR.FL,
    '220058': vr_1.VR.SQ,
    '221007': vr_1.VR.SQ,
    '221008': vr_1.VR.SQ,
    '221009': vr_1.VR.CS,
    '221010': vr_1.VR.CS,
    '221012': vr_1.VR.SQ,
    '221019': vr_1.VR.FL,
    '221024': vr_1.VR.SQ,
    '221025': vr_1.VR.SQ,
    '221028': vr_1.VR.SQ,
    '221029': vr_1.VR.LO,
    '221033': vr_1.VR.FL,
    '221035': vr_1.VR.SQ,
    '221037': vr_1.VR.FL,
    '221039': vr_1.VR.CS,
    '221040': vr_1.VR.SQ,
    '221044': vr_1.VR.SQ,
    '221050': vr_1.VR.SQ,
    '221053': vr_1.VR.FL,
    '221054': vr_1.VR.FL,
    '221059': vr_1.VR.FL,
    '221065': vr_1.VR.LO,
    '221066': vr_1.VR.LO,
    '221090': vr_1.VR.SQ,
    '221092': vr_1.VR.SQ,
    '221093': vr_1.VR.LO,
    '221094': vr_1.VR.LO,
    '221095': vr_1.VR.LO,
    '221096': vr_1.VR.SQ,
    '221097': vr_1.VR.LO,
    '221100': vr_1.VR.SQ,
    '221101': vr_1.VR.SQ,
    '221103': vr_1.VR.SQ,
    '221121': vr_1.VR.FL,
    '221122': vr_1.VR.FL,
    '221125': vr_1.VR.SQ,
    '221127': vr_1.VR.SQ,
    '221128': vr_1.VR.SQ,
    '221130': vr_1.VR.FL,
    '221131': vr_1.VR.FL,
    '221132': vr_1.VR.SQ,
    '221133': vr_1.VR.SQ,
    '221134': vr_1.VR.SQ,
    '221135': vr_1.VR.SQ,
    '221140': vr_1.VR.CS,
    '221150': vr_1.VR.SQ,
    '221153': vr_1.VR.SQ,
    '221155': vr_1.VR.FL,
    '221159': vr_1.VR.LO,
    '221210': vr_1.VR.SQ,
    '221211': vr_1.VR.SQ,
    '221212': vr_1.VR.SQ,
    '221220': vr_1.VR.SQ,
    '221225': vr_1.VR.SQ,
    '221230': vr_1.VR.SQ,
    '221250': vr_1.VR.SQ,
    '221255': vr_1.VR.SQ,
    '221257': vr_1.VR.SQ,
    '221260': vr_1.VR.SQ,
    '221262': vr_1.VR.SQ,
    '221265': vr_1.VR.SQ,
    '221273': vr_1.VR.LO,
    '221300': vr_1.VR.SQ,
    '221310': vr_1.VR.SQ,
    '221330': vr_1.VR.SQ,
    '221415': vr_1.VR.CS,
    '221420': vr_1.VR.SQ,
    '221423': vr_1.VR.SQ,
    '221436': vr_1.VR.SQ,
    '221443': vr_1.VR.SQ,
    '221445': vr_1.VR.SQ,
    '221450': vr_1.VR.SQ,
    '221452': vr_1.VR.SS,
    '221454': vr_1.VR.LO,
    '221458': vr_1.VR.SQ,
    '221460': vr_1.VR.FL,
    '221463': vr_1.VR.FL,
    '221465': vr_1.VR.SQ,
    '221466': vr_1.VR.CS,
    '221467': vr_1.VR.FL,
    '221468': vr_1.VR.FL,
    '221470': vr_1.VR.SQ,
    '221472': vr_1.VR.SQ,
    '221512': vr_1.VR.SQ,
    '221513': vr_1.VR.SQ,
    '221515': vr_1.VR.CS,
    '221517': vr_1.VR.FL,
    '221518': vr_1.VR.SQ,
    '221525': vr_1.VR.SQ,
    '221526': vr_1.VR.SQ,
    '221527': vr_1.VR.FL,
    '221528': vr_1.VR.FL,
    '221529': vr_1.VR.FL,
    '221530': vr_1.VR.UL,
    '221531': vr_1.VR.OF,
    '221612': vr_1.VR.SQ,
    '221615': vr_1.VR.SQ,
    '221616': vr_1.VR.LO,
    '221618': vr_1.VR.SQ,
    '221620': vr_1.VR.SQ,
    '221622': vr_1.VR.CS,
    '221624': vr_1.VR.FL,
    '221626': vr_1.VR.FL,
    '221628': vr_1.VR.SQ,
    '221630': vr_1.VR.DS,
    '221640': vr_1.VR.SQ,
    '221642': vr_1.VR.UL,
    '221643': vr_1.VR.FL,
    '221644': vr_1.VR.FL,
    '221645': vr_1.VR.FL,
    '221646': vr_1.VR.FL,
    '221649': vr_1.VR.FL,
    '221650': vr_1.VR.FL,
    '221658': vr_1.VR.UL,
    '240010': vr_1.VR.FL,
    '240011': vr_1.VR.FL,
    '240012': vr_1.VR.CS,
    '240016': vr_1.VR.SQ,
    '240018': vr_1.VR.FL,
    '240020': vr_1.VR.FL,
    '240021': vr_1.VR.SQ,
    '240024': vr_1.VR.SQ,
    '240025': vr_1.VR.FL,
    '240028': vr_1.VR.FL,
    '240032': vr_1.VR.SQ,
    '240033': vr_1.VR.SQ,
    '240034': vr_1.VR.SQ,
    '240035': vr_1.VR.US,
    '240036': vr_1.VR.US,
    '240037': vr_1.VR.CS,
    '240038': vr_1.VR.US,
    '240039': vr_1.VR.CS,
    '240040': vr_1.VR.CS,
    '240042': vr_1.VR.US,
    '240044': vr_1.VR.LT,
    '240045': vr_1.VR.CS,
    '240046': vr_1.VR.FL,
    '240048': vr_1.VR.US,
    '240050': vr_1.VR.US,
    '240051': vr_1.VR.CS,
    '240052': vr_1.VR.CS,
    '240053': vr_1.VR.CS,
    '240054': vr_1.VR.FL,
    '240055': vr_1.VR.CS,
    '240056': vr_1.VR.US,
    '240057': vr_1.VR.CS,
    '240058': vr_1.VR.SQ,
    '240059': vr_1.VR.CS,
    '240060': vr_1.VR.US,
    '240061': vr_1.VR.CS,
    '240062': vr_1.VR.CS,
    '240063': vr_1.VR.CS,
    '240064': vr_1.VR.SQ,
    '240065': vr_1.VR.SQ,
    '240066': vr_1.VR.FL,
    '240067': vr_1.VR.SQ,
    '240068': vr_1.VR.FL,
    '240069': vr_1.VR.LO,
    '240070': vr_1.VR.FL,
    '240071': vr_1.VR.FL,
    '240072': vr_1.VR.CS,
    '240073': vr_1.VR.FL,
    '240074': vr_1.VR.CS,
    '240075': vr_1.VR.FL,
    '240076': vr_1.VR.CS,
    '240077': vr_1.VR.FL,
    '240078': vr_1.VR.CS,
    '240079': vr_1.VR.FL,
    '240080': vr_1.VR.CS,
    '240081': vr_1.VR.FL,
    '240083': vr_1.VR.SQ,
    '240085': vr_1.VR.SQ,
    '240086': vr_1.VR.CS,
    '240087': vr_1.VR.FL,
    '240088': vr_1.VR.FL,
    '240089': vr_1.VR.SQ,
    '240090': vr_1.VR.FL,
    '240091': vr_1.VR.FL,
    '240092': vr_1.VR.FL,
    '240093': vr_1.VR.CS,
    '240094': vr_1.VR.FL,
    '240095': vr_1.VR.CS,
    '240096': vr_1.VR.FL,
    '240097': vr_1.VR.SQ,
    '240098': vr_1.VR.FL,
    '240100': vr_1.VR.FL,
    '240102': vr_1.VR.CS,
    '240103': vr_1.VR.FL,
    '240104': vr_1.VR.FL,
    '240105': vr_1.VR.FL,
    '240106': vr_1.VR.CS,
    '240107': vr_1.VR.FL,
    '240108': vr_1.VR.FL,
    '240110': vr_1.VR.SQ,
    '240112': vr_1.VR.SQ,
    '240113': vr_1.VR.CS,
    '240114': vr_1.VR.SQ,
    '240115': vr_1.VR.SQ,
    '240117': vr_1.VR.CS,
    '240118': vr_1.VR.FL,
    '240120': vr_1.VR.CS,
    '240122': vr_1.VR.SQ,
    '240124': vr_1.VR.CS,
    '240126': vr_1.VR.FL,
    '240202': vr_1.VR.LO,
    '240306': vr_1.VR.LO,
    '240307': vr_1.VR.LO,
    '240308': vr_1.VR.LO,
    '240309': vr_1.VR.LO,
    '240317': vr_1.VR.SQ,
    '240320': vr_1.VR.SQ,
    '240325': vr_1.VR.SQ,
    '240338': vr_1.VR.CS,
    '240341': vr_1.VR.FL,
    '240344': vr_1.VR.SQ,
    '280002': vr_1.VR.US,
    '280003': vr_1.VR.US,
    '280004': vr_1.VR.CS,
    '280005': vr_1.VR.US,
    '280006': vr_1.VR.US,
    '280008': vr_1.VR.IS,
    '280009': vr_1.VR.AT,
    '28000a': vr_1.VR.AT,
    '280010': vr_1.VR.US,
    '280011': vr_1.VR.US,
    '280012': vr_1.VR.US,
    '280014': vr_1.VR.US,
    '280030': vr_1.VR.DS,
    '280031': vr_1.VR.DS,
    '280032': vr_1.VR.DS,
    '280034': vr_1.VR.IS,
    '280040': vr_1.VR.CS,
    '280050': vr_1.VR.LO,
    '280051': vr_1.VR.CS,
    '28005f': vr_1.VR.LO,
    '280060': vr_1.VR.CS,
    '280061': vr_1.VR.SH,
    '280062': vr_1.VR.LO,
    '280063': vr_1.VR.SH,
    '280065': vr_1.VR.CS,
    '280066': vr_1.VR.AT,
    '280068': vr_1.VR.US,
    '280069': vr_1.VR.US,
    '280070': vr_1.VR.US,
    '280071': vr_1.VR.SS,
    '280080': vr_1.VR.US,
    '280081': vr_1.VR.US,
    '280082': vr_1.VR.US,
    '280090': vr_1.VR.CS,
    '280091': vr_1.VR.US,
    '280092': vr_1.VR.US,
    '280093': vr_1.VR.US,
    '280094': vr_1.VR.US,
    '280100': vr_1.VR.US,
    '280101': vr_1.VR.US,
    '280102': vr_1.VR.US,
    '280103': vr_1.VR.US,
    '280104': vr_1.VR.SS,
    '280105': vr_1.VR.SS,
    '280106': vr_1.VR.SS,
    '280107': vr_1.VR.SS,
    '280108': vr_1.VR.SS,
    '280109': vr_1.VR.SS,
    '280110': vr_1.VR.SS,
    '280111': vr_1.VR.SS,
    '280120': vr_1.VR.SS,
    '280121': vr_1.VR.SS,
    '280122': vr_1.VR.FL,
    '280123': vr_1.VR.FD,
    '280124': vr_1.VR.FL,
    '280125': vr_1.VR.FD,
    '280200': vr_1.VR.US,
    '280300': vr_1.VR.CS,
    '280301': vr_1.VR.CS,
    '280302': vr_1.VR.CS,
    '280303': vr_1.VR.CS,
    '280304': vr_1.VR.UI,
    '280400': vr_1.VR.LO,
    '280401': vr_1.VR.LO,
    '280402': vr_1.VR.US,
    '280403': vr_1.VR.LO,
    '280404': vr_1.VR.AT,
    '280700': vr_1.VR.LO,
    '280701': vr_1.VR.CS,
    '280702': vr_1.VR.AT,
    '280710': vr_1.VR.US,
    '280720': vr_1.VR.US,
    '280721': vr_1.VR.AT,
    '280722': vr_1.VR.US,
    '280730': vr_1.VR.US,
    '280740': vr_1.VR.US,
    '280800': vr_1.VR.CS,
    '280802': vr_1.VR.US,
    '280803': vr_1.VR.AT,
    '280804': vr_1.VR.US,
    '280808': vr_1.VR.AT,
    '280a02': vr_1.VR.CS,
    '280a04': vr_1.VR.LO,
    '281040': vr_1.VR.CS,
    '281041': vr_1.VR.SS,
    '281050': vr_1.VR.DS,
    '281051': vr_1.VR.DS,
    '281052': vr_1.VR.DS,
    '281053': vr_1.VR.DS,
    '281054': vr_1.VR.LO,
    '281055': vr_1.VR.LO,
    '281056': vr_1.VR.CS,
    '281080': vr_1.VR.CS,
    '281090': vr_1.VR.CS,
    '281100': vr_1.VR.SS,
    '281101': vr_1.VR.SS,
    '281102': vr_1.VR.SS,
    '281103': vr_1.VR.SS,
    '281104': vr_1.VR.US,
    '281111': vr_1.VR.SS,
    '281112': vr_1.VR.SS,
    '281113': vr_1.VR.SS,
    '281199': vr_1.VR.UI,
    '281200': vr_1.VR.OW,
    '281201': vr_1.VR.OW,
    '281202': vr_1.VR.OW,
    '281203': vr_1.VR.OW,
    '281204': vr_1.VR.OW,
    '281211': vr_1.VR.OW,
    '281212': vr_1.VR.OW,
    '281213': vr_1.VR.OW,
    '281214': vr_1.VR.UI,
    '281221': vr_1.VR.OW,
    '281222': vr_1.VR.OW,
    '281223': vr_1.VR.OW,
    '281224': vr_1.VR.OW,
    '281230': vr_1.VR.SQ,
    '281231': vr_1.VR.FD,
    '281232': vr_1.VR.FD,
    '281300': vr_1.VR.CS,
    '281350': vr_1.VR.CS,
    '281351': vr_1.VR.ST,
    '281352': vr_1.VR.SQ,
    '28135a': vr_1.VR.CS,
    '281401': vr_1.VR.SQ,
    '281402': vr_1.VR.CS,
    '281403': vr_1.VR.US,
    '281404': vr_1.VR.SQ,
    '281405': vr_1.VR.CS,
    '281406': vr_1.VR.FD,
    '281407': vr_1.VR.US,
    '281408': vr_1.VR.OW,
    '28140b': vr_1.VR.SQ,
    '28140c': vr_1.VR.SQ,
    '28140d': vr_1.VR.CS,
    '28140e': vr_1.VR.CS,
    '28140f': vr_1.VR.CS,
    '281410': vr_1.VR.CS,
    '282000': vr_1.VR.OB,
    '282002': vr_1.VR.CS,
    '282110': vr_1.VR.CS,
    '282112': vr_1.VR.DS,
    '282114': vr_1.VR.CS,
    '283000': vr_1.VR.SQ,
    '283002': vr_1.VR.SS,
    '283003': vr_1.VR.LO,
    '283004': vr_1.VR.LO,
    '283006': vr_1.VR.OW,
    '283010': vr_1.VR.SQ,
    '283110': vr_1.VR.SQ,
    '284000': vr_1.VR.LT,
    '285000': vr_1.VR.SQ,
    '286010': vr_1.VR.US,
    '286020': vr_1.VR.US,
    '286022': vr_1.VR.LO,
    '286023': vr_1.VR.CS,
    '286030': vr_1.VR.US,
    '286040': vr_1.VR.US,
    '286100': vr_1.VR.SQ,
    '286101': vr_1.VR.CS,
    '286102': vr_1.VR.US,
    '286110': vr_1.VR.US,
    '286112': vr_1.VR.US,
    '286114': vr_1.VR.FL,
    '286120': vr_1.VR.SS,
    '286190': vr_1.VR.ST,
    '287000': vr_1.VR.SQ,
    '287001': vr_1.VR.US,
    '287002': vr_1.VR.US,
    '287003': vr_1.VR.US,
    '287004': vr_1.VR.SH,
    '287005': vr_1.VR.LO,
    '287006': vr_1.VR.CS,
    '287007': vr_1.VR.LO,
    '287008': vr_1.VR.SQ,
    '287009': vr_1.VR.US,
    '28700a': vr_1.VR.SQ,
    '28700b': vr_1.VR.US,
    '28700c': vr_1.VR.SH,
    '28700d': vr_1.VR.LO,
    '28700e': vr_1.VR.US,
    '28700f': vr_1.VR.SQ,
    '287010': vr_1.VR.SQ,
    '287011': vr_1.VR.SQ,
    '287012': vr_1.VR.SQ,
    '287013': vr_1.VR.CS,
    '287014': vr_1.VR.CS,
    '287015': vr_1.VR.SQ,
    '287016': vr_1.VR.SQ,
    '287017': vr_1.VR.US,
    '287018': vr_1.VR.FL,
    '287019': vr_1.VR.CS,
    '28701a': vr_1.VR.FL,
    '28701b': vr_1.VR.US,
    '28701c': vr_1.VR.SQ,
    '28701d': vr_1.VR.FL,
    '28701e': vr_1.VR.FL,
    '28701f': vr_1.VR.FL,
    '287020': vr_1.VR.LO,
    '287021': vr_1.VR.CS,
    '287022': vr_1.VR.SQ,
    '287023': vr_1.VR.SQ,
    '287024': vr_1.VR.SQ,
    '287025': vr_1.VR.CS,
    '287026': vr_1.VR.CS,
    '287027': vr_1.VR.SQ,
    '287028': vr_1.VR.SQ,
    '287029': vr_1.VR.CS,
    '28702a': vr_1.VR.LO,
    '28702b': vr_1.VR.CS,
    '28702c': vr_1.VR.SQ,
    '28702d': vr_1.VR.SQ,
    '28702e': vr_1.VR.SQ,
    '287fe0': vr_1.VR.UR,
    '289001': vr_1.VR.UL,
    '289002': vr_1.VR.UL,
    '289003': vr_1.VR.CS,
    '289099': vr_1.VR.US,
    '289108': vr_1.VR.CS,
    '289110': vr_1.VR.SQ,
    '289132': vr_1.VR.SQ,
    '289145': vr_1.VR.SQ,
    '289235': vr_1.VR.CS,
    '289411': vr_1.VR.FL,
    '289415': vr_1.VR.SQ,
    '289416': vr_1.VR.US,
    '289422': vr_1.VR.SQ,
    '289443': vr_1.VR.SQ,
    '289444': vr_1.VR.CS,
    '289445': vr_1.VR.FL,
    '289446': vr_1.VR.CS,
    '289454': vr_1.VR.CS,
    '289474': vr_1.VR.CS,
    '289478': vr_1.VR.FL,
    '289501': vr_1.VR.SQ,
    '289502': vr_1.VR.SQ,
    '289503': vr_1.VR.SS,
    '289505': vr_1.VR.SQ,
    '289506': vr_1.VR.US,
    '289507': vr_1.VR.US,
    '289520': vr_1.VR.DS,
    '289537': vr_1.VR.CS,
    '32000a': vr_1.VR.CS,
    '32000c': vr_1.VR.CS,
    '320012': vr_1.VR.LO,
    '320032': vr_1.VR.DA,
    '320033': vr_1.VR.TM,
    '320034': vr_1.VR.DA,
    '320035': vr_1.VR.TM,
    '321000': vr_1.VR.DA,
    '321001': vr_1.VR.TM,
    '321010': vr_1.VR.DA,
    '321011': vr_1.VR.TM,
    '321020': vr_1.VR.LO,
    '321021': vr_1.VR.AE,
    '321030': vr_1.VR.LO,
    '321031': vr_1.VR.SQ,
    '321032': vr_1.VR.PN,
    '321033': vr_1.VR.LO,
    '321034': vr_1.VR.SQ,
    '321040': vr_1.VR.DA,
    '321041': vr_1.VR.TM,
    '321050': vr_1.VR.DA,
    '321051': vr_1.VR.TM,
    '321055': vr_1.VR.CS,
    '321060': vr_1.VR.LO,
    '321064': vr_1.VR.SQ,
    '321070': vr_1.VR.LO,
    '324000': vr_1.VR.LT,
    '380004': vr_1.VR.SQ,
    '380008': vr_1.VR.CS,
    '380010': vr_1.VR.LO,
    '380011': vr_1.VR.LO,
    '380014': vr_1.VR.SQ,
    '380016': vr_1.VR.LO,
    '38001a': vr_1.VR.DA,
    '38001b': vr_1.VR.TM,
    '38001c': vr_1.VR.DA,
    '38001d': vr_1.VR.TM,
    '38001e': vr_1.VR.LO,
    '380020': vr_1.VR.DA,
    '380021': vr_1.VR.TM,
    '380030': vr_1.VR.DA,
    '380032': vr_1.VR.TM,
    '380040': vr_1.VR.LO,
    '380044': vr_1.VR.SQ,
    '380050': vr_1.VR.LO,
    '380060': vr_1.VR.LO,
    '380061': vr_1.VR.LO,
    '380062': vr_1.VR.LO,
    '380064': vr_1.VR.SQ,
    '380100': vr_1.VR.SQ,
    '380101': vr_1.VR.SQ,
    '380102': vr_1.VR.LO,
    '380300': vr_1.VR.LO,
    '380400': vr_1.VR.LO,
    '380500': vr_1.VR.LO,
    '380502': vr_1.VR.SQ,
    '384000': vr_1.VR.LT,
    '3a0004': vr_1.VR.CS,
    '3a0005': vr_1.VR.US,
    '3a0010': vr_1.VR.UL,
    '3a001a': vr_1.VR.DS,
    '3a0020': vr_1.VR.SH,
    '3a0200': vr_1.VR.SQ,
    '3a0202': vr_1.VR.IS,
    '3a0203': vr_1.VR.SH,
    '3a0205': vr_1.VR.CS,
    '3a0208': vr_1.VR.SQ,
    '3a0209': vr_1.VR.SQ,
    '3a020a': vr_1.VR.SQ,
    '3a020c': vr_1.VR.LO,
    '3a0210': vr_1.VR.DS,
    '3a0211': vr_1.VR.SQ,
    '3a0212': vr_1.VR.DS,
    '3a0213': vr_1.VR.DS,
    '3a0214': vr_1.VR.DS,
    '3a0215': vr_1.VR.DS,
    '3a0218': vr_1.VR.DS,
    '3a021a': vr_1.VR.US,
    '3a0220': vr_1.VR.DS,
    '3a0221': vr_1.VR.DS,
    '3a0222': vr_1.VR.DS,
    '3a0223': vr_1.VR.DS,
    '3a0230': vr_1.VR.FL,
    '3a0231': vr_1.VR.US,
    '3a0240': vr_1.VR.SQ,
    '3a0241': vr_1.VR.US,
    '3a0242': vr_1.VR.SQ,
    '3a0244': vr_1.VR.US,
    '3a0245': vr_1.VR.FL,
    '3a0246': vr_1.VR.CS,
    '3a0247': vr_1.VR.FL,
    '3a0248': vr_1.VR.FL,
    '3a0300': vr_1.VR.SQ,
    '3a0301': vr_1.VR.IS,
    '3a0302': vr_1.VR.CS,
    '400001': vr_1.VR.AE,
    '400002': vr_1.VR.DA,
    '400003': vr_1.VR.TM,
    '400004': vr_1.VR.DA,
    '400005': vr_1.VR.TM,
    '400006': vr_1.VR.PN,
    '400007': vr_1.VR.LO,
    '400008': vr_1.VR.SQ,
    '400009': vr_1.VR.SH,
    '40000a': vr_1.VR.SQ,
    '40000b': vr_1.VR.SQ,
    '400010': vr_1.VR.SH,
    '400011': vr_1.VR.SH,
    '400012': vr_1.VR.LO,
    '400020': vr_1.VR.CS,
    '400026': vr_1.VR.SQ,
    '400027': vr_1.VR.SQ,
    '400031': vr_1.VR.UT,
    '400032': vr_1.VR.UT,
    '400033': vr_1.VR.CS,
    '400035': vr_1.VR.CS,
    '400036': vr_1.VR.SQ,
    '400039': vr_1.VR.SQ,
    '40003a': vr_1.VR.SQ,
    '400100': vr_1.VR.SQ,
    '400220': vr_1.VR.SQ,
    '400241': vr_1.VR.AE,
    '400242': vr_1.VR.SH,
    '400243': vr_1.VR.SH,
    '400244': vr_1.VR.DA,
    '400245': vr_1.VR.TM,
    '400250': vr_1.VR.DA,
    '400251': vr_1.VR.TM,
    '400252': vr_1.VR.CS,
    '400253': vr_1.VR.SH,
    '400254': vr_1.VR.LO,
    '400255': vr_1.VR.LO,
    '400260': vr_1.VR.SQ,
    '400261': vr_1.VR.CS,
    '400270': vr_1.VR.SQ,
    '400275': vr_1.VR.SQ,
    '400280': vr_1.VR.ST,
    '400281': vr_1.VR.SQ,
    '400293': vr_1.VR.SQ,
    '400294': vr_1.VR.DS,
    '400295': vr_1.VR.SQ,
    '400296': vr_1.VR.SQ,
    '400300': vr_1.VR.US,
    '400301': vr_1.VR.US,
    '400302': vr_1.VR.US,
    '400303': vr_1.VR.US,
    '400306': vr_1.VR.DS,
    '400307': vr_1.VR.DS,
    '40030e': vr_1.VR.SQ,
    '400310': vr_1.VR.ST,
    '400312': vr_1.VR.DS,
    '400314': vr_1.VR.DS,
    '400316': vr_1.VR.DS,
    '400318': vr_1.VR.CS,
    '400320': vr_1.VR.SQ,
    '400321': vr_1.VR.SQ,
    '400324': vr_1.VR.SQ,
    '400330': vr_1.VR.SQ,
    '400340': vr_1.VR.SQ,
    '400400': vr_1.VR.LT,
    '400440': vr_1.VR.SQ,
    '400441': vr_1.VR.SQ,
    '400500': vr_1.VR.SQ,
    '40050a': vr_1.VR.LO,
    '400512': vr_1.VR.LO,
    '400513': vr_1.VR.SQ,
    '400515': vr_1.VR.SQ,
    '400518': vr_1.VR.SQ,
    '40051a': vr_1.VR.LO,
    '400520': vr_1.VR.SQ,
    '400550': vr_1.VR.SQ,
    '400551': vr_1.VR.LO,
    '400552': vr_1.VR.SQ,
    '400553': vr_1.VR.ST,
    '400554': vr_1.VR.UI,
    '400555': vr_1.VR.SQ,
    '400556': vr_1.VR.ST,
    '400560': vr_1.VR.SQ,
    '400562': vr_1.VR.SQ,
    '40059a': vr_1.VR.SQ,
    '400600': vr_1.VR.LO,
    '400602': vr_1.VR.UT,
    '400610': vr_1.VR.SQ,
    '400612': vr_1.VR.SQ,
    '400620': vr_1.VR.SQ,
    '4006fa': vr_1.VR.LO,
    '40071a': vr_1.VR.SQ,
    '40072a': vr_1.VR.DS,
    '40073a': vr_1.VR.DS,
    '40074a': vr_1.VR.DS,
    '4008d8': vr_1.VR.SQ,
    '4008da': vr_1.VR.SQ,
    '4008ea': vr_1.VR.SQ,
    '4009f8': vr_1.VR.SQ,
    '401001': vr_1.VR.SH,
    '401002': vr_1.VR.LO,
    '401003': vr_1.VR.SH,
    '401004': vr_1.VR.LO,
    '401005': vr_1.VR.LO,
    '401006': vr_1.VR.SH,
    '401007': vr_1.VR.SH,
    '401008': vr_1.VR.LO,
    '401009': vr_1.VR.SH,
    '40100a': vr_1.VR.SQ,
    '401010': vr_1.VR.PN,
    '401011': vr_1.VR.SQ,
    '401012': vr_1.VR.SQ,
    '401060': vr_1.VR.LO,
    '401101': vr_1.VR.SQ,
    '401102': vr_1.VR.ST,
    '401103': vr_1.VR.LO,
    '401104': vr_1.VR.LT,
    '401400': vr_1.VR.LT,
    '402001': vr_1.VR.LO,
    '402004': vr_1.VR.DA,
    '402005': vr_1.VR.TM,
    '402006': vr_1.VR.SH,
    '402007': vr_1.VR.SH,
    '402008': vr_1.VR.PN,
    '402009': vr_1.VR.SH,
    '402010': vr_1.VR.SH,
    '402011': vr_1.VR.LT,
    '402016': vr_1.VR.LO,
    '402017': vr_1.VR.LO,
    '402400': vr_1.VR.LT,
    '403001': vr_1.VR.LO,
    '404001': vr_1.VR.CS,
    '404002': vr_1.VR.CS,
    '404003': vr_1.VR.CS,
    '404004': vr_1.VR.SQ,
    '404005': vr_1.VR.DT,
    '404006': vr_1.VR.CS,
    '404007': vr_1.VR.SQ,
    '404009': vr_1.VR.SQ,
    '404010': vr_1.VR.DT,
    '404011': vr_1.VR.DT,
    '404015': vr_1.VR.SQ,
    '404016': vr_1.VR.SQ,
    '404018': vr_1.VR.SQ,
    '404019': vr_1.VR.SQ,
    '404020': vr_1.VR.CS,
    '404021': vr_1.VR.SQ,
    '404022': vr_1.VR.SQ,
    '404023': vr_1.VR.UI,
    '404025': vr_1.VR.SQ,
    '404026': vr_1.VR.SQ,
    '404027': vr_1.VR.SQ,
    '404028': vr_1.VR.SQ,
    '404029': vr_1.VR.SQ,
    '404030': vr_1.VR.SQ,
    '404031': vr_1.VR.SQ,
    '404032': vr_1.VR.SQ,
    '404033': vr_1.VR.SQ,
    '404034': vr_1.VR.SQ,
    '404035': vr_1.VR.SQ,
    '404036': vr_1.VR.LO,
    '404037': vr_1.VR.PN,
    '404040': vr_1.VR.CS,
    '404041': vr_1.VR.CS,
    '404050': vr_1.VR.DT,
    '404051': vr_1.VR.DT,
    '404052': vr_1.VR.DT,
    '404070': vr_1.VR.SQ,
    '404071': vr_1.VR.SQ,
    '404072': vr_1.VR.SQ,
    '404073': vr_1.VR.UR,
    '404074': vr_1.VR.SQ,
    '408302': vr_1.VR.DS,
    '408303': vr_1.VR.CS,
    '409092': vr_1.VR.SQ,
    '409094': vr_1.VR.SQ,
    '409096': vr_1.VR.SQ,
    '409098': vr_1.VR.SQ,
    '409210': vr_1.VR.SH,
    '409211': vr_1.VR.SS,
    '409212': vr_1.VR.FD,
    '409213': vr_1.VR.FD,
    '409214': vr_1.VR.FD,
    '409216': vr_1.VR.SS,
    '409220': vr_1.VR.SQ,
    '409224': vr_1.VR.FD,
    '409225': vr_1.VR.FD,
    '40a007': vr_1.VR.CS,
    '40a010': vr_1.VR.CS,
    '40a020': vr_1.VR.SQ,
    '40a021': vr_1.VR.UI,
    '40a022': vr_1.VR.UI,
    '40a023': vr_1.VR.DA,
    '40a024': vr_1.VR.TM,
    '40a026': vr_1.VR.SQ,
    '40a027': vr_1.VR.LO,
    '40a028': vr_1.VR.SQ,
    '40a030': vr_1.VR.DT,
    '40a032': vr_1.VR.DT,
    '40a040': vr_1.VR.CS,
    '40a043': vr_1.VR.SQ,
    '40a047': vr_1.VR.LO,
    '40a050': vr_1.VR.CS,
    '40a057': vr_1.VR.CS,
    '40a060': vr_1.VR.LO,
    '40a066': vr_1.VR.SQ,
    '40a067': vr_1.VR.PN,
    '40a068': vr_1.VR.SQ,
    '40a070': vr_1.VR.SQ,
    '40a073': vr_1.VR.SQ,
    '40a074': vr_1.VR.OB,
    '40a075': vr_1.VR.PN,
    '40a076': vr_1.VR.SQ,
    '40a078': vr_1.VR.SQ,
    '40a07a': vr_1.VR.SQ,
    '40a07c': vr_1.VR.SQ,
    '40a080': vr_1.VR.CS,
    '40a082': vr_1.VR.DT,
    '40a084': vr_1.VR.CS,
    '40a085': vr_1.VR.SQ,
    '40a088': vr_1.VR.SQ,
    '40a089': vr_1.VR.OB,
    '40a090': vr_1.VR.SQ,
    '40a0b0': vr_1.VR.US,
    '40a110': vr_1.VR.DA,
    '40a112': vr_1.VR.TM,
    '40a120': vr_1.VR.DT,
    '40a121': vr_1.VR.DA,
    '40a122': vr_1.VR.TM,
    '40a123': vr_1.VR.PN,
    '40a124': vr_1.VR.UI,
    '40a125': vr_1.VR.CS,
    '40a130': vr_1.VR.CS,
    '40a132': vr_1.VR.UL,
    '40a136': vr_1.VR.US,
    '40a138': vr_1.VR.DS,
    '40a13a': vr_1.VR.DT,
    '40a160': vr_1.VR.UT,
    '40a161': vr_1.VR.FD,
    '40a162': vr_1.VR.SL,
    '40a163': vr_1.VR.UL,
    '40a167': vr_1.VR.SQ,
    '40a168': vr_1.VR.SQ,
    '40a16a': vr_1.VR.ST,
    '40a170': vr_1.VR.SQ,
    '40a171': vr_1.VR.UI,
    '40a172': vr_1.VR.UI,
    '40a173': vr_1.VR.CS,
    '40a174': vr_1.VR.CS,
    '40a180': vr_1.VR.US,
    '40a192': vr_1.VR.DA,
    '40a193': vr_1.VR.TM,
    '40a194': vr_1.VR.CS,
    '40a195': vr_1.VR.SQ,
    '40a224': vr_1.VR.ST,
    '40a290': vr_1.VR.CS,
    '40a296': vr_1.VR.SQ,
    '40a297': vr_1.VR.ST,
    '40a29a': vr_1.VR.SL,
    '40a300': vr_1.VR.SQ,
    '40a301': vr_1.VR.SQ,
    '40a307': vr_1.VR.PN,
    '40a30a': vr_1.VR.DS,
    '40a313': vr_1.VR.SQ,
    '40a33a': vr_1.VR.ST,
    '40a340': vr_1.VR.SQ,
    '40a352': vr_1.VR.PN,
    '40a353': vr_1.VR.ST,
    '40a354': vr_1.VR.LO,
    '40a358': vr_1.VR.SQ,
    '40a360': vr_1.VR.SQ,
    '40a370': vr_1.VR.SQ,
    '40a372': vr_1.VR.SQ,
    '40a375': vr_1.VR.SQ,
    '40a380': vr_1.VR.SQ,
    '40a385': vr_1.VR.SQ,
    '40a390': vr_1.VR.SQ,
    '40a402': vr_1.VR.UI,
    '40a403': vr_1.VR.CS,
    '40a404': vr_1.VR.SQ,
    '40a491': vr_1.VR.CS,
    '40a492': vr_1.VR.LO,
    '40a493': vr_1.VR.CS,
    '40a494': vr_1.VR.CS,
    '40a496': vr_1.VR.CS,
    '40a504': vr_1.VR.SQ,
    '40a525': vr_1.VR.SQ,
    '40a600': vr_1.VR.CS,
    '40a601': vr_1.VR.CS,
    '40a603': vr_1.VR.CS,
    '40a730': vr_1.VR.SQ,
    '40a731': vr_1.VR.SQ,
    '40a732': vr_1.VR.SQ,
    '40a744': vr_1.VR.SQ,
    '40a992': vr_1.VR.ST,
    '40b020': vr_1.VR.SQ,
    '40db00': vr_1.VR.CS,
    '40db06': vr_1.VR.DT,
    '40db07': vr_1.VR.DT,
    '40db0b': vr_1.VR.CS,
    '40db0c': vr_1.VR.UI,
    '40db0d': vr_1.VR.UI,
    '40db73': vr_1.VR.UL,
    '40e001': vr_1.VR.ST,
    '40e004': vr_1.VR.DT,
    '40e006': vr_1.VR.SQ,
    '40e008': vr_1.VR.SQ,
    '40e010': vr_1.VR.UR,
    '40e011': vr_1.VR.UI,
    '40e020': vr_1.VR.CS,
    '40e021': vr_1.VR.SQ,
    '40e022': vr_1.VR.SQ,
    '40e023': vr_1.VR.SQ,
    '40e024': vr_1.VR.SQ,
    '40e025': vr_1.VR.SQ,
    '40e030': vr_1.VR.UI,
    '40e031': vr_1.VR.UI,
    '420010': vr_1.VR.ST,
    '420011': vr_1.VR.OB,
    '420012': vr_1.VR.LO,
    '420013': vr_1.VR.SQ,
    '420014': vr_1.VR.LO,
    '440001': vr_1.VR.ST,
    '440002': vr_1.VR.CS,
    '440003': vr_1.VR.LT,
    '440004': vr_1.VR.DT,
    '440007': vr_1.VR.SQ,
    '440008': vr_1.VR.LO,
    '440009': vr_1.VR.LT,
    '44000a': vr_1.VR.LO,
    '44000b': vr_1.VR.DT,
    '440010': vr_1.VR.DT,
    '440011': vr_1.VR.LO,
    '440012': vr_1.VR.LO,
    '440013': vr_1.VR.SQ,
    '440019': vr_1.VR.SQ,
    '440100': vr_1.VR.SQ,
    '440101': vr_1.VR.SQ,
    '440102': vr_1.VR.UI,
    '440103': vr_1.VR.SQ,
    '440104': vr_1.VR.DT,
    '440105': vr_1.VR.DT,
    '440106': vr_1.VR.UT,
    '440107': vr_1.VR.SQ,
    '440108': vr_1.VR.UI,
    '440109': vr_1.VR.SQ,
    '44010a': vr_1.VR.SQ,
    '460012': vr_1.VR.LO,
    '460014': vr_1.VR.SQ,
    '460015': vr_1.VR.SQ,
    '460016': vr_1.VR.SQ,
    '460018': vr_1.VR.SQ,
    '460028': vr_1.VR.SQ,
    '460030': vr_1.VR.FD,
    '460032': vr_1.VR.CS,
    '460034': vr_1.VR.FD,
    '460036': vr_1.VR.CS,
    '460038': vr_1.VR.CS,
    '460040': vr_1.VR.FD,
    '460042': vr_1.VR.FD,
    '460044': vr_1.VR.FD,
    '460046': vr_1.VR.FD,
    '460050': vr_1.VR.SQ,
    '460052': vr_1.VR.SQ,
    '460060': vr_1.VR.FD,
    '460062': vr_1.VR.FD,
    '460063': vr_1.VR.FD,
    '460064': vr_1.VR.FD,
    '460070': vr_1.VR.SQ,
    '460071': vr_1.VR.SQ,
    '460074': vr_1.VR.SQ,
    '460075': vr_1.VR.FD,
    '460076': vr_1.VR.FD,
    '460077': vr_1.VR.FD,
    '460080': vr_1.VR.SQ,
    '460092': vr_1.VR.CS,
    '460094': vr_1.VR.CS,
    '460095': vr_1.VR.CS,
    '460097': vr_1.VR.SQ,
    '460098': vr_1.VR.SQ,
    '460100': vr_1.VR.SQ,
    '460101': vr_1.VR.SQ,
    '460102': vr_1.VR.SQ,
    '460104': vr_1.VR.FD,
    '460106': vr_1.VR.FD,
    '460121': vr_1.VR.SQ,
    '460122': vr_1.VR.SQ,
    '460123': vr_1.VR.SQ,
    '460124': vr_1.VR.SQ,
    '460125': vr_1.VR.CS,
    '460135': vr_1.VR.SS,
    '460137': vr_1.VR.FD,
    '460139': vr_1.VR.LO,
    '460145': vr_1.VR.SQ,
    '460146': vr_1.VR.FD,
    '460147': vr_1.VR.FD,
    '460201': vr_1.VR.CS,
    '460202': vr_1.VR.FL,
    '460203': vr_1.VR.FL,
    '460204': vr_1.VR.FL,
    '460205': vr_1.VR.FL,
    '460207': vr_1.VR.SQ,
    '460208': vr_1.VR.IS,
    '460210': vr_1.VR.SQ,
    '460211': vr_1.VR.SQ,
    '460212': vr_1.VR.FL,
    '460213': vr_1.VR.FL,
    '460215': vr_1.VR.SQ,
    '460218': vr_1.VR.SQ,
    '460220': vr_1.VR.FL,
    '460224': vr_1.VR.FL,
    '460227': vr_1.VR.FL,
    '460230': vr_1.VR.FL,
    '460232': vr_1.VR.FL,
    '460234': vr_1.VR.FL,
    '460236': vr_1.VR.FL,
    '460238': vr_1.VR.FL,
    '460242': vr_1.VR.CS,
    '460244': vr_1.VR.SQ,
    '460247': vr_1.VR.FL,
    '460248': vr_1.VR.CS,
    '460249': vr_1.VR.FL,
    '460250': vr_1.VR.FL,
    '460251': vr_1.VR.FL,
    '460252': vr_1.VR.FL,
    '460253': vr_1.VR.FL,
    '480001': vr_1.VR.FL,
    '480002': vr_1.VR.FL,
    '480003': vr_1.VR.FL,
    '480006': vr_1.VR.UL,
    '480007': vr_1.VR.UL,
    '480008': vr_1.VR.SQ,
    '480010': vr_1.VR.CS,
    '480011': vr_1.VR.CS,
    '480012': vr_1.VR.CS,
    '480013': vr_1.VR.US,
    '480014': vr_1.VR.FL,
    '480015': vr_1.VR.US,
    '480100': vr_1.VR.SQ,
    '480102': vr_1.VR.DS,
    '480105': vr_1.VR.SQ,
    '480106': vr_1.VR.SH,
    '480107': vr_1.VR.ST,
    '480108': vr_1.VR.SQ,
    '480110': vr_1.VR.SQ,
    '480111': vr_1.VR.DS,
    '480112': vr_1.VR.DS,
    '480113': vr_1.VR.DS,
    '480120': vr_1.VR.SQ,
    '480200': vr_1.VR.SQ,
    '480201': vr_1.VR.US,
    '480202': vr_1.VR.US,
    '480207': vr_1.VR.SQ,
    '48021a': vr_1.VR.SQ,
    '48021e': vr_1.VR.SL,
    '48021f': vr_1.VR.SL,
    '480301': vr_1.VR.CS,
    '500004': vr_1.VR.CS,
    '500010': vr_1.VR.SQ,
    '500012': vr_1.VR.SQ,
    '500013': vr_1.VR.FD,
    '500014': vr_1.VR.DS,
    '500015': vr_1.VR.FD,
    '500016': vr_1.VR.DS,
    '500017': vr_1.VR.CS,
    '500018': vr_1.VR.DS,
    '500019': vr_1.VR.DS,
    '50001a': vr_1.VR.CS,
    '50001b': vr_1.VR.LO,
    '50001c': vr_1.VR.FD,
    '50001d': vr_1.VR.FD,
    '50001e': vr_1.VR.LO,
    '500020': vr_1.VR.LO,
    '520001': vr_1.VR.FL,
    '520002': vr_1.VR.FD,
    '520003': vr_1.VR.FD,
    '520004': vr_1.VR.FD,
    '520006': vr_1.VR.CS,
    '520007': vr_1.VR.FD,
    '520008': vr_1.VR.FD,
    '520009': vr_1.VR.FD,
    '520011': vr_1.VR.FD,
    '520012': vr_1.VR.US,
    '520013': vr_1.VR.FD,
    '520014': vr_1.VR.FD,
    '520016': vr_1.VR.SQ,
    '520025': vr_1.VR.SQ,
    '520026': vr_1.VR.CS,
    '520027': vr_1.VR.SQ,
    '520028': vr_1.VR.FD,
    '520029': vr_1.VR.SQ,
    '520030': vr_1.VR.SS,
    '520031': vr_1.VR.CS,
    '520033': vr_1.VR.FD,
    '520034': vr_1.VR.FD,
    '520036': vr_1.VR.US,
    '520038': vr_1.VR.US,
    '520039': vr_1.VR.CS,
    '52003a': vr_1.VR.CS,
    '540010': vr_1.VR.US,
    '540011': vr_1.VR.US,
    '540012': vr_1.VR.SQ,
    '540013': vr_1.VR.SQ,
    '540014': vr_1.VR.DS,
    '540015': vr_1.VR.DS,
    '540016': vr_1.VR.SQ,
    '540017': vr_1.VR.IS,
    '540018': vr_1.VR.SH,
    '540020': vr_1.VR.US,
    '540021': vr_1.VR.US,
    '540022': vr_1.VR.SQ,
    '540030': vr_1.VR.US,
    '540031': vr_1.VR.US,
    '540032': vr_1.VR.SQ,
    '540033': vr_1.VR.US,
    '540036': vr_1.VR.IS,
    '540038': vr_1.VR.IS,
    '540039': vr_1.VR.CS,
    '540050': vr_1.VR.US,
    '540051': vr_1.VR.US,
    '540052': vr_1.VR.SQ,
    '540053': vr_1.VR.US,
    '540060': vr_1.VR.US,
    '540061': vr_1.VR.US,
    '540062': vr_1.VR.SQ,
    '540063': vr_1.VR.SQ,
    '540070': vr_1.VR.US,
    '540071': vr_1.VR.US,
    '540072': vr_1.VR.SQ,
    '540073': vr_1.VR.DS,
    '540080': vr_1.VR.US,
    '540081': vr_1.VR.US,
    '540090': vr_1.VR.US,
    '540100': vr_1.VR.US,
    '540101': vr_1.VR.US,
    '540200': vr_1.VR.DS,
    '540202': vr_1.VR.CS,
    '540210': vr_1.VR.IS,
    '540211': vr_1.VR.US,
    '540220': vr_1.VR.SQ,
    '540222': vr_1.VR.SQ,
    '540300': vr_1.VR.SQ,
    '540302': vr_1.VR.SQ,
    '540304': vr_1.VR.SQ,
    '540306': vr_1.VR.SQ,
    '540308': vr_1.VR.US,
    '540400': vr_1.VR.SH,
    '540410': vr_1.VR.SQ,
    '540412': vr_1.VR.SQ,
    '540414': vr_1.VR.SQ,
    '540500': vr_1.VR.CS,
    '540501': vr_1.VR.CS,
    '541000': vr_1.VR.CS,
    '541001': vr_1.VR.CS,
    '541002': vr_1.VR.CS,
    '541004': vr_1.VR.CS,
    '541006': vr_1.VR.CS,
    '541100': vr_1.VR.CS,
    '541101': vr_1.VR.LO,
    '541102': vr_1.VR.CS,
    '541103': vr_1.VR.LO,
    '541104': vr_1.VR.LO,
    '541105': vr_1.VR.LO,
    '541200': vr_1.VR.DS,
    '541201': vr_1.VR.IS,
    '541202': vr_1.VR.IS,
    '541203': vr_1.VR.DS,
    '541210': vr_1.VR.DS,
    '541220': vr_1.VR.CS,
    '541300': vr_1.VR.DS,
    '541310': vr_1.VR.IS,
    '541311': vr_1.VR.IS,
    '541320': vr_1.VR.DS,
    '541321': vr_1.VR.DS,
    '541322': vr_1.VR.DS,
    '541323': vr_1.VR.DS,
    '541324': vr_1.VR.DS,
    '541330': vr_1.VR.US,
    '541400': vr_1.VR.CS,
    '541401': vr_1.VR.CS,
    '603000': vr_1.VR.SQ,
    '603002': vr_1.VR.US,
    '603004': vr_1.VR.SS,
    '603006': vr_1.VR.SS,
    '603008': vr_1.VR.US,
    '603010': vr_1.VR.LO,
    '603020': vr_1.VR.UL,
    '620001': vr_1.VR.CS,
    '620002': vr_1.VR.SQ,
    '620003': vr_1.VR.SQ,
    '620004': vr_1.VR.US,
    '620005': vr_1.VR.LO,
    '620006': vr_1.VR.ST,
    '620007': vr_1.VR.SQ,
    '620008': vr_1.VR.CS,
    '620009': vr_1.VR.LO,
    '62000a': vr_1.VR.SQ,
    '62000b': vr_1.VR.US,
    '62000c': vr_1.VR.US,
    '62000d': vr_1.VR.US,
    '62000e': vr_1.VR.US,
    '62000f': vr_1.VR.SQ,
    '620010': vr_1.VR.CS,
    '620011': vr_1.VR.SQ,
    '620012': vr_1.VR.SQ,
    '620020': vr_1.VR.UT,
    '620021': vr_1.VR.UI,
    '640002': vr_1.VR.SQ,
    '640003': vr_1.VR.UI,
    '640005': vr_1.VR.SQ,
    '640007': vr_1.VR.UL,
    '640008': vr_1.VR.FD,
    '640009': vr_1.VR.OF,
    '64000f': vr_1.VR.SQ,
    '640010': vr_1.VR.SQ,
    '660001': vr_1.VR.UL,
    '660002': vr_1.VR.SQ,
    '660003': vr_1.VR.UL,
    '660004': vr_1.VR.LT,
    '660009': vr_1.VR.CS,
    '66000a': vr_1.VR.FL,
    '66000b': vr_1.VR.LO,
    '66000c': vr_1.VR.FL,
    '66000d': vr_1.VR.CS,
    '66000e': vr_1.VR.CS,
    '660010': vr_1.VR.CS,
    '660011': vr_1.VR.SQ,
    '660012': vr_1.VR.SQ,
    '660013': vr_1.VR.SQ,
    '660015': vr_1.VR.UL,
    '660016': vr_1.VR.OF,
    '660017': vr_1.VR.FL,
    '660018': vr_1.VR.FL,
    '660019': vr_1.VR.FL,
    '66001a': vr_1.VR.FL,
    '66001b': vr_1.VR.FL,
    '66001c': vr_1.VR.FL,
    '66001e': vr_1.VR.UL,
    '66001f': vr_1.VR.US,
    '660020': vr_1.VR.FL,
    '660021': vr_1.VR.OF,
    '660023': vr_1.VR.OW,
    '660024': vr_1.VR.OW,
    '660025': vr_1.VR.OW,
    '660026': vr_1.VR.SQ,
    '660027': vr_1.VR.SQ,
    '660028': vr_1.VR.SQ,
    '660029': vr_1.VR.OW,
    '66002a': vr_1.VR.UL,
    '66002b': vr_1.VR.SQ,
    '66002c': vr_1.VR.UL,
    '66002d': vr_1.VR.SQ,
    '66002e': vr_1.VR.SQ,
    '66002f': vr_1.VR.SQ,
    '660030': vr_1.VR.SQ,
    '660031': vr_1.VR.LO,
    '660032': vr_1.VR.LT,
    '660034': vr_1.VR.SQ,
    '660035': vr_1.VR.SQ,
    '660036': vr_1.VR.LO,
    '660037': vr_1.VR.FL,
    '660038': vr_1.VR.FL,
    '660040': vr_1.VR.OL,
    '660041': vr_1.VR.OL,
    '660042': vr_1.VR.OL,
    '660043': vr_1.VR.OL,
    '660101': vr_1.VR.SQ,
    '660102': vr_1.VR.SQ,
    '660103': vr_1.VR.OW,
    '660104': vr_1.VR.SQ,
    '660105': vr_1.VR.UL,
    '660106': vr_1.VR.LO,
    '660107': vr_1.VR.UT,
    '660108': vr_1.VR.SQ,
    '660121': vr_1.VR.SQ,
    '660124': vr_1.VR.SQ,
    '660125': vr_1.VR.OF,
    '660129': vr_1.VR.OL,
    '660130': vr_1.VR.SQ,
    '660132': vr_1.VR.SQ,
    '660133': vr_1.VR.SQ,
    '660134': vr_1.VR.SQ,
    '686210': vr_1.VR.LO,
    '686221': vr_1.VR.LO,
    '686222': vr_1.VR.SQ,
    '686223': vr_1.VR.CS,
    '686224': vr_1.VR.SQ,
    '686225': vr_1.VR.SQ,
    '686226': vr_1.VR.DT,
    '686230': vr_1.VR.SQ,
    '686260': vr_1.VR.SQ,
    '686265': vr_1.VR.SQ,
    '686270': vr_1.VR.DT,
    '686280': vr_1.VR.ST,
    '6862a0': vr_1.VR.SQ,
    '6862a5': vr_1.VR.FD,
    '6862c0': vr_1.VR.SQ,
    '6862d0': vr_1.VR.US,
    '6862d5': vr_1.VR.LO,
    '6862e0': vr_1.VR.SQ,
    '6862f0': vr_1.VR.SQ,
    '6862f2': vr_1.VR.FD,
    '686300': vr_1.VR.OB,
    '686310': vr_1.VR.US,
    '686320': vr_1.VR.SQ,
    '686330': vr_1.VR.US,
    '686340': vr_1.VR.LO,
    '686345': vr_1.VR.ST,
    '686346': vr_1.VR.FD,
    '686347': vr_1.VR.FD,
    '686350': vr_1.VR.US,
    '686360': vr_1.VR.SQ,
    '686380': vr_1.VR.LO,
    '686390': vr_1.VR.FD,
    '6863a0': vr_1.VR.SQ,
    '6863a4': vr_1.VR.SQ,
    '6863a8': vr_1.VR.SQ,
    '6863ac': vr_1.VR.SQ,
    '6863b0': vr_1.VR.SQ,
    '6863c0': vr_1.VR.US,
    '6863d0': vr_1.VR.LO,
    '6863e0': vr_1.VR.SQ,
    '6863f0': vr_1.VR.US,
    '686400': vr_1.VR.SQ,
    '686410': vr_1.VR.US,
    '686420': vr_1.VR.CS,
    '686430': vr_1.VR.SQ,
    '686440': vr_1.VR.US,
    '686450': vr_1.VR.FD,
    '686460': vr_1.VR.FD,
    '686470': vr_1.VR.SQ,
    '686490': vr_1.VR.FD,
    '6864a0': vr_1.VR.FD,
    '6864c0': vr_1.VR.FD,
    '6864d0': vr_1.VR.FD,
    '6864f0': vr_1.VR.FD,
    '686500': vr_1.VR.SQ,
    '686510': vr_1.VR.SQ,
    '686520': vr_1.VR.SQ,
    '686530': vr_1.VR.US,
    '686540': vr_1.VR.LO,
    '686545': vr_1.VR.SQ,
    '686550': vr_1.VR.SQ,
    '686560': vr_1.VR.FD,
    '686590': vr_1.VR.FD,
    '6865a0': vr_1.VR.SQ,
    '6865b0': vr_1.VR.FD,
    '6865d0': vr_1.VR.FD,
    '6865e0': vr_1.VR.SQ,
    '6865f0': vr_1.VR.FD,
    '686610': vr_1.VR.FD,
    '686620': vr_1.VR.FD,
    '700001': vr_1.VR.SQ,
    '700002': vr_1.VR.CS,
    '700003': vr_1.VR.CS,
    '700004': vr_1.VR.CS,
    '700005': vr_1.VR.CS,
    '700006': vr_1.VR.ST,
    '700008': vr_1.VR.SQ,
    '700009': vr_1.VR.SQ,
    '700010': vr_1.VR.FL,
    '700011': vr_1.VR.FL,
    '700012': vr_1.VR.CS,
    '700014': vr_1.VR.FL,
    '700015': vr_1.VR.CS,
    '700020': vr_1.VR.US,
    '700021': vr_1.VR.US,
    '700022': vr_1.VR.FL,
    '700023': vr_1.VR.CS,
    '700024': vr_1.VR.CS,
    '700040': vr_1.VR.IS,
    '700041': vr_1.VR.CS,
    '700042': vr_1.VR.US,
    '700050': vr_1.VR.US,
    '700051': vr_1.VR.US,
    '700052': vr_1.VR.SL,
    '700053': vr_1.VR.SL,
    '70005a': vr_1.VR.SQ,
    '700060': vr_1.VR.SQ,
    '700062': vr_1.VR.IS,
    '700066': vr_1.VR.US,
    '700067': vr_1.VR.US,
    '700068': vr_1.VR.LO,
    '700080': vr_1.VR.CS,
    '700081': vr_1.VR.LO,
    '700082': vr_1.VR.DA,
    '700083': vr_1.VR.TM,
    '700084': vr_1.VR.PN,
    '700086': vr_1.VR.SQ,
    '700087': vr_1.VR.SQ,
    '700100': vr_1.VR.CS,
    '700101': vr_1.VR.DS,
    '700102': vr_1.VR.IS,
    '700103': vr_1.VR.FL,
    '700207': vr_1.VR.LO,
    '700208': vr_1.VR.ST,
    '700209': vr_1.VR.SQ,
    '700226': vr_1.VR.UL,
    '700227': vr_1.VR.LO,
    '700228': vr_1.VR.CS,
    '700229': vr_1.VR.LO,
    '700230': vr_1.VR.FD,
    '700231': vr_1.VR.SQ,
    '700232': vr_1.VR.SQ,
    '700233': vr_1.VR.SQ,
    '700234': vr_1.VR.SQ,
    '700241': vr_1.VR.US,
    '700242': vr_1.VR.CS,
    '700243': vr_1.VR.CS,
    '700244': vr_1.VR.CS,
    '700245': vr_1.VR.FL,
    '700246': vr_1.VR.FL,
    '700247': vr_1.VR.US,
    '700248': vr_1.VR.CS,
    '700249': vr_1.VR.CS,
    '700250': vr_1.VR.CS,
    '700251': vr_1.VR.US,
    '700252': vr_1.VR.US,
    '700253': vr_1.VR.FL,
    '700254': vr_1.VR.CS,
    '700255': vr_1.VR.UL,
    '700256': vr_1.VR.OB,
    '700257': vr_1.VR.CS,
    '700258': vr_1.VR.FL,
    '700261': vr_1.VR.FL,
    '700262': vr_1.VR.FL,
    '700273': vr_1.VR.FL,
    '700274': vr_1.VR.CS,
    '700278': vr_1.VR.CS,
    '700279': vr_1.VR.CS,
    '700282': vr_1.VR.CS,
    '700284': vr_1.VR.FL,
    '700285': vr_1.VR.FL,
    '700287': vr_1.VR.SQ,
    '700288': vr_1.VR.FL,
    '700289': vr_1.VR.SH,
    '700294': vr_1.VR.CS,
    '700295': vr_1.VR.UL,
    '700306': vr_1.VR.CS,
    '700308': vr_1.VR.SQ,
    '700309': vr_1.VR.SQ,
    '70030a': vr_1.VR.SQ,
    '70030b': vr_1.VR.FD,
    '70030c': vr_1.VR.CS,
    '70030d': vr_1.VR.SQ,
    '70030f': vr_1.VR.ST,
    '700310': vr_1.VR.SH,
    '700311': vr_1.VR.SQ,
    '700312': vr_1.VR.FD,
    '700314': vr_1.VR.SQ,
    '700318': vr_1.VR.SQ,
    '70031a': vr_1.VR.UI,
    '70031b': vr_1.VR.UI,
    '70031c': vr_1.VR.SQ,
    '70031e': vr_1.VR.SQ,
    '70031f': vr_1.VR.SQ,
    '700401': vr_1.VR.US,
    '700402': vr_1.VR.SQ,
    '700403': vr_1.VR.FL,
    '700404': vr_1.VR.SQ,
    '700405': vr_1.VR.CS,
    '701101': vr_1.VR.UI,
    '701102': vr_1.VR.UI,
    '701103': vr_1.VR.US,
    '701104': vr_1.VR.SQ,
    '701201': vr_1.VR.SQ,
    '701202': vr_1.VR.CS,
    '701203': vr_1.VR.US,
    '701204': vr_1.VR.CS,
    '701205': vr_1.VR.US,
    '701206': vr_1.VR.CS,
    '701207': vr_1.VR.US,
    '701208': vr_1.VR.CS,
    '701209': vr_1.VR.UI,
    '70120a': vr_1.VR.SQ,
    '70120b': vr_1.VR.CS,
    '70120c': vr_1.VR.US,
    '70120d': vr_1.VR.CS,
    '701301': vr_1.VR.SQ,
    '701302': vr_1.VR.CS,
    '701303': vr_1.VR.FD,
    '701304': vr_1.VR.SQ,
    '701305': vr_1.VR.FD,
    '701306': vr_1.VR.FD,
    '701309': vr_1.VR.US,
    '701501': vr_1.VR.CS,
    '701502': vr_1.VR.CS,
    '701503': vr_1.VR.FD,
    '701505': vr_1.VR.FD,
    '701507': vr_1.VR.FD,
    '701508': vr_1.VR.FD,
    '70150c': vr_1.VR.UL,
    '70150d': vr_1.VR.OD,
    '701511': vr_1.VR.FD,
    '701512': vr_1.VR.FD,
    '701602': vr_1.VR.CS,
    '701603': vr_1.VR.FD,
    '701604': vr_1.VR.FD,
    '701605': vr_1.VR.FD,
    '701606': vr_1.VR.FD,
    '701607': vr_1.VR.FD,
    '701701': vr_1.VR.CS,
    '701702': vr_1.VR.FD,
    '701703': vr_1.VR.FD,
    '701704': vr_1.VR.FD,
    '701705': vr_1.VR.FD,
    '701706': vr_1.VR.FD,
    '701801': vr_1.VR.SQ,
    '701802': vr_1.VR.CS,
    '701803': vr_1.VR.SQ,
    '701804': vr_1.VR.US,
    '701805': vr_1.VR.SQ,
    '701806': vr_1.VR.SQ,
    '701807': vr_1.VR.US,
    '701808': vr_1.VR.OB,
    '701901': vr_1.VR.SQ,
    '701903': vr_1.VR.SQ,
    '701904': vr_1.VR.UI,
    '701905': vr_1.VR.SQ,
    '701907': vr_1.VR.CS,
    '701a01': vr_1.VR.CS,
    '701a03': vr_1.VR.FD,
    '701a04': vr_1.VR.SQ,
    '701a05': vr_1.VR.FD,
    '701a06': vr_1.VR.FD,
    '701a07': vr_1.VR.OD,
    '701a08': vr_1.VR.SQ,
    '701a09': vr_1.VR.LO,
    '701b01': vr_1.VR.SQ,
    '701b02': vr_1.VR.US,
    '701b03': vr_1.VR.SQ,
    '701b04': vr_1.VR.SQ,
    '701b06': vr_1.VR.CS,
    '701b07': vr_1.VR.CS,
    '701b08': vr_1.VR.CS,
    '701b11': vr_1.VR.SQ,
    '701b12': vr_1.VR.SQ,
    '701b13': vr_1.VR.CS,
    '701b14': vr_1.VR.FD,
    '720002': vr_1.VR.SH,
    '720004': vr_1.VR.LO,
    '720006': vr_1.VR.CS,
    '720008': vr_1.VR.LO,
    '72000a': vr_1.VR.DT,
    '72000c': vr_1.VR.SQ,
    '72000e': vr_1.VR.SQ,
    '720010': vr_1.VR.LO,
    '720012': vr_1.VR.SQ,
    '720014': vr_1.VR.US,
    '720020': vr_1.VR.SQ,
    '720022': vr_1.VR.SQ,
    '720024': vr_1.VR.CS,
    '720026': vr_1.VR.AT,
    '720028': vr_1.VR.US,
    '720030': vr_1.VR.SQ,
    '720032': vr_1.VR.US,
    '720034': vr_1.VR.CS,
    '720038': vr_1.VR.US,
    '72003a': vr_1.VR.CS,
    '72003c': vr_1.VR.SS,
    '72003e': vr_1.VR.SQ,
    '720040': vr_1.VR.LO,
    '720050': vr_1.VR.CS,
    '720052': vr_1.VR.AT,
    '720054': vr_1.VR.LO,
    '720056': vr_1.VR.LO,
    '72005e': vr_1.VR.AE,
    '72005f': vr_1.VR.AS,
    '720060': vr_1.VR.AT,
    '720061': vr_1.VR.DA,
    '720062': vr_1.VR.CS,
    '720063': vr_1.VR.DT,
    '720064': vr_1.VR.IS,
    '720065': vr_1.VR.OB,
    '720066': vr_1.VR.LO,
    '720067': vr_1.VR.OF,
    '720068': vr_1.VR.LT,
    '720069': vr_1.VR.OW,
    '72006a': vr_1.VR.PN,
    '72006b': vr_1.VR.TM,
    '72006c': vr_1.VR.SH,
    '72006d': vr_1.VR.UN,
    '72006e': vr_1.VR.ST,
    '72006f': vr_1.VR.UC,
    '720070': vr_1.VR.UT,
    '720071': vr_1.VR.UR,
    '720072': vr_1.VR.DS,
    '720073': vr_1.VR.OD,
    '720074': vr_1.VR.FD,
    '720075': vr_1.VR.OL,
    '720076': vr_1.VR.FL,
    '720078': vr_1.VR.UL,
    '72007a': vr_1.VR.US,
    '72007c': vr_1.VR.SL,
    '72007e': vr_1.VR.SS,
    '72007f': vr_1.VR.UI,
    '720080': vr_1.VR.SQ,
    '720100': vr_1.VR.US,
    '720102': vr_1.VR.SQ,
    '720104': vr_1.VR.US,
    '720106': vr_1.VR.US,
    '720108': vr_1.VR.FD,
    '72010a': vr_1.VR.US,
    '72010c': vr_1.VR.US,
    '72010e': vr_1.VR.US,
    '720200': vr_1.VR.SQ,
    '720202': vr_1.VR.US,
    '720203': vr_1.VR.LO,
    '720204': vr_1.VR.US,
    '720206': vr_1.VR.LO,
    '720208': vr_1.VR.CS,
    '720210': vr_1.VR.SQ,
    '720212': vr_1.VR.US,
    '720214': vr_1.VR.SQ,
    '720216': vr_1.VR.US,
    '720218': vr_1.VR.US,
    '720300': vr_1.VR.SQ,
    '720302': vr_1.VR.US,
    '720304': vr_1.VR.CS,
    '720306': vr_1.VR.US,
    '720308': vr_1.VR.US,
    '720310': vr_1.VR.CS,
    '720312': vr_1.VR.CS,
    '720314': vr_1.VR.US,
    '720316': vr_1.VR.CS,
    '720318': vr_1.VR.US,
    '720320': vr_1.VR.US,
    '720330': vr_1.VR.FD,
    '720400': vr_1.VR.SQ,
    '720402': vr_1.VR.CS,
    '720404': vr_1.VR.CS,
    '720406': vr_1.VR.CS,
    '720420': vr_1.VR.US,
    '720421': vr_1.VR.US,
    '720422': vr_1.VR.SQ,
    '720424': vr_1.VR.SQ,
    '720427': vr_1.VR.SQ,
    '720430': vr_1.VR.SQ,
    '720432': vr_1.VR.US,
    '720434': vr_1.VR.CS,
    '720500': vr_1.VR.CS,
    '720510': vr_1.VR.CS,
    '720512': vr_1.VR.FD,
    '720514': vr_1.VR.FD,
    '720516': vr_1.VR.CS,
    '720520': vr_1.VR.CS,
    '720600': vr_1.VR.SQ,
    '720602': vr_1.VR.CS,
    '720604': vr_1.VR.CS,
    '720700': vr_1.VR.CS,
    '720702': vr_1.VR.CS,
    '720704': vr_1.VR.CS,
    '720705': vr_1.VR.SQ,
    '720706': vr_1.VR.CS,
    '720710': vr_1.VR.CS,
    '720712': vr_1.VR.CS,
    '720714': vr_1.VR.CS,
    '720716': vr_1.VR.CS,
    '720717': vr_1.VR.CS,
    '720718': vr_1.VR.CS,
    '740120': vr_1.VR.FD,
    '740121': vr_1.VR.FD,
    '741000': vr_1.VR.CS,
    '741002': vr_1.VR.SQ,
    '741004': vr_1.VR.DS,
    '741006': vr_1.VR.ST,
    '741007': vr_1.VR.SQ,
    '741008': vr_1.VR.SQ,
    '74100a': vr_1.VR.UR,
    '74100c': vr_1.VR.LO,
    '74100e': vr_1.VR.SQ,
    '741020': vr_1.VR.SQ,
    '741022': vr_1.VR.CS,
    '741024': vr_1.VR.IS,
    '741025': vr_1.VR.CS,
    '741026': vr_1.VR.FD,
    '741027': vr_1.VR.FD,
    '741028': vr_1.VR.FD,
    '74102a': vr_1.VR.FD,
    '74102b': vr_1.VR.FD,
    '74102c': vr_1.VR.FD,
    '74102d': vr_1.VR.FD,
    '741030': vr_1.VR.SQ,
    '741032': vr_1.VR.CS,
    '741034': vr_1.VR.CS,
    '741036': vr_1.VR.CS,
    '741038': vr_1.VR.DS,
    '74103a': vr_1.VR.DS,
    '741040': vr_1.VR.SQ,
    '741042': vr_1.VR.SQ,
    '741044': vr_1.VR.SQ,
    '741046': vr_1.VR.SQ,
    '741048': vr_1.VR.SQ,
    '74104a': vr_1.VR.SQ,
    '74104c': vr_1.VR.SQ,
    '74104e': vr_1.VR.SQ,
    '741050': vr_1.VR.SQ,
    '741052': vr_1.VR.AT,
    '741054': vr_1.VR.UL,
    '741056': vr_1.VR.LO,
    '741057': vr_1.VR.IS,
    '741200': vr_1.VR.CS,
    '741202': vr_1.VR.LO,
    '741204': vr_1.VR.LO,
    '741210': vr_1.VR.SQ,
    '741212': vr_1.VR.SQ,
    '741216': vr_1.VR.SQ,
    '741220': vr_1.VR.SQ,
    '741222': vr_1.VR.LO,
    '741224': vr_1.VR.SQ,
    '741230': vr_1.VR.LO,
    '741234': vr_1.VR.AE,
    '741236': vr_1.VR.AE,
    '741238': vr_1.VR.LT,
    '741242': vr_1.VR.CS,
    '741244': vr_1.VR.CS,
    '741246': vr_1.VR.CS,
    '741324': vr_1.VR.UL,
    '741338': vr_1.VR.FD,
    '74133a': vr_1.VR.FD,
    '741401': vr_1.VR.SQ,
    '741402': vr_1.VR.DS,
    '741403': vr_1.VR.DS,
    '741404': vr_1.VR.IS,
    '741405': vr_1.VR.SQ,
    '741406': vr_1.VR.IS,
    '741407': vr_1.VR.DS,
    '741408': vr_1.VR.DS,
    '741409': vr_1.VR.SQ,
    '74140a': vr_1.VR.CS,
    '74140b': vr_1.VR.LO,
    '74140c': vr_1.VR.IS,
    '74140d': vr_1.VR.SQ,
    '74140e': vr_1.VR.SQ,
    '760001': vr_1.VR.LO,
    '760003': vr_1.VR.LO,
    '760006': vr_1.VR.LO,
    '760008': vr_1.VR.SQ,
    '76000a': vr_1.VR.CS,
    '76000c': vr_1.VR.SQ,
    '76000e': vr_1.VR.SQ,
    '760010': vr_1.VR.SQ,
    '760020': vr_1.VR.SQ,
    '760030': vr_1.VR.LO,
    '760032': vr_1.VR.SQ,
    '760034': vr_1.VR.SQ,
    '760036': vr_1.VR.CS,
    '760038': vr_1.VR.CS,
    '760040': vr_1.VR.SQ,
    '760055': vr_1.VR.US,
    '760060': vr_1.VR.SQ,
    '760070': vr_1.VR.US,
    '760080': vr_1.VR.US,
    '760090': vr_1.VR.US,
    '7600a0': vr_1.VR.US,
    '7600b0': vr_1.VR.US,
    '7600c0': vr_1.VR.US,
    '780001': vr_1.VR.LO,
    '780010': vr_1.VR.ST,
    '780020': vr_1.VR.LO,
    '780024': vr_1.VR.LO,
    '780026': vr_1.VR.SQ,
    '780028': vr_1.VR.SQ,
    '78002a': vr_1.VR.SQ,
    '78002e': vr_1.VR.US,
    '780050': vr_1.VR.FD,
    '780060': vr_1.VR.FD,
    '780070': vr_1.VR.SQ,
    '780090': vr_1.VR.FD,
    '7800a0': vr_1.VR.FD,
    '7800b0': vr_1.VR.SQ,
    '7800b2': vr_1.VR.LO,
    '7800b4': vr_1.VR.SQ,
    '7800b6': vr_1.VR.US,
    '7800b8': vr_1.VR.US,
    '800001': vr_1.VR.SQ,
    '800002': vr_1.VR.SQ,
    '800003': vr_1.VR.SQ,
    '800004': vr_1.VR.FD,
    '800005': vr_1.VR.FD,
    '800006': vr_1.VR.US,
    '800007': vr_1.VR.US,
    '800008': vr_1.VR.SQ,
    '800009': vr_1.VR.SH,
    '800010': vr_1.VR.OF,
    '800011': vr_1.VR.OF,
    '800012': vr_1.VR.SQ,
    '800013': vr_1.VR.SQ,
    '820001': vr_1.VR.CS,
    '820003': vr_1.VR.UT,
    '820004': vr_1.VR.SQ,
    '820005': vr_1.VR.SQ,
    '820006': vr_1.VR.UL,
    '820007': vr_1.VR.SQ,
    '820008': vr_1.VR.CS,
    '82000a': vr_1.VR.UT,
    '82000c': vr_1.VR.SQ,
    '820010': vr_1.VR.SQ,
    '820016': vr_1.VR.LO,
    '820017': vr_1.VR.SQ,
    '820018': vr_1.VR.LO,
    '820019': vr_1.VR.LO,
    '820021': vr_1.VR.SQ,
    '820022': vr_1.VR.SQ,
    '820023': vr_1.VR.LO,
    '820032': vr_1.VR.CS,
    '820033': vr_1.VR.UT,
    '820034': vr_1.VR.SQ,
    '820035': vr_1.VR.SQ,
    '820036': vr_1.VR.CS,
    '820037': vr_1.VR.UT,
    '820038': vr_1.VR.CS,
    '880130': vr_1.VR.SH,
    '880140': vr_1.VR.UI,
    '880200': vr_1.VR.SQ,
    '880904': vr_1.VR.LO,
    '880906': vr_1.VR.ST,
    '880910': vr_1.VR.LO,
    '880912': vr_1.VR.LO,
    '1000410': vr_1.VR.CS,
    '1000420': vr_1.VR.DT,
    '1000424': vr_1.VR.LT,
    '1000426': vr_1.VR.LO,
    '4000005': vr_1.VR.US,
    '4000010': vr_1.VR.UI,
    '4000015': vr_1.VR.CS,
    '4000020': vr_1.VR.AT,
    '4000100': vr_1.VR.UI,
    '4000105': vr_1.VR.DT,
    '4000110': vr_1.VR.CS,
    '4000115': vr_1.VR.OB,
    '4000120': vr_1.VR.OB,
    '4000305': vr_1.VR.CS,
    '4000310': vr_1.VR.OB,
    '4000401': vr_1.VR.SQ,
    '4000402': vr_1.VR.SQ,
    '4000403': vr_1.VR.SQ,
    '4000404': vr_1.VR.OB,
    '4000500': vr_1.VR.SQ,
    '4000510': vr_1.VR.UI,
    '4000520': vr_1.VR.OB,
    '4000550': vr_1.VR.SQ,
    '4000561': vr_1.VR.SQ,
    '4000562': vr_1.VR.DT,
    '4000563': vr_1.VR.LO,
    '4000564': vr_1.VR.LO,
    '4000565': vr_1.VR.CS,
    '10000000': vr_1.VR.US,
    '10000001': vr_1.VR.US,
    '10000002': vr_1.VR.US,
    '10000003': vr_1.VR.US,
    '10000004': vr_1.VR.US,
    '10000005': vr_1.VR.US,
    '10100000': vr_1.VR.US,
    '20000010': vr_1.VR.IS,
    '2000001e': vr_1.VR.SQ,
    '20000020': vr_1.VR.CS,
    '20000030': vr_1.VR.CS,
    '20000040': vr_1.VR.CS,
    '20000050': vr_1.VR.LO,
    '20000060': vr_1.VR.IS,
    '20000061': vr_1.VR.IS,
    '20000062': vr_1.VR.CS,
    '20000063': vr_1.VR.CS,
    '20000065': vr_1.VR.CS,
    '20000067': vr_1.VR.CS,
    '20000069': vr_1.VR.CS,
    '2000006a': vr_1.VR.CS,
    '200000a0': vr_1.VR.US,
    '200000a1': vr_1.VR.US,
    '200000a2': vr_1.VR.SQ,
    '200000a4': vr_1.VR.SQ,
    '200000a8': vr_1.VR.SQ,
    '20000500': vr_1.VR.SQ,
    '20000510': vr_1.VR.SQ,
    '20100010': vr_1.VR.ST,
    '20100030': vr_1.VR.CS,
    '20100040': vr_1.VR.CS,
    '20100050': vr_1.VR.CS,
    '20100052': vr_1.VR.CS,
    '20100054': vr_1.VR.CS,
    '20100060': vr_1.VR.CS,
    '20100080': vr_1.VR.CS,
    '201000a6': vr_1.VR.CS,
    '201000a7': vr_1.VR.CS,
    '201000a8': vr_1.VR.CS,
    '201000a9': vr_1.VR.CS,
    '20100100': vr_1.VR.CS,
    '20100110': vr_1.VR.CS,
    '20100120': vr_1.VR.US,
    '20100130': vr_1.VR.US,
    '20100140': vr_1.VR.CS,
    '20100150': vr_1.VR.ST,
    '20100152': vr_1.VR.LT,
    '20100154': vr_1.VR.IS,
    '2010015e': vr_1.VR.US,
    '20100160': vr_1.VR.US,
    '20100376': vr_1.VR.DS,
    '20100500': vr_1.VR.SQ,
    '20100510': vr_1.VR.SQ,
    '20100520': vr_1.VR.SQ,
    '20200010': vr_1.VR.US,
    '20200020': vr_1.VR.CS,
    '20200030': vr_1.VR.DS,
    '20200040': vr_1.VR.CS,
    '20200050': vr_1.VR.CS,
    '202000a0': vr_1.VR.CS,
    '202000a2': vr_1.VR.CS,
    '20200110': vr_1.VR.SQ,
    '20200111': vr_1.VR.SQ,
    '20200130': vr_1.VR.SQ,
    '20200140': vr_1.VR.SQ,
    '20300010': vr_1.VR.US,
    '20300020': vr_1.VR.LO,
    '20400010': vr_1.VR.SQ,
    '20400011': vr_1.VR.US,
    '20400020': vr_1.VR.SQ,
    '20400060': vr_1.VR.CS,
    '20400070': vr_1.VR.CS,
    '20400072': vr_1.VR.CS,
    '20400074': vr_1.VR.US,
    '20400080': vr_1.VR.CS,
    '20400082': vr_1.VR.CS,
    '20400090': vr_1.VR.CS,
    '20400100': vr_1.VR.CS,
    '20400500': vr_1.VR.SQ,
    '20500010': vr_1.VR.SQ,
    '20500020': vr_1.VR.CS,
    '20500500': vr_1.VR.SQ,
    '21000010': vr_1.VR.SH,
    '21000020': vr_1.VR.CS,
    '21000030': vr_1.VR.CS,
    '21000040': vr_1.VR.DA,
    '21000050': vr_1.VR.TM,
    '21000070': vr_1.VR.AE,
    '21000140': vr_1.VR.AE,
    '21000160': vr_1.VR.SH,
    '21000170': vr_1.VR.IS,
    '21000500': vr_1.VR.SQ,
    '21100010': vr_1.VR.CS,
    '21100020': vr_1.VR.CS,
    '21100030': vr_1.VR.LO,
    '21100099': vr_1.VR.SH,
    '21200010': vr_1.VR.CS,
    '21200050': vr_1.VR.SQ,
    '21200070': vr_1.VR.SQ,
    '21300010': vr_1.VR.SQ,
    '21300015': vr_1.VR.SQ,
    '21300030': vr_1.VR.SQ,
    '21300040': vr_1.VR.SQ,
    '21300050': vr_1.VR.SQ,
    '21300060': vr_1.VR.SQ,
    '21300080': vr_1.VR.SQ,
    '213000a0': vr_1.VR.SQ,
    '213000c0': vr_1.VR.SQ,
    '22000001': vr_1.VR.CS,
    '22000002': vr_1.VR.UT,
    '22000003': vr_1.VR.CS,
    '22000004': vr_1.VR.LT,
    '22000005': vr_1.VR.LT,
    '22000006': vr_1.VR.CS,
    '22000007': vr_1.VR.CS,
    '22000008': vr_1.VR.CS,
    '22000009': vr_1.VR.CS,
    '2200000a': vr_1.VR.CS,
    '2200000b': vr_1.VR.US,
    '2200000c': vr_1.VR.LO,
    '2200000d': vr_1.VR.SQ,
    '2200000e': vr_1.VR.AT,
    '2200000f': vr_1.VR.CS,
    '22000020': vr_1.VR.CS,
    '30020002': vr_1.VR.SH,
    '30020003': vr_1.VR.LO,
    '30020004': vr_1.VR.ST,
    '3002000a': vr_1.VR.CS,
    '3002000c': vr_1.VR.CS,
    '3002000d': vr_1.VR.DS,
    '3002000e': vr_1.VR.DS,
    '30020010': vr_1.VR.DS,
    '30020011': vr_1.VR.DS,
    '30020012': vr_1.VR.DS,
    '30020020': vr_1.VR.SH,
    '30020022': vr_1.VR.DS,
    '30020024': vr_1.VR.DS,
    '30020026': vr_1.VR.DS,
    '30020028': vr_1.VR.DS,
    '30020029': vr_1.VR.IS,
    '30020030': vr_1.VR.SQ,
    '30020032': vr_1.VR.DS,
    '30020034': vr_1.VR.DS,
    '30020040': vr_1.VR.SQ,
    '30020041': vr_1.VR.CS,
    '30020042': vr_1.VR.DS,
    '30020050': vr_1.VR.SQ,
    '30020051': vr_1.VR.CS,
    '30020052': vr_1.VR.SH,
    '30040001': vr_1.VR.CS,
    '30040002': vr_1.VR.CS,
    '30040004': vr_1.VR.CS,
    '30040005': vr_1.VR.CS,
    '30040006': vr_1.VR.LO,
    '30040008': vr_1.VR.DS,
    '3004000a': vr_1.VR.CS,
    '3004000c': vr_1.VR.DS,
    '3004000e': vr_1.VR.DS,
    '30040010': vr_1.VR.SQ,
    '30040012': vr_1.VR.DS,
    '30040014': vr_1.VR.CS,
    '30040040': vr_1.VR.DS,
    '30040042': vr_1.VR.DS,
    '30040050': vr_1.VR.SQ,
    '30040052': vr_1.VR.DS,
    '30040054': vr_1.VR.CS,
    '30040056': vr_1.VR.IS,
    '30040058': vr_1.VR.DS,
    '30040060': vr_1.VR.SQ,
    '30040062': vr_1.VR.CS,
    '30040070': vr_1.VR.DS,
    '30040072': vr_1.VR.DS,
    '30040074': vr_1.VR.DS,
    '30060002': vr_1.VR.SH,
    '30060004': vr_1.VR.LO,
    '30060006': vr_1.VR.ST,
    '30060008': vr_1.VR.DA,
    '30060009': vr_1.VR.TM,
    '30060010': vr_1.VR.SQ,
    '30060012': vr_1.VR.SQ,
    '30060014': vr_1.VR.SQ,
    '30060016': vr_1.VR.SQ,
    '30060018': vr_1.VR.SQ,
    '30060020': vr_1.VR.SQ,
    '30060022': vr_1.VR.IS,
    '30060024': vr_1.VR.UI,
    '30060026': vr_1.VR.LO,
    '30060028': vr_1.VR.ST,
    '3006002a': vr_1.VR.IS,
    '3006002c': vr_1.VR.DS,
    '30060030': vr_1.VR.SQ,
    '30060033': vr_1.VR.CS,
    '30060036': vr_1.VR.CS,
    '30060038': vr_1.VR.LO,
    '30060039': vr_1.VR.SQ,
    '30060040': vr_1.VR.SQ,
    '30060042': vr_1.VR.CS,
    '30060044': vr_1.VR.DS,
    '30060045': vr_1.VR.DS,
    '30060046': vr_1.VR.IS,
    '30060048': vr_1.VR.IS,
    '30060049': vr_1.VR.IS,
    '30060050': vr_1.VR.DS,
    '30060080': vr_1.VR.SQ,
    '30060082': vr_1.VR.IS,
    '30060084': vr_1.VR.IS,
    '30060085': vr_1.VR.SH,
    '30060086': vr_1.VR.SQ,
    '30060088': vr_1.VR.ST,
    '300600a0': vr_1.VR.SQ,
    '300600a4': vr_1.VR.CS,
    '300600a6': vr_1.VR.PN,
    '300600b0': vr_1.VR.SQ,
    '300600b2': vr_1.VR.CS,
    '300600b4': vr_1.VR.DS,
    '300600b6': vr_1.VR.SQ,
    '300600b7': vr_1.VR.US,
    '300600b8': vr_1.VR.FL,
    '300600b9': vr_1.VR.SQ,
    '300600c0': vr_1.VR.SQ,
    '300600c2': vr_1.VR.UI,
    '300600c4': vr_1.VR.CS,
    '300600c6': vr_1.VR.DS,
    '300600c8': vr_1.VR.LO,
    '30080010': vr_1.VR.SQ,
    '30080012': vr_1.VR.ST,
    '30080014': vr_1.VR.CS,
    '30080016': vr_1.VR.DS,
    '30080020': vr_1.VR.SQ,
    '30080021': vr_1.VR.SQ,
    '30080022': vr_1.VR.IS,
    '30080024': vr_1.VR.DA,
    '30080025': vr_1.VR.TM,
    '3008002a': vr_1.VR.CS,
    '3008002b': vr_1.VR.SH,
    '3008002c': vr_1.VR.CS,
    '30080030': vr_1.VR.SQ,
    '30080032': vr_1.VR.DS,
    '30080033': vr_1.VR.DS,
    '30080036': vr_1.VR.DS,
    '30080037': vr_1.VR.DS,
    '3008003a': vr_1.VR.DS,
    '3008003b': vr_1.VR.DS,
    '30080040': vr_1.VR.SQ,
    '30080041': vr_1.VR.SQ,
    '30080042': vr_1.VR.DS,
    '30080044': vr_1.VR.DS,
    '30080045': vr_1.VR.FL,
    '30080046': vr_1.VR.FL,
    '30080047': vr_1.VR.FL,
    '30080048': vr_1.VR.DS,
    '30080050': vr_1.VR.SQ,
    '30080052': vr_1.VR.DS,
    '30080054': vr_1.VR.DA,
    '30080056': vr_1.VR.DA,
    '3008005a': vr_1.VR.IS,
    '30080060': vr_1.VR.SQ,
    '30080061': vr_1.VR.AT,
    '30080062': vr_1.VR.AT,
    '30080063': vr_1.VR.IS,
    '30080064': vr_1.VR.IS,
    '30080065': vr_1.VR.AT,
    '30080066': vr_1.VR.ST,
    '30080067': vr_1.VR.US,
    '30080068': vr_1.VR.SQ,
    '3008006a': vr_1.VR.FL,
    '30080070': vr_1.VR.SQ,
    '30080072': vr_1.VR.IS,
    '30080074': vr_1.VR.ST,
    '30080076': vr_1.VR.DS,
    '30080078': vr_1.VR.DS,
    '3008007a': vr_1.VR.DS,
    '30080080': vr_1.VR.SQ,
    '30080082': vr_1.VR.IS,
    '30080090': vr_1.VR.SQ,
    '30080092': vr_1.VR.IS,
    '300800a0': vr_1.VR.SQ,
    '300800b0': vr_1.VR.SQ,
    '300800c0': vr_1.VR.SQ,
    '300800d0': vr_1.VR.SQ,
    '300800e0': vr_1.VR.SQ,
    '300800f0': vr_1.VR.SQ,
    '300800f2': vr_1.VR.SQ,
    '300800f4': vr_1.VR.SQ,
    '300800f6': vr_1.VR.SQ,
    '30080100': vr_1.VR.SQ,
    '30080105': vr_1.VR.LO,
    '30080110': vr_1.VR.SQ,
    '30080116': vr_1.VR.CS,
    '30080120': vr_1.VR.SQ,
    '30080122': vr_1.VR.IS,
    '30080130': vr_1.VR.SQ,
    '30080132': vr_1.VR.DS,
    '30080134': vr_1.VR.DS,
    '30080136': vr_1.VR.IS,
    '30080138': vr_1.VR.IS,
    '3008013a': vr_1.VR.DS,
    '3008013c': vr_1.VR.DS,
    '30080140': vr_1.VR.SQ,
    '30080142': vr_1.VR.IS,
    '30080150': vr_1.VR.SQ,
    '30080152': vr_1.VR.IS,
    '30080160': vr_1.VR.SQ,
    '30080162': vr_1.VR.DA,
    '30080164': vr_1.VR.TM,
    '30080166': vr_1.VR.DA,
    '30080168': vr_1.VR.TM,
    '30080171': vr_1.VR.SQ,
    '30080172': vr_1.VR.US,
    '30080173': vr_1.VR.SQ,
    '30080200': vr_1.VR.CS,
    '30080202': vr_1.VR.ST,
    '30080220': vr_1.VR.SQ,
    '30080223': vr_1.VR.IS,
    '30080224': vr_1.VR.CS,
    '30080230': vr_1.VR.CS,
    '30080240': vr_1.VR.SQ,
    '30080250': vr_1.VR.DA,
    '30080251': vr_1.VR.TM,
    '300a0002': vr_1.VR.SH,
    '300a0003': vr_1.VR.LO,
    '300a0004': vr_1.VR.ST,
    '300a0006': vr_1.VR.DA,
    '300a0007': vr_1.VR.TM,
    '300a0009': vr_1.VR.LO,
    '300a000a': vr_1.VR.CS,
    '300a000b': vr_1.VR.LO,
    '300a000c': vr_1.VR.CS,
    '300a000e': vr_1.VR.ST,
    '300a0010': vr_1.VR.SQ,
    '300a0012': vr_1.VR.IS,
    '300a0013': vr_1.VR.UI,
    '300a0014': vr_1.VR.CS,
    '300a0015': vr_1.VR.CS,
    '300a0016': vr_1.VR.LO,
    '300a0018': vr_1.VR.DS,
    '300a001a': vr_1.VR.DS,
    '300a0020': vr_1.VR.CS,
    '300a0021': vr_1.VR.DS,
    '300a0022': vr_1.VR.DS,
    '300a0023': vr_1.VR.DS,
    '300a0025': vr_1.VR.DS,
    '300a0026': vr_1.VR.DS,
    '300a0027': vr_1.VR.DS,
    '300a0028': vr_1.VR.DS,
    '300a002a': vr_1.VR.DS,
    '300a002b': vr_1.VR.DS,
    '300a002c': vr_1.VR.DS,
    '300a002d': vr_1.VR.DS,
    '300a0040': vr_1.VR.SQ,
    '300a0042': vr_1.VR.IS,
    '300a0043': vr_1.VR.SH,
    '300a0044': vr_1.VR.DS,
    '300a0046': vr_1.VR.DS,
    '300a0048': vr_1.VR.SQ,
    '300a004a': vr_1.VR.DS,
    '300a004b': vr_1.VR.FL,
    '300a004c': vr_1.VR.DS,
    '300a004e': vr_1.VR.DS,
    '300a004f': vr_1.VR.FL,
    '300a0050': vr_1.VR.FL,
    '300a0051': vr_1.VR.DS,
    '300a0052': vr_1.VR.DS,
    '300a0053': vr_1.VR.DS,
    '300a0055': vr_1.VR.CS,
    '300a0070': vr_1.VR.SQ,
    '300a0071': vr_1.VR.IS,
    '300a0072': vr_1.VR.LO,
    '300a0078': vr_1.VR.IS,
    '300a0079': vr_1.VR.IS,
    '300a007a': vr_1.VR.IS,
    '300a007b': vr_1.VR.LT,
    '300a0080': vr_1.VR.IS,
    '300a0082': vr_1.VR.DS,
    '300a0083': vr_1.VR.UI,
    '300a0084': vr_1.VR.DS,
    '300a0086': vr_1.VR.DS,
    '300a0088': vr_1.VR.FL,
    '300a0089': vr_1.VR.FL,
    '300a008a': vr_1.VR.FL,
    '300a008b': vr_1.VR.CS,
    '300a008c': vr_1.VR.SQ,
    '300a008d': vr_1.VR.FL,
    '300a008e': vr_1.VR.FL,
    '300a008f': vr_1.VR.FL,
    '300a0090': vr_1.VR.CS,
    '300a0091': vr_1.VR.DS,
    '300a0092': vr_1.VR.CS,
    '300a0093': vr_1.VR.CS,
    '300a00a0': vr_1.VR.IS,
    '300a00a2': vr_1.VR.DS,
    '300a00a4': vr_1.VR.DS,
    '300a00b0': vr_1.VR.SQ,
    '300a00b2': vr_1.VR.SH,
    '300a00b3': vr_1.VR.CS,
    '300a00b4': vr_1.VR.DS,
    '300a00b6': vr_1.VR.SQ,
    '300a00b8': vr_1.VR.CS,
    '300a00ba': vr_1.VR.DS,
    '300a00bb': vr_1.VR.FL,
    '300a00bc': vr_1.VR.IS,
    '300a00be': vr_1.VR.DS,
    '300a00c0': vr_1.VR.IS,
    '300a00c2': vr_1.VR.LO,
    '300a00c3': vr_1.VR.ST,
    '300a00c4': vr_1.VR.CS,
    '300a00c5': vr_1.VR.FD,
    '300a00c6': vr_1.VR.CS,
    '300a00c7': vr_1.VR.CS,
    '300a00c8': vr_1.VR.IS,
    '300a00ca': vr_1.VR.SQ,
    '300a00cc': vr_1.VR.LO,
    '300a00ce': vr_1.VR.CS,
    '300a00d0': vr_1.VR.IS,
    '300a00d1': vr_1.VR.SQ,
    '300a00d2': vr_1.VR.IS,
    '300a00d3': vr_1.VR.CS,
    '300a00d4': vr_1.VR.SH,
    '300a00d5': vr_1.VR.IS,
    '300a00d6': vr_1.VR.DS,
    '300a00d7': vr_1.VR.FL,
    '300a00d8': vr_1.VR.DS,
    '300a00d9': vr_1.VR.FL,
    '300a00da': vr_1.VR.DS,
    '300a00db': vr_1.VR.FL,
    '300a00dc': vr_1.VR.SH,
    '300a00dd': vr_1.VR.ST,
    '300a00de': vr_1.VR.DS,
    '300a00e0': vr_1.VR.IS,
    '300a00e1': vr_1.VR.SH,
    '300a00e2': vr_1.VR.DS,
    '300a00e3': vr_1.VR.SQ,
    '300a00e4': vr_1.VR.IS,
    '300a00e5': vr_1.VR.SH,
    '300a00e6': vr_1.VR.DS,
    '300a00e7': vr_1.VR.IS,
    '300a00e8': vr_1.VR.IS,
    '300a00e9': vr_1.VR.DS,
    '300a00ea': vr_1.VR.DS,
    '300a00eb': vr_1.VR.DS,
    '300a00ec': vr_1.VR.DS,
    '300a00ed': vr_1.VR.IS,
    '300a00ee': vr_1.VR.CS,
    '300a00ef': vr_1.VR.SH,
    '300a00f0': vr_1.VR.IS,
    '300a00f2': vr_1.VR.DS,
    '300a00f3': vr_1.VR.FL,
    '300a00f4': vr_1.VR.SQ,
    '300a00f5': vr_1.VR.SH,
    '300a00f6': vr_1.VR.DS,
    '300a00f7': vr_1.VR.FL,
    '300a00f8': vr_1.VR.CS,
    '300a00f9': vr_1.VR.LO,
    '300a00fa': vr_1.VR.CS,
    '300a00fb': vr_1.VR.CS,
    '300a00fc': vr_1.VR.IS,
    '300a00fe': vr_1.VR.LO,
    '300a0100': vr_1.VR.DS,
    '300a0102': vr_1.VR.DS,
    '300a0104': vr_1.VR.IS,
    '300a0106': vr_1.VR.DS,
    '300a0107': vr_1.VR.SQ,
    '300a0108': vr_1.VR.SH,
    '300a0109': vr_1.VR.CS,
    '300a010a': vr_1.VR.LO,
    '300a010c': vr_1.VR.DS,
    '300a010e': vr_1.VR.DS,
    '300a0110': vr_1.VR.IS,
    '300a0111': vr_1.VR.SQ,
    '300a0112': vr_1.VR.IS,
    '300a0114': vr_1.VR.DS,
    '300a0115': vr_1.VR.DS,
    '300a0116': vr_1.VR.SQ,
    '300a0118': vr_1.VR.CS,
    '300a011a': vr_1.VR.SQ,
    '300a011c': vr_1.VR.DS,
    '300a011e': vr_1.VR.DS,
    '300a011f': vr_1.VR.CS,
    '300a0120': vr_1.VR.DS,
    '300a0121': vr_1.VR.CS,
    '300a0122': vr_1.VR.DS,
    '300a0123': vr_1.VR.CS,
    '300a0124': vr_1.VR.DS,
    '300a0125': vr_1.VR.DS,
    '300a0126': vr_1.VR.CS,
    '300a0128': vr_1.VR.DS,
    '300a0129': vr_1.VR.DS,
    '300a012a': vr_1.VR.DS,
    '300a012c': vr_1.VR.DS,
    '300a012e': vr_1.VR.DS,
    '300a0130': vr_1.VR.DS,
    '300a0131': vr_1.VR.FL,
    '300a0132': vr_1.VR.FL,
    '300a0133': vr_1.VR.FL,
    '300a0134': vr_1.VR.DS,
    '300a0140': vr_1.VR.FL,
    '300a0142': vr_1.VR.CS,
    '300a0144': vr_1.VR.FL,
    '300a0146': vr_1.VR.CS,
    '300a0148': vr_1.VR.FL,
    '300a014a': vr_1.VR.FL,
    '300a014c': vr_1.VR.CS,
    '300a014e': vr_1.VR.FL,
    '300a0150': vr_1.VR.CS,
    '300a0151': vr_1.VR.DS,
    '300a0152': vr_1.VR.DS,
    '300a0153': vr_1.VR.DS,
    '300a0154': vr_1.VR.DS,
    '300a0155': vr_1.VR.DS,
    '300a0180': vr_1.VR.SQ,
    '300a0182': vr_1.VR.IS,
    '300a0183': vr_1.VR.LO,
    '300a0184': vr_1.VR.LO,
    '300a0190': vr_1.VR.SQ,
    '300a0192': vr_1.VR.CS,
    '300a0194': vr_1.VR.SH,
    '300a0196': vr_1.VR.ST,
    '300a0198': vr_1.VR.SH,
    '300a0199': vr_1.VR.FL,
    '300a019a': vr_1.VR.FL,
    '300a01a0': vr_1.VR.SQ,
    '300a01a2': vr_1.VR.CS,
    '300a01a4': vr_1.VR.SH,
    '300a01a6': vr_1.VR.ST,
    '300a01a8': vr_1.VR.SH,
    '300a01b0': vr_1.VR.CS,
    '300a01b2': vr_1.VR.ST,
    '300a01b4': vr_1.VR.SQ,
    '300a01b6': vr_1.VR.CS,
    '300a01b8': vr_1.VR.SH,
    '300a01ba': vr_1.VR.ST,
    '300a01bc': vr_1.VR.DS,
    '300a01d0': vr_1.VR.ST,
    '300a01d2': vr_1.VR.DS,
    '300a01d4': vr_1.VR.DS,
    '300a01d6': vr_1.VR.DS,
    '300a0200': vr_1.VR.CS,
    '300a0202': vr_1.VR.CS,
    '300a0206': vr_1.VR.SQ,
    '300a0210': vr_1.VR.SQ,
    '300a0212': vr_1.VR.IS,
    '300a0214': vr_1.VR.CS,
    '300a0216': vr_1.VR.LO,
    '300a0218': vr_1.VR.DS,
    '300a021a': vr_1.VR.DS,
    '300a021b': vr_1.VR.SH,
    '300a021c': vr_1.VR.LO,
    '300a0222': vr_1.VR.DS,
    '300a0224': vr_1.VR.DS,
    '300a0226': vr_1.VR.LO,
    '300a0228': vr_1.VR.DS,
    '300a0229': vr_1.VR.CS,
    '300a022a': vr_1.VR.DS,
    '300a022b': vr_1.VR.DS,
    '300a022c': vr_1.VR.DA,
    '300a022e': vr_1.VR.TM,
    '300a0230': vr_1.VR.SQ,
    '300a0232': vr_1.VR.CS,
    '300a0234': vr_1.VR.IS,
    '300a0236': vr_1.VR.LO,
    '300a0238': vr_1.VR.LO,
    '300a0240': vr_1.VR.IS,
    '300a0242': vr_1.VR.SH,
    '300a0244': vr_1.VR.LO,
    '300a0250': vr_1.VR.DS,
    '300a0260': vr_1.VR.SQ,
    '300a0262': vr_1.VR.IS,
    '300a0263': vr_1.VR.SH,
    '300a0264': vr_1.VR.CS,
    '300a0266': vr_1.VR.LO,
    '300a026a': vr_1.VR.DS,
    '300a026c': vr_1.VR.DS,
    '300a0271': vr_1.VR.DS,
    '300a0272': vr_1.VR.DS,
    '300a0273': vr_1.VR.SH,
    '300a0274': vr_1.VR.DS,
    '300a0280': vr_1.VR.SQ,
    '300a0282': vr_1.VR.IS,
    '300a0284': vr_1.VR.DS,
    '300a0286': vr_1.VR.DS,
    '300a0288': vr_1.VR.CS,
    '300a028a': vr_1.VR.IS,
    '300a028c': vr_1.VR.DS,
    '300a0290': vr_1.VR.IS,
    '300a0291': vr_1.VR.SH,
    '300a0292': vr_1.VR.CS,
    '300a0294': vr_1.VR.LO,
    '300a0296': vr_1.VR.DS,
    '300a0298': vr_1.VR.LO,
    '300a029c': vr_1.VR.DS,
    '300a029e': vr_1.VR.DS,
    '300a02a0': vr_1.VR.DS,
    '300a02a2': vr_1.VR.IS,
    '300a02a4': vr_1.VR.DS,
    '300a02b0': vr_1.VR.SQ,
    '300a02b2': vr_1.VR.IS,
    '300a02b3': vr_1.VR.SH,
    '300a02b4': vr_1.VR.LO,
    '300a02b8': vr_1.VR.DS,
    '300a02ba': vr_1.VR.DS,
    '300a02c8': vr_1.VR.DS,
    '300a02d0': vr_1.VR.SQ,
    '300a02d2': vr_1.VR.DS,
    '300a02d4': vr_1.VR.DS,
    '300a02d6': vr_1.VR.DS,
    '300a02e0': vr_1.VR.CS,
    '300a02e1': vr_1.VR.CS,
    '300a02e2': vr_1.VR.DS,
    '300a02e3': vr_1.VR.FL,
    '300a02e4': vr_1.VR.FL,
    '300a02e5': vr_1.VR.FL,
    '300a02e6': vr_1.VR.FL,
    '300a02e7': vr_1.VR.FL,
    '300a02e8': vr_1.VR.FL,
    '300a02ea': vr_1.VR.SQ,
    '300a02eb': vr_1.VR.LT,
    '300a0302': vr_1.VR.IS,
    '300a0304': vr_1.VR.IS,
    '300a0306': vr_1.VR.SS,
    '300a0308': vr_1.VR.CS,
    '300a0309': vr_1.VR.CS,
    '300a030a': vr_1.VR.FL,
    '300a030c': vr_1.VR.SQ,
    '300a030d': vr_1.VR.FL,
    '300a030f': vr_1.VR.SH,
    '300a0312': vr_1.VR.IS,
    '300a0314': vr_1.VR.SQ,
    '300a0316': vr_1.VR.IS,
    '300a0318': vr_1.VR.SH,
    '300a0320': vr_1.VR.CS,
    '300a0322': vr_1.VR.LO,
    '300a0330': vr_1.VR.IS,
    '300a0332': vr_1.VR.SQ,
    '300a0334': vr_1.VR.IS,
    '300a0336': vr_1.VR.SH,
    '300a0338': vr_1.VR.CS,
    '300a033a': vr_1.VR.LO,
    '300a033c': vr_1.VR.FL,
    '300a0340': vr_1.VR.IS,
    '300a0342': vr_1.VR.SQ,
    '300a0344': vr_1.VR.IS,
    '300a0346': vr_1.VR.SH,
    '300a0348': vr_1.VR.CS,
    '300a034a': vr_1.VR.LO,
    '300a034c': vr_1.VR.SH,
    '300a0350': vr_1.VR.CS,
    '300a0352': vr_1.VR.SH,
    '300a0354': vr_1.VR.LO,
    '300a0355': vr_1.VR.LO,
    '300a0356': vr_1.VR.FL,
    '300a0358': vr_1.VR.FL,
    '300a035a': vr_1.VR.FL,
    '300a0360': vr_1.VR.SQ,
    '300a0362': vr_1.VR.LO,
    '300a0364': vr_1.VR.FL,
    '300a0366': vr_1.VR.FL,
    '300a0370': vr_1.VR.SQ,
    '300a0372': vr_1.VR.LO,
    '300a0374': vr_1.VR.FL,
    '300a0380': vr_1.VR.SQ,
    '300a0382': vr_1.VR.FL,
    '300a0384': vr_1.VR.FL,
    '300a0386': vr_1.VR.FL,
    '300a0388': vr_1.VR.FL,
    '300a038a': vr_1.VR.FL,
    '300a038f': vr_1.VR.FL,
    '300a0390': vr_1.VR.SH,
    '300a0391': vr_1.VR.IS,
    '300a0392': vr_1.VR.IS,
    '300a0393': vr_1.VR.CS,
    '300a0394': vr_1.VR.FL,
    '300a0395': vr_1.VR.CS,
    '300a0396': vr_1.VR.FL,
    '300a0398': vr_1.VR.FL,
    '300a039a': vr_1.VR.IS,
    '300a03a0': vr_1.VR.SQ,
    '300a03a2': vr_1.VR.SQ,
    '300a03a4': vr_1.VR.SQ,
    '300a03a6': vr_1.VR.SQ,
    '300a03a8': vr_1.VR.SQ,
    '300a03aa': vr_1.VR.SQ,
    '300a03ac': vr_1.VR.SQ,
    '300a0401': vr_1.VR.SQ,
    '300a0402': vr_1.VR.ST,
    '300a0410': vr_1.VR.SQ,
    '300a0412': vr_1.VR.FL,
    '300a0420': vr_1.VR.SQ,
    '300a0421': vr_1.VR.SH,
    '300a0422': vr_1.VR.ST,
    '300a0423': vr_1.VR.CS,
    '300a0424': vr_1.VR.IS,
    '300a0425': vr_1.VR.FL,
    '300a0431': vr_1.VR.SQ,
    '300a0432': vr_1.VR.CS,
    '300a0433': vr_1.VR.FL,
    '300a0434': vr_1.VR.FL,
    '300a0435': vr_1.VR.FL,
    '300a0436': vr_1.VR.FL,
    '300a0440': vr_1.VR.IS,
    '300a0441': vr_1.VR.SQ,
    '300a0442': vr_1.VR.DS,
    '300a0443': vr_1.VR.US,
    '300a0450': vr_1.VR.SQ,
    '300a0451': vr_1.VR.CS,
    '300a0452': vr_1.VR.CS,
    '300a0453': vr_1.VR.SQ,
    '300a0501': vr_1.VR.FL,
    '300a0502': vr_1.VR.FL,
    '300a0503': vr_1.VR.FL,
    '300a0504': vr_1.VR.FL,
    '300a0505': vr_1.VR.SQ,
    '300a0506': vr_1.VR.SQ,
    '300a0507': vr_1.VR.FL,
    '300a0508': vr_1.VR.FL,
    '300a0509': vr_1.VR.FL,
    '300a0510': vr_1.VR.FL,
    '300a0511': vr_1.VR.CS,
    '300a0512': vr_1.VR.CS,
    '300c0002': vr_1.VR.SQ,
    '300c0004': vr_1.VR.SQ,
    '300c0006': vr_1.VR.IS,
    '300c0007': vr_1.VR.IS,
    '300c0008': vr_1.VR.DS,
    '300c0009': vr_1.VR.DS,
    '300c000a': vr_1.VR.SQ,
    '300c000c': vr_1.VR.IS,
    '300c000e': vr_1.VR.IS,
    '300c0020': vr_1.VR.SQ,
    '300c0022': vr_1.VR.IS,
    '300c0040': vr_1.VR.SQ,
    '300c0042': vr_1.VR.SQ,
    '300c0050': vr_1.VR.SQ,
    '300c0051': vr_1.VR.IS,
    '300c0055': vr_1.VR.SQ,
    '300c0060': vr_1.VR.SQ,
    '300c006a': vr_1.VR.IS,
    '300c0080': vr_1.VR.SQ,
    '300c00a0': vr_1.VR.IS,
    '300c00b0': vr_1.VR.SQ,
    '300c00c0': vr_1.VR.IS,
    '300c00d0': vr_1.VR.IS,
    '300c00e0': vr_1.VR.IS,
    '300c00f0': vr_1.VR.IS,
    '300c00f2': vr_1.VR.SQ,
    '300c00f4': vr_1.VR.IS,
    '300c00f6': vr_1.VR.IS,
    '300c0100': vr_1.VR.IS,
    '300c0102': vr_1.VR.IS,
    '300c0104': vr_1.VR.IS,
    '300c0111': vr_1.VR.SQ,
    '300c0112': vr_1.VR.CS,
    '300c0113': vr_1.VR.LO,
    '300e0002': vr_1.VR.CS,
    '300e0004': vr_1.VR.DA,
    '300e0005': vr_1.VR.TM,
    '300e0008': vr_1.VR.PN,
    '40000010': vr_1.VR.LT,
    '40004000': vr_1.VR.LT,
    '40080040': vr_1.VR.SH,
    '40080042': vr_1.VR.LO,
    '40080050': vr_1.VR.SQ,
    '400800ff': vr_1.VR.CS,
    '40080100': vr_1.VR.DA,
    '40080101': vr_1.VR.TM,
    '40080102': vr_1.VR.PN,
    '40080103': vr_1.VR.LO,
    '40080108': vr_1.VR.DA,
    '40080109': vr_1.VR.TM,
    '4008010a': vr_1.VR.PN,
    '4008010b': vr_1.VR.ST,
    '4008010c': vr_1.VR.PN,
    '40080111': vr_1.VR.SQ,
    '40080112': vr_1.VR.DA,
    '40080113': vr_1.VR.TM,
    '40080114': vr_1.VR.PN,
    '40080115': vr_1.VR.LT,
    '40080117': vr_1.VR.SQ,
    '40080118': vr_1.VR.SQ,
    '40080119': vr_1.VR.PN,
    '4008011a': vr_1.VR.LO,
    '40080200': vr_1.VR.SH,
    '40080202': vr_1.VR.LO,
    '40080210': vr_1.VR.CS,
    '40080212': vr_1.VR.CS,
    '40080300': vr_1.VR.ST,
    '40084000': vr_1.VR.ST,
    '40100001': vr_1.VR.CS,
    '40100002': vr_1.VR.CS,
    '40100004': vr_1.VR.SQ,
    '40101001': vr_1.VR.SQ,
    '40101004': vr_1.VR.FL,
    '40101005': vr_1.VR.FL,
    '40101006': vr_1.VR.OB,
    '40101007': vr_1.VR.SH,
    '40101008': vr_1.VR.CS,
    '40101009': vr_1.VR.CS,
    '4010100a': vr_1.VR.SQ,
    '40101010': vr_1.VR.US,
    '40101011': vr_1.VR.SQ,
    '40101012': vr_1.VR.CS,
    '40101013': vr_1.VR.LT,
    '40101014': vr_1.VR.CS,
    '40101015': vr_1.VR.CS,
    '40101016': vr_1.VR.FL,
    '40101017': vr_1.VR.FL,
    '40101018': vr_1.VR.FL,
    '40101019': vr_1.VR.FL,
    '4010101a': vr_1.VR.SH,
    '4010101b': vr_1.VR.FL,
    '4010101c': vr_1.VR.FL,
    '4010101d': vr_1.VR.FL,
    '4010101e': vr_1.VR.SH,
    '4010101f': vr_1.VR.SH,
    '40101020': vr_1.VR.CS,
    '40101021': vr_1.VR.CS,
    '40101023': vr_1.VR.FL,
    '40101024': vr_1.VR.CS,
    '40101025': vr_1.VR.DT,
    '40101026': vr_1.VR.DT,
    '40101027': vr_1.VR.CS,
    '40101028': vr_1.VR.CS,
    '40101029': vr_1.VR.LO,
    '4010102a': vr_1.VR.SH,
    '4010102b': vr_1.VR.DT,
    '40101031': vr_1.VR.CS,
    '40101033': vr_1.VR.US,
    '40101034': vr_1.VR.US,
    '40101037': vr_1.VR.SQ,
    '40101038': vr_1.VR.SQ,
    '40101039': vr_1.VR.CS,
    '4010103a': vr_1.VR.CS,
    '40101041': vr_1.VR.DT,
    '40101042': vr_1.VR.CS,
    '40101043': vr_1.VR.FL,
    '40101044': vr_1.VR.CS,
    '40101045': vr_1.VR.SQ,
    '40101046': vr_1.VR.CS,
    '40101047': vr_1.VR.SQ,
    '40101048': vr_1.VR.CS,
    '40101051': vr_1.VR.LO,
    '40101052': vr_1.VR.SH,
    '40101053': vr_1.VR.LO,
    '40101054': vr_1.VR.SH,
    '40101055': vr_1.VR.SH,
    '40101056': vr_1.VR.CS,
    '40101058': vr_1.VR.SH,
    '40101059': vr_1.VR.CS,
    '40101060': vr_1.VR.FL,
    '40101061': vr_1.VR.FL,
    '40101062': vr_1.VR.FL,
    '40101064': vr_1.VR.SQ,
    '40101067': vr_1.VR.CS,
    '40101068': vr_1.VR.LT,
    '40101069': vr_1.VR.FL,
    '4010106c': vr_1.VR.OB,
    '4010106d': vr_1.VR.CS,
    '4010106e': vr_1.VR.CS,
    '4010106f': vr_1.VR.SQ,
    '40101070': vr_1.VR.CS,
    '40101071': vr_1.VR.SQ,
    '40101072': vr_1.VR.SQ,
    '40101073': vr_1.VR.FD,
    '40101075': vr_1.VR.DS,
    '40101076': vr_1.VR.SQ,
    '40101077': vr_1.VR.SQ,
    '40101078': vr_1.VR.ST,
    '40101079': vr_1.VR.SQ,
    '4010107a': vr_1.VR.FL,
    '4010107b': vr_1.VR.SQ,
    '4010107c': vr_1.VR.CS,
    '4010107d': vr_1.VR.SQ,
    '4010107e': vr_1.VR.DS,
    '4ffe0001': vr_1.VR.SQ,
    '50000005': vr_1.VR.US,
    '50000010': vr_1.VR.US,
    '50000020': vr_1.VR.CS,
    '50000022': vr_1.VR.LO,
    '50000030': vr_1.VR.SH,
    '50000040': vr_1.VR.SH,
    '50000103': vr_1.VR.US,
    '50000104': vr_1.VR.US,
    '50000105': vr_1.VR.US,
    '50000106': vr_1.VR.SH,
    '50000110': vr_1.VR.US,
    '50000112': vr_1.VR.US,
    '50000114': vr_1.VR.US,
    '50001001': vr_1.VR.CS,
    '50002000': vr_1.VR.US,
    '50002002': vr_1.VR.US,
    '50002004': vr_1.VR.US,
    '50002006': vr_1.VR.UL,
    '50002008': vr_1.VR.UL,
    '5000200a': vr_1.VR.UL,
    '5000200c': vr_1.VR.OW,
    '5000200e': vr_1.VR.LT,
    '50002500': vr_1.VR.LO,
    '50002600': vr_1.VR.SQ,
    '50002610': vr_1.VR.US,
    '50003000': vr_1.VR.OW,
    '52009229': vr_1.VR.SQ,
    '52009230': vr_1.VR.SQ,
    '54000100': vr_1.VR.SQ,
    '54000110': vr_1.VR.OW,
    '54000112': vr_1.VR.OW,
    '54001004': vr_1.VR.US,
    '54001006': vr_1.VR.CS,
    '5400100a': vr_1.VR.OW,
    '54001010': vr_1.VR.OW,
    '56000010': vr_1.VR.OF,
    '56000020': vr_1.VR.OF,
    '60000010': vr_1.VR.US,
    '60000011': vr_1.VR.US,
    '60000012': vr_1.VR.US,
    '60000015': vr_1.VR.IS,
    '60000022': vr_1.VR.LO,
    '60000040': vr_1.VR.CS,
    '60000045': vr_1.VR.LO,
    '60000050': vr_1.VR.SS,
    '60000051': vr_1.VR.US,
    '60000052': vr_1.VR.US,
    '60000060': vr_1.VR.CS,
    '60000061': vr_1.VR.SH,
    '60000062': vr_1.VR.SH,
    '60000063': vr_1.VR.CS,
    '60000066': vr_1.VR.AT,
    '60000068': vr_1.VR.US,
    '60000069': vr_1.VR.US,
    '60000100': vr_1.VR.US,
    '60000102': vr_1.VR.US,
    '60000110': vr_1.VR.CS,
    '60000200': vr_1.VR.US,
    '60000800': vr_1.VR.CS,
    '60000802': vr_1.VR.US,
    '60000803': vr_1.VR.AT,
    '60000804': vr_1.VR.US,
    '60001001': vr_1.VR.CS,
    '60001100': vr_1.VR.US,
    '60001101': vr_1.VR.US,
    '60001102': vr_1.VR.US,
    '60001103': vr_1.VR.US,
    '60001200': vr_1.VR.US,
    '60001201': vr_1.VR.US,
    '60001202': vr_1.VR.US,
    '60001203': vr_1.VR.US,
    '60001301': vr_1.VR.IS,
    '60001302': vr_1.VR.DS,
    '60001303': vr_1.VR.DS,
    '60001500': vr_1.VR.LO,
    '60003000': vr_1.VR.OW,
    '60004000': vr_1.VR.LT,
    '7f000010': vr_1.VR.OW,
    '7f000011': vr_1.VR.US,
    '7f000020': vr_1.VR.OW,
    '7f000030': vr_1.VR.OW,
    '7f000040': vr_1.VR.OW,
    '7fe00008': vr_1.VR.OF,
    '7fe00009': vr_1.VR.OD,
    '7fe00010': vr_1.VR.OW,
    '7fe00020': vr_1.VR.OW,
    '7fe00030': vr_1.VR.OW,
    '7fe00040': vr_1.VR.OW,
    fffafffa: vr_1.VR.SQ,
    fffcfffc: vr_1.VR.OB,
};
exports.TagToVR = TagToVR;
//# sourceMappingURL=tag-to-vr.js.map